/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TSampleTQryTS.cpp
 * Description    : connect to Text Search.
 *                  create query string
 *                  create a text query object  and submit query.
 *                  get the results in a collection.
 *                  display items with Text Search Information.
 *                  disconnect from Text Search
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKTextQuery.hpp>
#include <dkCollection.hpp>
#include <dkXDO.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //--------------------------------------------------------------   
      if (argc < 3)
      {
       cout<<"Invoke Syntax: "<<endl;
       cout<<"  SampleTQryTS <srchSrv> <srchIndex>" << endl;
       return;
      }
      // ### get input parameters
      DKString srchSrv = DKString(argv[1]);
      DKString srchIndex = DKString(argv[2]);
      DKDatastoreTS dsTS; // must be outside try/catch block
      try {
         dkQuery* pQry;
         DKAny any;
         DKResults* pResults;

         cout << "connecting to datastore" << endl;
         dsTS.connect(srchSrv,"","");
         cout << "connected to datastore srchSrv: " << srchSrv << endl;

         DKString cmd = "SEARCH=";
         cmd += "(COND=(UNIX OR member));";
         cmd += "OPTION=(SEARCH_INDEX=";
		 cmd += srchIndex;
		 cmd += ")";
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsTS.createQuery(cmd);
         cout << "executing query" << endl;
         pQry->execute();
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result(); 
         pResults = (DKResults*)((dkCollection*) any);
         
         processResults(pResults);

         dsTS.disconnect();
         cout << "datastore disconnected" << endl;
         delete pQry;
         delete pResults;
    }
    catch(DKException &exc)
    {
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i < exc.textCount(); i++) {
             cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long j=0; j < exc.locationCount(); j++) {
              const DKExceptionLocation* p = exc.locationAtIndex(j);
              cout << "Filename: "   << p->fileName()     << endl;
              cout << "Function: "   << p->functionName() << endl;
              cout << "LineNumber: " << p->lineNumber()   << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;
}

void processResults(dkCollection* results)
{
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;
   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more())
   {
      element = iter->next();
      item = (DKDDO*) element->value();
      cout << "----------------------------------------------- " << endl;      
      for (i = 1; i <= item->dataCount(); i++)
      {
        dataName = item->getDataName(i);
        cout << "    data-item[" << i << "] = "
             << dataName  << ", \t\t value "
             << item->getData(i) << endl;
      }     
   }
   delete iter;
}


