/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating an Attribute Definition
    Attribute Definitions exist in two forms, the general Attribute Definition and the
    tailored Attribute Definition for a specific Component Type.  Modifications of the
    general Attribute Definition will not affect those already added to Item or Component
    Types.  This prevents breaking, destroying, or corrupting existing Data Models and 
    existing Items of those Data Models.
    
    As explained in both SAttributeDefinitionCreationICM & SItemTypeCreationICM, Attributes
    may be tailored for a particular situation applicable to the specific Item/Component
    Type that it belongs to.
    
    Datastore Definition's Attributes
        To modify the general characteristics of an Attribute Definition, such as name or
        description, and will be applicable only to new Item or Component Types created
        using this Attribute, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To modify any property of an Attribute Definition already added to an Item or 
        Component Type, the definition object must be retrieved from the Component Type
        Definition.  Modifications will apply only to this particular Component Type.
        
        Hint:  Use DKComponentTypeDef.retrieveAttr().  Please refer to SItemTypeRetrievalICM
               for more information on how to retrieve an existing Component Type Definition.

This Sample will update the Datastore Definition's Attributes.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeDefinitionRetrievalICM.hpp"
#include "SAttributeDefinitionUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionUpdateICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Saving Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Attribute Definitions in Memory
 *                    Saving / Updating Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeDefinitionUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes must already be defined.  If not, please run Sample
 *                    SAttributeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeDefinitionRetrievalICM.hpp/cpp
 *                    SAttributeDefinitionUpdateICM.hpp
 *                    SAttributeDefinitionUpdateICM_launch.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeDefinitionUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SAttributeDefinitionUpdateICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "=============================================="     << endl;
        cout << "IBM Information Integrator for Content    v" << ver << endl;
        cout << "Sample Program:  SAttributeDefinitionUpdateICM"     << endl;
        cout << "----------------------------------------------"     << endl;
        cout << " Database: " << database                            << endl;
        cout << " UserName: " << userName                            << endl;
        cout << "==============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Retrieve & Print the S_varchar Attribute Definition
            //----------------------------------------------------------------
            // Retrieving the Datastore Definition's Attribute Definition.  Please
            // refer to the header documentation for an explanation of the difference
            // between that defined within the Datastore Definition compared to the 
            // Component Type Definition.
            //
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            cout << "Retrieving the Attribute Definition of 'S_varchar'..." << endl;

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();     // Get the datastore definition object from the connected datastore.
                DKAttrDefICM* attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_varchar"); // Retrieve the definition.
                SAttributeDefinitionRetrievalICM::printAttrDef(attrDef);    // Use wrapper funciton from this Sample.
                
            cout << "Retrieved the Attribute Definition of 'S_varchar'." << endl;

            //----------------------------------------------------------------
            // Modify the S_varchar Attribute Definition in Memory
            //----------------------------------------------------------------
            // Change any of the characteristics of an attribute definition.  This example
            // will change just one characteristic, the description.
            //
            // Updating the Datastore Definition's Attribute Definition.  Please
            // refer to the header documentation for an explanation of the difference
            // between that defined in the Datastore Definition and the Component Type
            // Definition.
            cout << "Modify the Attribute Definition 'S_varchar' in Memory..." << endl;
            
                attrDef->setDescription("My modified varchar attribute description.");

            cout << "Modified the Attribute Definition of 'S_varchar' in Memory." << endl;

            //----------------------------------------------------------------
            // Update the S_varchar Attribute Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The attribute definition can be updated through its own update() method
            // or by using the Datastore Definition's update() method, passing the attribute
            // definition as the parameter.
            cout << "Updating the Attribute Definition 'S_varchar' in Datastore Definition..." << endl;

                attrDef->update(); // save the attribute definition changes.

            cout << "Modified the Attribute Definition of 'S_varchar' in Datastore Definition."<< endl;

            //----------------------------------------------------------------
            // Print the Attribute Definition with changes.
            //----------------------------------------------------------------
            cout << "Printing Attribute Definition..." << endl;

                SAttributeDefinitionRetrievalICM::printAttrDef(attrDef); // Sample's Wrapper            
                delete(attrDef);   // Free Memory
            
            cout << "Printed Attribute Definition." << endl;

            //----------------------------------------------------------------
            // Retrieve & Print the S_integer Attribute Definition
            //----------------------------------------------------------------
            cout << "Retrieving the Attribute Definition of 'S_integer'..." << endl;

                attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_integer"); // Retrieve the definition.
                SAttributeDefinitionRetrievalICM::printAttrDef(attrDef);        // Use wrapper funciton from this Sample.
                
            cout << "Retrieved the Attribute Definition of 'S_integer'." << endl;

            //----------------------------------------------------------------
            // Modify the S_integer Attribute Definition in Memory
            //----------------------------------------------------------------
            // Change any of the characteristics of an attribute definition.
            cout << "Modify the Attribute Definition 'S_integer' in Memory..." << endl;
            
                attrDef->setDescription("My modified Integer attribute description.");

            cout << "Modified the Attribute Definition of 'S_integer' in Memory."<< endl;

            //----------------------------------------------------------------
            // Update the S_integer Attribute Definition using an Alternative Method.
            //----------------------------------------------------------------
            cout << "Updating the Attribute Definition 'S_integer' Using an Alternative Method..." << endl;

                dsDefICM->update(attrDef); // save the attribute definition changes.

            cout << "Modified the Attribute Definition of 'S_integer' Using an Alternative Method." << endl;

            //----------------------------------------------------------------
            // Print the Attribute Definition with changes.
            //----------------------------------------------------------------
            cout << "Printing Attribute Definition..." << endl;

                SAttributeDefinitionRetrievalICM::printAttrDef(attrDef); // Sample's Wrapper            
                delete(attrDef);  // Free Memory

            cout << "Printed Attribute Definition." << endl;


            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
