/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Default Resource Manager & SMS Collection
    The Item Type Definition contains information that specifies the default Resource 
    Manager and default SMS Collection that Items of that Item Type are to be stored
    if not otherwise specified.
    
    Please refer to the corresponding wrapper function at the bottom of this sample, 
    and to the Item Type Creation sample, SItemTypeCreationICM for more information
    and examples of how to set and modify these properties.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKItemTypeRelationDefICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SResourceMgrDefSetDefaultICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceMgrDefSetDefaultICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Updating Item Type Definitions for Default RM specifications.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating the Default RM for an Item Type
 *                    Updates All Resource Item Types in Samples
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SResourceMgrDefSetDefaultICM <Existing RM User-Defined Reference Name>
 *                                                 <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.  It should consist of all upper-case letters.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SResourceMgrDefSetDefaultICM RMDB
 *                    ex: SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SResourceMgrDefSetDefaultICM_launch.cpp
 *                    SResourceMgrDefSetDefaultICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceMgrDefSetDefaultICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: " << endl;
            cout << "  SResourceMgrDefSetDefaultICM <Existing RM User-Defined Reference Name>"     << endl;
            cout << "                               <database> <userName> <password>"              << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"    << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"     << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"      << endl;
            cout << "        Sample.  It should consist of all upper-case letters."                << endl;
            cout << endl; 
            cout << "  NOTE: Defaults will be used if the database, username, & password are not"  << endl;
            cout << "        specified.  All others are required."                                 << endl;
            cout << endl;
            cout << "  ex: SResourceMgrDefSetDefaultICM RMDB"                                      << endl;
            cout << "  ex: SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password"          << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="        << endl;
        cout << "IBM Information Integrator for Content   v" << ver    << endl;
        cout << "Sample Program:  SResourceMgrDefSetDefaultICM"        << endl;
        cout << "---------------------------------------------"        << endl;
        cout << "           Database:  " << database                   << endl;
        cout << "           UserName:  " << userName                   << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName << endl;       
        cout << "=============================================\n"      << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // Set Default Resource Manager
            //------------------------------------------------------------------
            // Set the Default Resource Manager for Resource Item Types.
            cout << "Setting Default RM for Sample Resource Item Types..." << endl;

                // For each Resource Item Type created in SItemTypeCreationICM, each will be
                // updated to use by default the specified Resource Manager.
                setDefaultResourceManagerForItemType(dsICM, "S_autoClaim",     rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_adjustrReport", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_policeReport",  rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_insurePolicy",  rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_autoPhoto",     rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_docModel",      rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_lob",           rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_text",          rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_image",         rmUserDefinedReferenceName);

            cout << "Set Default RM Coll for all Sample Resource Item Types." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setDefaultResourceManagerForItemType
    //******************************************************************
   /**
    * Set the Default Resource Manager in which to store items of this Item Types.  Item Type
    * must already be defined.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that Items of this Item Type should be stored in.  It should consist of all upper-case letters.
    **/
    void SResourceMgrDefSetDefaultICM::setDefaultResourceManagerForItemType(DKDatastoreICM* dsICM, char* itemTypeName, char* rmUserDefinedReferenceName){
        cout << "Setting the default Resource Manager for Item Type Named '"<< itemTypeName << "' to Resource Manager Referred to as '" << rmUserDefinedReferenceName << "'." << endl;
        
        // Make sure the RM Definition is setup.
        if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
            throw DKException("Resource Manager not already setup.  RM must be first setup.");

        short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM,rmUserDefinedReferenceName); // get the RM Code.

        setDefaultResourceManagerForItemType(dsICM,itemTypeName,rmCode); // set the item type to use this default rmCode.
    }

   /**
    * Set the Default Resource Manager in which to store items of this 
    * Item Types.  The Item Type must already be defined.  For more information on updating
    * Item Type Definitions, please consult SItemTypeUpdateICM Sample.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmCode        The ID Code in the datastore for this Resource Manager that Items of this Item Type should be stored in.
    **/
    void SResourceMgrDefSetDefaultICM::setDefaultResourceManagerForItemType(DKDatastoreICM* dsICM, char* itemTypeName, short rmCode){
        cout << "Setting the default Resource Manager for Item Type Named '" << itemTypeName << "' to Resource Manager with ID Code '" << rmCode << "'." << endl;

            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                     // Get the Datastore Definition Object

            DKItemTypeDefICM* itemTypeDef = (DKItemTypeDefICM*) dsDefICM->retrieveEntity(itemTypeName); // retrieve the Item Type Definition
            // If CM Doc Model Item Type, update Relations instead
            if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                cout << "Setting Default for Item Type Relations..." << endl;
                dkCollection* coll = dsDefICM->retrieveItemTypeRelations(itemTypeDef->getItemTypeId());
                dkIterator* iter = coll->createIterator();
                while(iter->more()){
                    DKItemTypeRelationDefICM* itemTypeRel = (DKItemTypeRelationDefICM*) iter->next()->value();
                    itemTypeRel->setDefaultRMCode(rmCode);
                    // Both RM & SMS must be set once one is specified.  Detect if SMS has not been set, 0,
                    // and set it in that case to '1' as default.
                    if(itemTypeRel->getDefaultCollCode()==0)
                        itemTypeRel->setDefaultCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.
                    // Item Type Relations also Require Prefetch Code, which can be set to Default CollCode.
                    if(itemTypeRel->getDefaultPrefetchCollCode()==0)
                        itemTypeRel->setDefaultPrefetchCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.

                    itemTypeRel->update();
                    delete(itemTypeRel);         // Free Memory
                }
                delete(iter);                    // Free Memory
                delete(coll);
            } else { // Othwerwise update item type directly
                itemTypeDef->setDefaultRMCode(rmCode);         // Set the default Resource Manager for Items of this Item Type to be stored to.

                // Both RM & SMS must be set once one is specified.  Detect if SMS has not been set, 0,
                // and set it in that case to '1' as default.
                if(itemTypeDef->getDefaultCollCode()==0)
                    itemTypeDef->setDefaultCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.
    
                itemTypeDef->update();                         // Update the Item Type Definition.
            }        
            delete(itemTypeDef);                           // Free Memory

        cout << "ItemType '" << itemTypeName << "' now set to default RM Code of '" << rmCode << "'." << endl;
    }
