/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SSMSCollectionDefRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: SMS Collection Definition Retrieval, Listing, & Printing.  Checking for
 *                    SMS Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: List SMS Collections in System
 *                    Retrieving SMS Collection Definitions
 *                    Checking to See if SMS Collection is Defined
 *                    Pre-defined SMS Collections
 *                    Print SMS Collection Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SSMSCollectionDefRetrievalICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SSMSCollectionDefRetrievalICM RMDB
 *                    ex: SSMSCollectionDefRetrievalICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    The SMS Collections uses here must already be defined.  Please refer
 *                    to the SSMSCollectionDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM_launch.cpp
 *                    SSMSCollectionDefRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSMSCollectionDefRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: " << endl;
            cout << "   SSMSCollectionDefRetrievalICM"                                             << endl;
            cout << "                         <Existing RM User-Defined Reference Name>"           << endl;
            cout << "                         <database> <userName> <password>"                    << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"    << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"     << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"      << endl;
            cout << "        Sample."                                                              << endl;
            cout << endl;
            cout << "  NOTE: Defaults will be used if the database, username, & password are not"  << endl;
            cout << "        specified.  All others are required."                                 << endl;
            cout << endl;
            cout << "  ex: SSMSCollectionDefRetrievalICM RMDB"                                     << endl;
            cout << "  ex: SSMSCollectionDefRetrievalICM MY_RM icmnlsdb icmadmin password"         << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "=============================================="         << endl;
        cout << "IBM Information Integrator for Content    v" << ver     << endl;
        cout << "Sample Program:  SSMSCollectionDefRetrievalICM"         << endl;
        cout << "----------------------------------------------"         << endl;
        cout << "           Database:  " << database                     << endl;
        cout << "           UserName:  " << userName                     << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName   << endl;       
        cout << "==============================================\n"       << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // List SMS Collections
            //------------------------------------------------------------------
            // Listing / Printing SMS Collections found in the system
            cout << "Listing / Printing SMS Collections found in the system..." << endl;

                printAllSMSCollections(dsICM); // Uses wrapper at bottom of sample.

            cout << "Listed / Printed SMS Collections found in the system." << endl;

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            cout << "Obtaining RM Code for RM Definition '" << rmUserDefinedReferenceName << "'..." << endl;
            
                if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw DKException(DKString("Resource Manager Definition Named '")+DKString(rmUserDefinedReferenceName)+DKString("' Not Defined."));

                short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            cout << "Obtained RM Code '" << rmCode << "' for RM Definition '" << rmUserDefinedReferenceName << "'." << endl;

            //-------------------------------------------------------------
            // Checking to See if SMS Collection Definitions Already Setup
            //-------------------------------------------------------------
            // This sample will skip over operations that have already been completed
            // in a previous run of this sample or other program.
            //
            // This sample will check for the system's determined collection names.
            // User-Defined Collections may be substituted.
            //
            // Demonstrating using Wrapper at bottom of sample.
            cout << "Checking to see if system pre-determined SMS Collection Definitions already setup..." << endl;

                if(isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode))
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_NONMEDIA << "' (Default) Already Setup." << endl;
                else 
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_NONMEDIA << "' (Default) Not Setup." << endl;

                if(isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode))
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_MEDIA << "' Already Setup." << endl;
                else 
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_MEDIA << "' Not Setup." << endl;

                if(isSMSCollectionSetupForRM(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode))
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_VIDEO << "' Already Setup." << endl;
                else 
                    cout << "Pre-Defined SMS Collection Name '" << RM_SMS_COLLECTION_VIDEO << "' Not Setup." << endl;
                
            cout << "Checked to see if system pre-determined SMS Collection Definitions already setup." << endl;

            //------------------------------------------------------------------
            // Obtaining an SMS Colleciton Code for a SMS Collection Definition
            //------------------------------------------------------------------
            // Obtaining the SMS Collection Code that is used to specify 
            // properties of Items and Item Types.
            cout << "Obtaining ID Codes for system pre-determined SMS Collection Definitions..." << endl;

                short smsCode = getSMSCollectionCodeForRM(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode); // Uses wrapper at bottom of sample.
                cout << "Collection '" << RM_SMS_COLLECTION_VIDEO << "' has ID Code '" << rmCode << "'." << endl;

                smsCode = getSMSCollectionCodeForRM(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode); // Uses wrapper at bottom of sample.
                cout << "Collection '" << RM_SMS_COLLECTION_MEDIA << "' has ID Code '" << rmCode << "'." << endl;

                smsCode = getSMSCollectionCodeForRM(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                cout << "Collection '" << RM_SMS_COLLECTION_NONMEDIA << "' has ID Code '" << rmCode << "'." << endl;
                
            cout << "Obtained ID Codes for system pre-determined SMS Collection Definitions." << endl;

            //------------------------------------------------------------------
            // Print SMS Collection Definition
            //------------------------------------------------------------------
            // User-Defined names may be substituted for the values below.
            cout << "Printing SMS Collection Definition..." << endl;

                printSMSCollectionDefinition(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                printSMSCollectionDefinition(dsICM, RM_SMS_COLLECTION_MEDIA, rmCode);    // Uses wrapper at bottom of sample.
                printSMSCollectionDefinition(dsICM, RM_SMS_COLLECTION_VIDEO, rmCode);    // Uses wrapper at bottom of sample.

            cout << "Printed SMS Collection Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // getSMSCollectionCodeForRM
    //******************************************************************
   /**
    * Obtain the specified SMS Collection's ID Code for the specified RM.  This function should not be used to simply validate that the SMS Collection exists or is setup.
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code that this SMS Collection is setup for.
    * @return Returns the SMS Collection ID Code for the specified Resource Manager.
    **/
    short SSMSCollectionDefRetrievalICM::getSMSCollectionCodeForRM(DKDatastoreICM* dsICM, char* rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode){
        cout << "Obtaining " << rmSMSCollectionUserOrPreDefinedReferenceName << "'s SMS Collection ID Code in Resource Manager wiht ID Code of '" << rmCode << "'." << endl;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        try { 
            // Find the SMS Collection ID code for the specified SMS Collection.
            DKSMSCollectionDefICM* smsCollDef = rmCfg->retrieveSMSCollection(rmSMSCollectionUserOrPreDefinedReferenceName, rmCode);  // Get the SMS Collection Definition for the SMS Colleciton referred to by the User-Defined Reference Name.
            short collCode = smsCollDef->getSMSCollectionCode();    // Get hte ID Code for the particular SMS Collection.
            cout << "Obtained " << rmSMSCollectionUserOrPreDefinedReferenceName << "'s SMS Collection ID Code of '" << collCode << "'." << endl;
            delete(smsCollDef);  // Free Memory
            return(collCode);
        } catch (DKException exc){ // Then it doesn't exist
            // IMPORTANT: Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listSMSCollections() to search available SMS Collections.
            cout << "An error had occurd probably because it did not find SMS Collection with User Defined Reference Name '" << rmSMSCollectionUserOrPreDefinedReferenceName << "' in Resource Manager with ID Code of '" << rmCode << "'.  Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listSMSCollections() to search available SMS Collections." << endl;
            throw exc;
        }
    }//end getSMSCollectionCodeForRM

    //******************************************************************
    // isSMSCollectionSetupForRM
    //******************************************************************
   /**
    * Determines if the SMS Collection is setup or not for the specified
    * Resource Manager.
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code.
    * @return Returns true if one is setup, false if not.
    **/
    bool SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(DKDatastoreICM* dsICM, char* rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode){
        cout << "Determining if SMS Collection '" << rmSMSCollectionUserOrPreDefinedReferenceName << "' is setup for Resource Manager with ID Code of '" << rmCode << "'..." << endl;

        bool found = false;  // Return value tracking if it was found.
        short smsCode = -1;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of SMS Collection Definitions.
        dkCollection* SMSDefinitions = rmCfg->listSMSCollections(); // Obtain the SMS Collection Definitions defined in the system.
        dkIterator* iter = SMSDefinitions->createIterator();        // create an iterator to search for the specified name.

        while(iter->more()){ // while there are more SMS Collection Definition objects & it has not been found, continue searching
            DKSMSCollectionDefICM* smsCollDef = (DKSMSCollectionDefICM*)iter->next()->value(); // Move pointer to next element & return that object.
            if(!found){      // Only continue checking if not found.   Otherwise, just continue to delete all in the collection to clean up memory.
              if((strcmp(smsCollDef->getName(), rmSMSCollectionUserOrPreDefinedReferenceName)==0)&& (rmCode == smsCollDef->getRMCode())){ // Check to see if names match
                found   = true;            // mark it as found
                smsCode = smsCollDef->getSMSCollectionCode();    // Get the ID Code for the particular SMS Collection.
              }
            }
            delete(smsCollDef);                // Free Memory (Must delete all in collection.  In this case, they will not be deleted for you.
        }
        delete(SMSDefinitions);                // Free Memory (contents must already be deleted).
        delete(iter);

        if(found) // Report Status
            cout << "Found " << rmSMSCollectionUserOrPreDefinedReferenceName << "'s SMS Collection ID Code of '" << smsCode << "'." << endl;
        else
            cout << "Did not find SMS Collection with User Defined Reference Name '" << rmSMSCollectionUserOrPreDefinedReferenceName << "' in Resource Manager with ID Code of '" << rmCode << "'." << endl;

        return(found);
    }//end IsSMSCollectionSetupForRM

    //******************************************************************
    // printAllSMSCollections
    //******************************************************************
   /**
    * Prints / Lists all SMS Collections found in the system.
    * @param dsICM   Connected DKDatastoreICM object.
    **/
    void SSMSCollectionDefRetrievalICM::printAllSMSCollections(DKDatastoreICM* dsICM){

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of SMS Collection Definitions.
        dkCollection* SMSDefinitions = rmCfg->listSMSCollections(); // Obtain the SMS Collection Definitions defined in the system.
        dkIterator* iter = SMSDefinitions->createIterator();        // create an iterator to search for the specified name.

        while(iter->more()){      // while there are more SMS Collection Definition objects & it has not been found, continue searching
            DKSMSCollectionDefICM* smsCollDef = (DKSMSCollectionDefICM*)iter->next()->value(); // Move pointer to next element & return that object.
            cout << "     - " << smsCollDef->getName() << " (" << smsCollDef->getSMSCollectionCode() << ")" << endl; // Print its name & code.
            delete(smsCollDef);   // Free Memory (Contents will not be deleted when collection is deleted).
        }
        delete(iter);             // Free Memory
        delete(SMSDefinitions);   // Free Memory (Contents not deleted with this destructor).
    }//end printAllSMSCollections

    //******************************************************************
    // printSMSCollectionDefinition
    //******************************************************************
   /**
    * Print the specified SMS Collection Definition. 
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code that this SMS Collection is setup for.
    **/
    void SSMSCollectionDefRetrievalICM::printSMSCollectionDefinition(DKDatastoreICM* dsICM, char* rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode){

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        DKSMSCollectionDefICM* smsDef = rmCfg->retrieveSMSCollection(rmSMSCollectionUserOrPreDefinedReferenceName,rmCode);  // Get the SMS Coll. Definition for the specified Resource Manager.
        cout << "-----------------------------------------------"    << endl;
        cout << "          SMS Collection Definition"                << endl;
        cout << "-----------------------------------------------"    << endl;
        cout << "         Name:  " << smsDef->getName()              << endl;
        cout << "      RM Code:  " << smsDef->getRMCode()            << endl;
        cout << "     SMS Code:  " << smsDef->getSMSCollectionCode() << endl;
        cout << "      User ID:  " << smsDef->getUserid()            << endl;
        cout << "-----------------------------------------------"    << endl;
        
        delete(smsDef);  // Free Memory
    }//end printSMSCollectionDefinition
