/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Playing Video Items
    Streaming video may be played through the Video Charger Player.
    
    Online Streaming Video
        When the DKVideoStreamICM.play() function is invoked, it will play the file / 
        return information for an ivs file, which may ben be used with Video Charger
        Player to play the ivs file.
        
        If the Video Charger player has been configured for the current machine and if
        the current machine is an x86 platform, this particular Sample will automatically
        launch the Video Charger Player application, sending it the ivs file obtained.
        
        This does not mean that it will only work on x86 platform.  This particular sample
        has been configured to automatically launch the applicaiton if it detects the x86
        platform.  In any case, the ivs file will still be obtained as shown in this sample,
        regardless of platform.  Video Charger Player may be launched seperately to later
        play the ivs file.
        
    Options & Properties
        For a complete list and explanation of options & properties, please refer to the
        Application Programming Reference (javadoc) for both DKStreamICM & DKVideoStreamICM.
        Several of the key properties are outlined below.
        
        Play Start
            Specifies the beginning time to start playing at.  
            
            Specify the time in the format of hh:mm:ss:ms.  hh - Hours, mm - minutes, 
            ss - seconds, ms - milliseconds.  00:00:00:00 is used to specify the beginning.
            
        Play Stop
            Specifies the time to stop playing.  00:00:00:00 is used to simply play the
            entire file.

            Specify the time in the format of hh:mm:ss:ms.  
            hh - Hours, mm - minutes, ss - seconds, ms - milliseconds.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKVideoStreamICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SVideoItemPlayICM.hpp"

/************************************************************************************************
 *          FILENAME: SVideoItemPlayICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Play a Streaming Video Item.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Playing Video Resource Items
 *                    Play a Streaming Video Online using Video Charger Player
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SVideoItemPlayICM  <3rd Party Host Name>
 *                                       <3rd Party Port>
 *                                       <3rd Party Protocol (http|ftp|file)>
 *                                       <3rd Party User Name>
 *                                       <3rd Party Password>
 *                                       <3rd Party Fully-Qualified File Name>
 *                                       <3rd Party File Path>
 *                                       <3rd Party MIME Type>
 *                                       <3rd Party Content Length>
 *                                       <Video Charger Player Program & Path (N/A in C++ Sample)>
 *                                       <database> <userName> <password>
 *
 *                    IMPORTANT NOTE:  This particular Sample will only launch Video Charger
 *                                     running on a x86 platform.  If not on an x86 platform,
 *                                     the ivs file will be downloaded, but VC must be
 *                                     launched seperately if available to play the video.
 *
 *                    NOTE:  No 3rd Pary information is actually required to Play a video.
 *                           This particular Sample needs the 3rd Party information to first
 *                           create the Video Item before playing it.
 * 
 *                    NOTE:  Video Charger Program, Database, UserName, & Password are
 *                           optional.  If any are not specified defaults will be used.
 *                           
 *                    Ex: SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 C:\Progra~1\IBM\IBMVid~1\iscview.exe icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SVideoItemPlayICM_launch.cpp
 *                    SVideoItemPlayICM.hpp
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SVideoItemPlayICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // User Input Variables
        char * videoChargerPlayer = "C:\\Progra~1\\IBM\\IBMVid~1\\iscview.exe";  // Default location of VideoCharger Player

        // 3rd Party Information
        char * thirdPartyHostName       = "NOT_SPECIFIED";                 // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        char * thirdPartyProtocol       = "NOT_SPECIFIED";
        char * thirdPartyUserName       = "NOT_SPECIFIED";
        char * thirdPartyPassword       = "NOT_SPECIFIED";
        char * thirdPartyFullFileName   = "NOT_SPECIFIED";     
        char * thirdPartyFilePath       = "NOT_SPECIFIED";
        char * thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 14) { // if not all 13 arguments were specified, report usage.
            cout << "Usage: "                                                                      << endl;
            cout << "  SVideoItemPlayICM  <3rd Party Host Name>"                                   << endl;
            cout << "                     <3rd Party Port>"                                        << endl;
            cout << "                     <3rd Party Protocol (http|ftp|file)>"                    << endl;
            cout << "                     <3rd Party User Name>"                                   << endl;
            cout << "                     <3rd Party Password>"                                    << endl;
            cout << "                     <3rd Party Fully-Qualified File Name>"                   << endl;
            cout << "                     <3rd Party File Path>"                                   << endl;
            cout << "                     <3rd Party MIME TYpe>"                                   << endl;
            cout << "                     <3rd Party Content Length>"                              << endl;
            cout << "                     <database> <userName> <password>"                        << endl;
            cout << endl;
            cout << "  IMPORTANT NOTE:  This particular Sample will only launch Video Charger"     << endl;
            cout << "                   running on a x86 platform.  If not on an x86 platform,"    << endl;
            cout << "                   the ivs file will be downloaded, but VC must be"           << endl;
            cout << "                   launched seperately if available to play the video."       << endl;
            cout << endl;
            cout << "  NOTE:  No 3rd Pary information is actually required to Play a video."       << endl;
            cout << "         This particular Sample needs the 3rd Party information to first"     << endl;
            cout << "         create the Video Item before playing it."                            << endl;
            cout << endl;
            cout << "  NOTE:  Video Charger Program, Database, UserName, & Password are optional." << endl;
            cout << "         If any are not specified defaults will be used."                     << endl;
            cout << endl;
            cout << "  Ex: SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428"                                                                     << endl;
            cout << "  Ex: SVideoItemPlayICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 C:\\Progra~1\\IBM\\IBMVid~1\\iscview.exe icmnlsdb icmadmin password" << endl;
            cout << endl;
        }
        if(argc < 10){ // If any 3rd Party info not specified, terminate
            cout << "===================================================" << endl;
            cout << " TERMINATED:  All 3rd Party info must be specified." << endl;
            cout << "===================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  
        } else if(argc < 14) {
            cout << "*** Some parameters not specified.  Using defaults for rest. ***" << endl;
        }
        // Set Command-line options
        if (argc > 1)  thirdPartyHostName      = argv[1];
        if (argc > 2)  thirdPartyPort          = atoi(argv[2]);
        if (argc > 3)  thirdPartyProtocol      = argv[3];
        if (argc > 4)  thirdPartyUserName      = argv[4];
        if (argc > 5)  thirdPartyPassword      = argv[5];
        if (argc > 6)  thirdPartyFullFileName  = argv[6];
        if (argc > 7)  thirdPartyFilePath      = argv[7];
        if (argc > 8)  thirdPartyMIMEType      = argv[8];
        if (argc > 9)  thirdPartyContentLength = atoi(argv[9]);
        if (argc > 10)  videoChargerPlayer      = argv[10];
        if (argc > 11) database                = argv[11];
        if (argc > 12) userName                = argv[12];
        if (argc > 13) password                = argv[13];
        
        char * ver = SConnectDisconnectICM_EIP_VERSION;
        
        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SVideoItemPlayICM"              << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Content Manager:"                               << endl;
        cout << "       Database:  " << database                  << endl;
        cout << "       UserName:  " << userName                  << endl;
        cout << " 3rd Party:"                                     << endl;
        cout << "      Host Name:  " << thirdPartyHostName        << endl;
        cout << "           Port:  " << thirdPartyPort            << endl;
        cout << "       Protocol:  " << thirdPartyProtocol        << endl;
        cout << "       UserName:  " << thirdPartyUserName        << endl;
        cout << "       Password:  " << thirdPartyPassword        << endl;
        cout << "           File:  " << thirdPartyFullFileName    << endl;
        cout << "      Mime Type:  " << thirdPartyMIMEType        << endl;
        cout << " Content Length:  " << thirdPartyContentLength   << endl;
        cout << "  Video Charger:  " << videoChargerPlayer        << endl;
        #ifdef WIN32
            cout << "OS Architecture:  Windows"                   << endl;
        #else
            cout << "OS Architecture:  Non-Windows"               << endl;
        #endif
        cout << "==========================================="     << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Video Resource Item
            //-------------------------------------------------------------
            // Create a Video Resource Item as shown in SVideoItemCreateICM.
            // For more information, please refer to that Sample.
            cout << "Creating Video Resource Item..." << endl;
    
                DKVideoStreamICM* video = (DKVideoStreamICM*)dsICM->createDDO("S_videoType", DK_CM_ITEM); 

                // Set the User-Defined Attributes
                video->setData(video->dataId(DK_CM_NAMESPACE_ATTR,"S_author"), DKString("IBM"));
                video->setData(video->dataId(DK_CM_NAMESPACE_ATTR,"S_length"), DKString("48Hours"));

                // Setting the MIME Type (type of content)
                video->setMimeType(thirdPartyMIMEType);

                // Set 3rd Party Server Info
                video->setThirdPartyHostname(thirdPartyHostName);
                video->setThirdPartyPort(thirdPartyPort);
                video->setInputProtocol(thirdPartyProtocol);
                video->setThirdPartyUserid(thirdPartyUserName);
                video->setThirdPartyPassword(thirdPartyPassword);
                video->setInputFilename(thirdPartyFullFileName);
                video->setInputFilepath(thirdPartyFilePath);
                video->setInputContentlength(thirdPartyContentLength);

                // Synchronous Store
                video->addFrom();  // Add from the 3rd Party specified above.

            cout << "Created Video Resource Item." << endl;

            //-------------------------------------------------------------
            // Recreating XDO objects given PIDs.
            //-------------------------------------------------------------
            // In order to retrieve an Item either the PID Information must
            // be known or Query / Search must be used.  This sample will 
            // use the PID Strings of the items added above.
            cout << "Recreating XDO Object Given PID..." << endl;
            
                // First we will need to know what items to retrieve.  The PID information is needed.
                DKString videoPidString = ((DKPidICM*)video->getPidObject())->pidString();

                // Knowing Pid strings, let's create new, blank DDOs.
                video = (DKVideoStreamICM*) dsICM->createDDO(videoPidString);

            cout << "Recreated XDO Object Given PID." << endl;

            //-------------------------------------------------------------
            // Retrieving Video Item, without Retrieving Video Content
            //-------------------------------------------------------------
            // For more information on Retrieval Options, please refer to
            // SItemRetrievalICM.
            cout << "Retrieving Video Item, without Retrieving Vidoe Content..." << endl;

                video->retrieve(DK_CM_CONTENT_NO);      // Item will be retrieved and resource content will be saved in the specified file.

            cout << "Retrieved Video Item, without Retrieving Vidoe Content." << endl;

            //-------------------------------------------------------------
            // Choose Options
            //-------------------------------------------------------------
            cout << "Choosing Options..." << endl;

                video->setPlayStart("00:00:00:00"); // Start at Beginning
                video->setPlayStop("00:00:00:00");  // Play entire file

            cout << "Choose Options." << endl;

            //-------------------------------------------------------------
            // Write ivs File
            //-------------------------------------------------------------
            // API will write IVS File and then report back the name of that
            // file.
            cout << "Writing ivs File..." << endl;

                    char* ivsfilename = video->retrieveStreamInfo();

                    cout << "API Wrote IVS File Named '" << ivsfilename << "'." << endl;

            cout << "Wrote ivs File." << endl;

            //-------------------------------------------------------------
            // Launch Player & Play Online Streaming Video
            //-------------------------------------------------------------
            // This particular Sample will not launch the player.  It requires
            // the user to launch the application and open the IVS file indicated.
            cout << "Launching Video Charger Player & Playing Online Streaming Video..." << endl;

                #ifdef WIN32
                    cout << "USER ACTION:  Please launch the Video Charger Player manually." << endl;
                    cout << "USER ACTION:  Play the file named '" << ivsfilename << "' in this directory." << endl;
                #else
                    cout << "USER ACTION:  Please launch the Video Charger Player manually." << endl;
                    cout << "USER ACTION:  Play the file named '" << ivsfilename << "' in this directory." << endl;
                #endif

            cout << "Launched Video Charger Player & Played Online Streaming Video." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
