// ----------------------------------------------------------------------------
// @copyright(disclaimer) 
// Licensed Materials - Property of IBM
// IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
// (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
// 
// US Government Users Restricted Rights                                  
// Use, duplication or disclosure restricted by GSA ADP Schedule          
// Contract with IBM Corp.     
// 
// DISCLAIMER OF WARRANTIES :                                             
//                                                                        
// Permission is granted to copy and modify this  Sample code, and to
// distribute modified versions provided that both the copyright        notice,-
// and this permission notice and warranty disclaimer  appear in all copies and
// modified  versions. 
// 
// THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM  AND ITS SUPPLIERS AND LICENSORS
// DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE,
// INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
// IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
// OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
// SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN
// NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST
// REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
// CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
// REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR
// SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
// @endCopyright       
// ----------------------------------------------------------------------------

//******************************************************************************
//*
//* Source File Name = TExecuteCQExprDD.CPP
//*
//* TExecuteCQExprDD - Test datastore.execute
//*               for Domino.Doc connector
//*      
//*****************************************************************************


#include "stdafx.h"
#include "TExecuteCQExprDD.h"

#include "DKDatastoreDD.hpp"
#include "dkQueryableCollection.hpp"
#include "dkResultSetCursor.hpp"
#include "DKAny.hpp"
#include "DKResults.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"

#include "dkSchemaMapping.hpp"
#include "dkAttrMapping.hpp"

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

class tstSchemaMapping : public dkSchemaMapping
{
public:
  tstSchemaMapping();
};
class tstAttrMapping : public dkAttrMapping
{
public:
  tstAttrMapping();
};

tstSchemaMapping::tstSchemaMapping()
: dkSchemaMapping()
{
}

tstAttrMapping::tstAttrMapping()
: dkAttrMapping()
{
}

void printTerm(const char* leftRight, DKQTerm* pTerm1 )
{
  short        aShort            = pTerm1->getOpCode();
  DKQTerm*     pTerm2            = pTerm1->getLeft();
  DKQTerm*     pTerm3            = pTerm1->getRight();
  DKBoolean    aBool             = pTerm1->isNotTerm();
               aBool             = pTerm1->isLeaf();
  DKString     str               = pTerm1->getStringTerm();
               str               = pTerm1->getName();
               str               = pTerm1->getValue();
               aBool             = pTerm1->hasMultiValues();

  if( pTerm2 )
    printTerm("Left", pTerm2 );
  if( pTerm3 )
    printTerm("Right", pTerm3 );
}


/////////////////////////////////////////////////////////////////////////////
// The one and only application object

// simple method to show DDO hit 
void displayDDO(DKDDO* ddo) 
{
   DKString   dataName;
   short      cnt;
   DKAny      any;
   
   if (ddo != NULL) {
      cnt = ddo->dataCount();
	  cout << "display DDO, data count in this DDO = " << cnt << endl;
      for (short i = 1; i <= cnt; i++) {
          any = ddo->getData(i);
          dataName = ddo->getDataName(i);
          cout << i << ": Attribute Name :" << dataName << endl;
          if (!any.isNull())
               cout << ",\t   Value :" << any << endl;
          else cout << ",\t   Value : null"  << endl;
      }  
   }
}

void printDDO(DKDDO* ddo)
{
  if(ddo != NULL)
  {
    DKPid p = ddo->getPid();
    DKString pidDSType = p.getDatastoreType();
    DKString pidDSName = p.getDatastoreName();
    DKString pidId = p.getPrimaryId();
    DKString pidOBType = p.getObjectType();
    DKBoolean pidSet = p.isSet();
    long pidStrCount = p.getIdStringCount();

    cout << "DDO" << endl;
    cout << "PID ID: " << pidId << endl;

    DKString ddoOBType = ddo->getObjectType();
    short ddoProt = ddo->protocol();
    DKBoolean ddoUpdatable = ddo->updatable();

    ushort ddoDataCount = ddo->dataCount();

    for( ushort i = 0; i < ddoDataCount; i++ )
    {
      DKString ddoDataName = ddo->getDataName( i );
      DKAny aData = ddo->getData(i);
      ushort ddoPropCount = ddo->dataPropertyCount(i);
      for (ushort x = 0; x < ddoPropCount; x++ )
      {
        DKString propName = ddo->getDataPropertyName( i, x );
        DKAny aProp = ddo->getDataProperty( i, x );
      }
    }

  
    DKString ddoDefOBType = ddo->getDefObjectType();
  }
}


void printResults(dkResultSetCursor* pResults)
{
  if(pResults != NULL)
  {
    while( pResults->isValid() )
    {
      DKDDO* item = pResults->fetchNext();
      if( item )
      {
        //printDDO(item);
        displayDDO(item);
        delete item;
      }
    }
  }
}



int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TExecuteCQExprDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	
	CTime startTime = CTime::GetCurrentTime();
	CString sTime = startTime.Format( "%B %d, %Y at %H:%M:%S" );
	printf("Begin TExecuteCQExprDD at %s.\n", sTime);

	if( nbrArgs < 5 )
	{
		cout << "TExecuteCQExprDD usage = <serverName> <userId> <pw> <\"cabinetTitle\">" << endl;
		return -1;
	}

	DKDatastoreDD* dd;
	try
	{
		DKString server  = argv[1];
		DKString uid     = argv[2];
	    DKString pw      = argv[3];
		DKString cabName = argv[4];
	    dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{
 
		// All set; Connect and execute the compound query expression
		// get the Def
		dkDatastoreDef*  defFed = dd->datastoreDef();
     
		// set up display list and option list.
		DKNVPair opt[1];
		opt[0].set("CONTENT", DKString("YES"));
		long optSize = 1;

		DKString dispList[] =        {"FedTitle","FedDocType"};
		long dispSize = 2;
      
		DKString _federatedNames[] = {"FedTitle", "FedDocType"};
		long fedNamesSize = 2;

		DKString _dominoFldNames[] = {"Title", "DocType"};
		long domNamesSize = 2;

		DKString fedCabName("DD_SEARCH");

      
		DKString strNativeEntityName = cabName;
		DKString strMappedEntityName = "DD_SEARCH";

		tstSchemaMapping* mapFed    = new tstSchemaMapping();
		tstAttrMapping* attrMapFed  = new tstAttrMapping();
		tstAttrMapping* attrMapFed2 = new tstAttrMapping();

		mapFed->setServerName(server);
		mapFed->setServerType(DK_DD_DSTYPE);
		mapFed->setEntityName(strNativeEntityName);
		mapFed->setMappedEntityName(strMappedEntityName);
	//	mapFed->setName("DD_SEARCH;http://smbdoc.austin.ibm.com/domdoc/webmaster02lib.nsf;");

		DKAttrDefDD* attrDef = new DKAttrDefDD(dd);
		attrDef->setName(_federatedNames[0]);
		attrDef->setType(DK_CM_VSTRING);
		attrMapFed->addMappedAttr(attrDef); // Add LHS

		attrDef = new DKAttrDefDD(dd);
		attrDef->setName(_dominoFldNames[0]);
		attrDef->setType(DK_CM_VSTRING);
		attrMapFed->addMapToAttr(attrDef); // Add RHS

		attrDef = new DKAttrDefDD(dd);
		attrDef->setName(_federatedNames[1]);
		attrDef->setType(DK_CM_VSTRING);
		attrMapFed2->addMappedAttr(attrDef); // Add LHS
		attrDef = new DKAttrDefDD(dd);
		attrDef->setName(_dominoFldNames[1]);
		attrDef->setType(DK_CM_VSTRING);
		attrMapFed2->addMapToAttr(attrDef); // Add RHS


		mapFed->addAttrMapping(attrMapFed);
		mapFed->addAttrMapping(attrMapFed2);

		DKNVPair* sourceMap = new DKNVPair(DK_CM_SM_SCHEMA, mapFed);
		dd->registerMapping(sourceMap);

		//===================================================================================================
		// Create search expresion
		//===================================================================================================
      
		DKQTerm* qTerm1 = new DKQTerm(_federatedNames[0], DK_CM_OPCODE_LIKE, "G*");
		DKQTerm* qTerm2 = new DKQTerm(_federatedNames[1], DK_CM_OPCODE_LIKE, "DWF*");
		DKQTerm* qTerm3 = new DKQTerm(_federatedNames[0], DK_CM_OPCODE_LIKE, "H*");

	//	DKQTerm* qTerm = new DKQTerm(qTerm1, DK_CM_OPCODE_AND, qTerm2);
		      
		cout << " Term COND = " << qTerm2->toString() << endl;

		// Create query sub expression
		DKQSubExpr* qse = new DKQSubExpr();
		qse->setTranslation(true);
		qse->setQueryTerm(qTerm2);
		qse->setEntityName(cabName);
		qse->setDisplayList(dispList, dispSize);
		qse->setOptionList(opt, optSize);
		qse->setMappingName(mapFed->getName());

		// Create an expression
		DKQExpr* qe = new DKQExpr();
		qe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
		qe->addSubQuery(qse);
		qe->setOptionList(opt, optSize);
		qe->setDatastoreType( dd->datastoreType() );
		qe->setDatastoreName( dd->datastoreName() );
   
		// Create a combined expression
		DKCQExpr* cqe = new DKCQExpr(qe);
		cqe->setQLType(DK_CM_PARAMETRIC_QL_TYPE);
		cqe->setDatastoreType( dd->datastoreType());
		cqe->setDatastoreName( dd->datastoreName() );

#ifdef NeverDefined
		{
			// Get Query Info, just to see if we built a good one
			DKString str      = cqe->getDatastoreName();
			str               = cqe->getDatastoreType();
			short aShort      = cqe->getQLType();
			DKQExpr* dkqexpr1 = cqe->getQueryExpr();

			aShort            = dkqexpr1->getQLType();
			str               = dkqexpr1->getDatastoreType();
			str               = dkqexpr1->getDatastoreName();
			DKBoolean aBool   = dkqexpr1->isTranslation();
			long aLong = 0;
			DKString* pStr    = dkqexpr1->getMappingNames(aLong);
			str               = dkqexpr1->getAssociatedMapping();
			aLong = 0;
			pStr              = dkqexpr1->getEntityNames(aLong);
			aLong             = dkqexpr1->subQueryCount();
			DKQSubExpr* pSub1 = dkqexpr1->getSubQuery(0);
			aLong = 0;
			DKNVPair* dknvpair = dkqexpr1->getOptionList( aLong );
			aShort            = dkqexpr1->opCode();
			DKQExpr* dkqexpr2 = dkqexpr1->left();
			dkqexpr2         = dkqexpr1->right();
			aBool             = dkqexpr1->isLeaf();
        
			str               = pSub1->getEntityName();
			str               = pSub1->getMappingName();
			int aInt          = pSub1->getMappingType();
			aBool             = pSub1->isTranslation();
			aLong=0;
			pStr              = pSub1->getDisplayList(aLong);
			DKQTerm* pTerm1   = pSub1->getQueryTerm();
        
			if( pTerm1 )
				printTerm( "Root", pTerm1 );
		}
#endif


		//===================================================================================================
		// Execute search expresion
		//===================================================================================================
		cout << " ** execute with result set cursor ** " << endl;
		//cout << " cqe expression is " << cqe->toString();
		dkResultSetCursor* rsc = dd->execute(cqe);

		//  get result set cursor
		if (rsc == NULL) 
			cout << " *** cursor is null" << endl;
		else 
		{
			int count = rsc->cardinality();
			cout << " hit count = " << count << endl;
			DKDDO* ddo;
            int someCount = 0;
			cout <<" Cursor 's position 1= " << rsc->getPosition() << endl;
		while (rsc->isValid())  
			{
				cout << " inside rsc is valid, count = " << someCount <<endl;
							cout <<" Cursor 's position 2= " << rsc->getPosition() << endl;
			
				ddo = rsc->fetchNext();
				cout << " Done with fetchNext \n";

				cout<<"check if cursor is valid after fetching next" << endl;

				if (ddo != NULL) 
				{  
					cout << " inside ddo isnot NULL\n";

					displayDDO(ddo);
					delete ddo;
				} // if
				else 
					break;
			}

			delete rsc;
		}
		dd->disconnect();
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
			cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TExecuteCQExprDD FAILURE! Caught unexpected exception." << endl;
	}
	
	delete dd;

	CTime endTime = CTime::GetCurrentTime();
	CString eTime = endTime.Format( "%B %d, %Y at %H:%M:%S" );
	CTimeSpan elapsedTime = endTime - startTime;
	CString elTime = elapsedTime.Format("%M:%S");
	printf("End TFedExecuteDD at %s.  Elapsed time: %s\n", eTime, elTime);

	return nRetCode;
}

