// ----------------------------------------------------------------------------
// @copyright(disclaimer) 
// Licensed Materials - Property of IBM
// IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
// (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
// 
// US Government Users Restricted Rights                                  
// Use, duplication or disclosure restricted by GSA ADP Schedule          
// Contract with IBM Corp.     
// 
// DISCLAIMER OF WARRANTIES :                                             
//                                                                        
// Permission is granted to copy and modify this  Sample code, and to
// distribute modified versions provided that both the copyright        notice,-
// and this permission notice and warranty disclaimer  appear in all copies and
// modified  versions. 
// 
// THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM  AND ITS SUPPLIERS AND LICENSORS
// DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE,
// INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
// IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT
// OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
// SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN
// NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST
// REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
// CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
// REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR
// SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
// @endCopyright       
// ----------------------------------------------------------------------------

/*
 * Filename       : TResultSetCursorScrollableDD.cpp
 * Description    : Execute Query to get scrollable result set cursor. We test
 *                  to move result set cursor backward and forward print out the
 *                  cursor information and correpondence DDO info
 *
 */
//  : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "TResultSetCursorScrollableDD.h"


#include "DKDatastoreDD.hpp"
#include "dkQueryableCollection.hpp"
#include "DKResultSetCursorScrollableDD.hpp"
#include "dkResultSetCursorScrollable.hpp"
#include "DKAny.hpp"
#include "DKResults.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

void printDDO(DKDDO* ddo)
{
	if(ddo != NULL)
	{
		DKPid p = ddo->getPid();
		cout << "************PID************" << endl;
		cout << "PID ID: " << p.getId() << endl;
		cout << "PID DS: " << p.getDatastoreType() << endl;
		cout << "PID DS: " << p.getDatastoreName() << endl;
		cout << "PID ObjType: " << p.getObjectType() << endl;
		cout << "ID String Count: " << p.getIdStringCount() << endl;
		cout << "************DDO************" << endl;
		cout << "Object Type: " << ddo->getObjectType() << endl;
		cout << "Data Count: " << ddo->dataCount() << endl;
		for(int i = 1; i <= ddo->dataCount(); i++)
		{
			DKAny any = ddo->getData(i);
			DKString dataName = ddo->getDataName(i);
			cout << dataName << ":\t\t" << any << endl;
		}
	}
}

void printCursor(DKResultSetCursorScrollableDD* cursor)
{
	if(cursor != NULL)
	{
		cout << "DKResultSetCursorScrollableDD" << endl;
		cout << "\tScrollable: " << ((cursor->isScrollable() == TRUE) ? "True" : "False" ) << endl;
		cout << "\tUpdateable: " << ((cursor->isUpdatable()	 == TRUE) ? "True" : "False" ) << endl;
		cout << "\tValid:      " << ((cursor->isValid()		 == TRUE) ? "True" : "False" ) << endl;
		cout << "\tOpen:       " << ((cursor->isOpen()		 == TRUE) ? "True" : "False" ) << endl;
		cout << "\tBeginning:  " << ((cursor->isBegin()		 == TRUE) ? "True" : "False" ) << endl;
	    cout << "\tEnd :       " << ((cursor->isEnd()		 == TRUE) ? "True" : "False" ) << endl;
		cout << "\tInBetween:  " << ((cursor->isInBetween()	 == TRUE) ? "True" : "False" ) << endl;
		cout << "\tPosition:   " << cursor->getPosition() << endl;
	}
}


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TResultSetCursorScrollableDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	if( nbrArgs < 4 )
	{
	    cout << "TResultSetCursorScrollableDD usage = <serverName> <userId> <pw>" << endl;
		return -1;
	}

	DKDatastoreDD* dd;
	try
	{
		DKString server = argv[1];
		DKString uid = argv[2];
		DKString pw = argv[3];
		int count = 1;
		dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{

			DKString qString0(" ([\"DocType\"]CONTAINS\"DWF*\")");
			DKString queryParms0("ENTITY=WebPublishing;QUERY=");
			DKString queryString0 = queryParms0 + qString0;
			cout << "Query: " << queryString0 << endl;
			
			dkResultSetCursor* rcs = dd->execute(queryString0, DK_PARAMETRIC_QL_TYPE, NULL);
			
			dkResultSetCursorScrollable* pRCS = (dkResultSetCursorScrollable*)rcs;
			DKResultSetCursorScrollableDD* rscDD = (DKResultSetCursorScrollableDD*)pRCS;
			if(rscDD != NULL)
			{
				unsigned long numResults = rscDD->cardinality();
				cout << " Print the first curson" << endl;
				printCursor(rscDD);
			
				DKDDO* ddo = rscDD->fetchObject();
				if(ddo == NULL)
				{
					cout << "In position -1, DDO is null" << endl;
				}
				
				if(numResults > 0)
				{
					//should be first element
					ddo = rscDD->fetchNext();
					if (ddo != NULL)
					   cout << "This is the first ddo0, postion 0\n";
					else
						cout << "DDO0 is null\n";
					printDDO(ddo);
					cout << "Done with print DDO 0, Start to print cursor 0\n";
					printCursor(rscDD);
	                cout << " Done with print cursor 0\n";
					// loop through the rest
					try
					{
						while(rscDD->isValid() && !rscDD->isEnd())
						{
   				  	        ddo = rscDD->fetchNext();
							if(ddo != NULL)
								cout << "Start to println DDO " << count << endl;
							else
								cout << "DDO " << count << " is NULL\n";
							printDDO(ddo);
							cout << " Done with print DDO " << count <<", Start to println cursor " << count << endl;
							printCursor(rscDD);
							count++;
							delete ddo;
						}
					}
					catch( DKException dke )
					{
						for( long i = 0; i < dke.textCount(); i++ )
						{
							const char* text = dke.text(i);
							if( text != 0 )
								cout << text << endl;
						}
					}

	                cout << " =================== Done with the one way print ======================\n";				
					rscDD->setToPrevious();
					cout << "===================Start to print cursor back ward\n";
					printCursor(rscDD);
					printDDO(rscDD->fetchObject());
					// should be at the end, now, loop backwards
					while(rscDD->isValid() && !rscDD->isBegin())
					{
						ddo = rscDD->fetchPrevious();
						if(ddo != NULL)
							cout << "Starting to print backward DDO " << count << endl;
			        	else 
							cout << "DDO " <<count <<" is NULL\n";
						printDDO(ddo);
		
						printCursor(rscDD);
						count--;
						delete ddo;
					}
				}
				
				pRCS->close();
				pRCS->destroy();
				delete pRCS;
			}

			dd->disconnect();
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
				cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TResultSetCursorScrollableDD FAILURE! Caught unexpected exception." << endl;
	}


	delete dd;
	return nRetCode;
}

