//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample submit a parametric query to DES datastore
/*
 * Filename       : SamplePQryDES.cpp
 * Description    : Connect to DES Datastore
 *                  Create query string
 *                  specify to retrieve folder only.
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from DES Datastore
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDES.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKDate.hpp>
#include <DKPid.hpp>
#include <DKDESQuery.hpp>
#include <dkCollection.hpp>
#include "DKBlobDES.hpp"


void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //--------------------------------------------------------------   
      if (argc < 5)
      {
       cout<<"Invoke Syntax: "<<endl;
	     cout<< argv[0] << " <libsrv> <userid> <pw> <PORT=80;DESAPPID=appid;DESAPPPW=apppass[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>;]>" << endl;
       return;
      }
      // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
	DKString str = argv[4];

    DKDatastoreDES dsDES;

    try {
         dkQuery* pQry;
         DKAny any;
         DKResults* pResults;

         cout << "connecting to datastore" << endl;
         dsDES.connect(libsrv,userid,pw,str);
         cout << "datastore connected libsrv: " << libsrv << " userid: " << userid << endl;

		DKString cmd = "SEARCH=(DATABASE=(Notes 5 Help);";
		cmd += "COND=((IN \"Subject\" \"Notes\")));";
		cmd += "OPTION=(SEARCHABLE_FIELD=(Doc_Number); RETRIEVABLE_FIELD=(Indent);";
		cmd += "MAX_RESULTS=2;TIME_LIMIT=60);";

         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsDES.createQuery(cmd);
         cout << "executing query" << endl;
         pQry->execute();
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result(); 
         pResults = (DKResults*)((dkCollection*) any);
         
         processResults(pResults);

         dsDES.disconnect();
         cout << "datastore disconnected" << endl;
         delete pQry;
         delete pResults;
    }
    catch(DKException &exc)
    {
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i < exc.textCount(); i++) {
             cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long j=0; j < exc.locationCount(); j++) {
              const DKExceptionLocation* p = exc.locationAtIndex(j);
              cout << "Filename: "   << p->fileName()     << endl;
              cout << "Function: "   << p->functionName() << endl;
              cout << "LineNumber: " << p->lineNumber()   << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny a;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
      itemType = item->getPropertyByName(DK_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_DOCUMENT:
             cout << " Item is a document, " 
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
				a = item->getData(i);
				if (a.typeCode() == DKAny::tc_dobase)
				{
					dkDataObjectBase* pDOBase = a;
					dkXDO* pXDO = (dkXDO*)pDOBase;
					((DKBlobDES*)pXDO)->getContentToClientFile("c:\\temp\\temp.html", 1);
				}
				ushort usCount = item->dataPropertyCount(i);
				for (ushort k = 1; k <= usCount; k++)
				{
					DKAny a = item->getDataProperty(i, k);
					cout << " property " << k << " " << a << endl;
				}	
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }     
   }
   delete iter;
}
