/************************************************************************/
/* @copyright(disclaimer)                                                               */
/* IBM Enterprise Information Portal for Multiplatforms V6.1  (program number 5648-D03) */
/* (c ) Copyright IBM Corp. 1998, 1999. All Rights Reserved.                            */
/* IBM Content Manager for Multiplatforms V6.1  (program number 5648-D04)               */
/* (c ) Copyright IBM Corp. 1994, 1999. All Rights Reserved.                            */
/*                                                                                      */
/* US Government Users Restricted Rights                                                */
/* Use, duplication or disclosure restricted by GSA ADP Schedule                        */
/* Contract with IBM Corp.  Licensed Materials - Property of IBM                        */
/*                                                                                      */
/* DISCLAIMER OF WARRANTIES :                                                           */
/*                                                                                      */
/* Permission is granted to copy and modify this code, and to                           */
/* distribute modified versions provided that both the copyright                        */
/* notice and this permission notice appear in all copies and modified                  */
/* versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS                   */
/* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM                  */
/* WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE                 */
/* OF THE CODE.                                                                         */
/*                                                                                      */
/* @endCopyright                                                                        */
/************************************************************************/

// sample connect/add ddo/disconnect from DL
/*
 * Filename       : TAddRetDelDDODL.cpp
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>
#include <DKEnvironment.hpp>
#include "DKFolder.hpp"
#include "DKParts.hpp"

void displayDDO(DKDDO* ddo);

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TAddRetDelDDODL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    try {
    DKDatastoreDL dsDL;
    DKPid pid, fpid;
    DKPidXDODL *jpid, *tpid;
    DKAny aValue, any; 
    
    DKString indexclass = "DLSAMPLE";
    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect(libsrv,userid,pw);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    pid.setObjectType(indexclass);        
    fpid.setObjectType(indexclass); 

    DKDDO* pddo = new DKDDO(&dsDL,pid);       // ddo for member
    DKDDO* pfddo = new DKDDO(&dsDL, fpid);    // ddo for folder   
    
    jpid = new DKPidXDODL();
    tpid = new DKPidXDODL();
    
    DKFolder* folder = new DKFolder();
  
    aValue = DK_DL_DOCUMENT;
    pddo->addProperty(DK_DL_PROPERTY_ITEM_TYPE,aValue);

       short dataId = 0;
    
       DKString* attrName = new DKString[12];
       attrName[0] = "DLSEARCH_Author";
       attrName[1] = "DLSEARCH_Publisher";
       attrName[2] = "DLSEARCH_DocType";
       attrName[3] = "DLSEARCH_Date";
       attrName[4] = "DLSEARCH_Title";
       attrName[5] = "DLSEARCH_Subject";
       attrName[6] = "DLSEARCH_Authors";
       attrName[7] = "DLSEARCH_Editors";
       attrName[8] = "DLSEARCH_Source";
       attrName[9] = "DLSEARCH_Volume";
       attrName[10] = "DLSEARCH_Issue";
       attrName[11] = "DLSEARCH_Relation";

        DKAny* attrValue = new DKAny[12];
        attrValue[0] = DKString("Robert");
        attrValue[1] = DKString("IBM");
        attrValue[2] = DKString("html");
        attrValue[3] = DKString("1999");
        attrValue[4] = DKString("Time after time");
        attrValue[5] = DKString("Space and Time");
        attrValue[6] = DKString("Robert, Jonathan");
        attrValue[7] = DKString("Green publishing");
        attrValue[8] = DKString("Space Quarterly");
        attrValue[9] = DKString("2");
        attrValue[10] = DKString("1");
        attrValue[11] = DKString("");

       short* attr_prop_type = new short[12];
       attr_prop_type[0] = DK_VSTRING;
       attr_prop_type[1] = DK_VSTRING;
       attr_prop_type[2] = DK_VSTRING;
       attr_prop_type[3] = DK_VSTRING;
       attr_prop_type[4] = DK_VSTRING;
       attr_prop_type[5] = DK_VSTRING;
       attr_prop_type[6] = DK_VSTRING;
       attr_prop_type[7] = DK_VSTRING;
       attr_prop_type[8] = DK_VSTRING;
       attr_prop_type[9] = DK_VSTRING;
       attr_prop_type[10] = DK_VSTRING;
       attr_prop_type[11] = DK_VSTRING;

       short attrCount = 12;

       short i;
       for (i=0; i < attrCount; i++)
       {
        dataId = pddo->addData(attrName[i]);
        aValue = attr_prop_type[i];
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, aValue); // mandatory
        aValue = DK_CM_TRUE;
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, aValue); // optional
        aValue = DK_CM_ATTRIBUTE_READWRITE;
        pddo->addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT, aValue); // optional
        aValue = attrValue[i];
        pddo->setData(dataId, aValue);
       }    

       cout << " displaying the member ddo before adding it " << endl;
       // display ddo 
       displayDDO(pddo);

       cout << " adding the ddo " << endl;
       pddo->add();

       pid = *((DKPid*)pddo->getPidObject());
       cout << " the new added member ddo itemid = " << pid.getPrimaryId() << endl;
       
       cout << " Add XDOs " << endl;
       DKParts* parts = new DKParts();
       DKBlobDL* jpg = new DKBlobDL(&dsDL);
       DKBlobDL* txt = new DKBlobDL(&dsDL);
                    
       cout << " set jpid up with the blob " << endl;
       jpid->setPartId(1); 
       jpid->setPrimaryId( pid.getPrimaryId() );

//       cout << " blob processing -- setPidObject " << endl;
       jpg->setPidObject(jpid);  
//       cout << " about to set CC " << endl;
       jpg->setContentClass(DK_CC_JPG);         
//       cout << " set Rep type " << endl;
       jpg->setRepType(DK_REP_NULL);  
//       cout << " set content from file " << endl;
       jpg->setContentFromClientFile("me.jpg");
//       cout << " add the xdo " << endl;
       jpg->add();
      
       cout << " set tpid up with the blob " << endl;
       tpid->setPartId(2);
       tpid->setPrimaryId( pid.getPrimaryId() );
       txt->setPidObject(tpid);
       txt->setContentClass(DK_CC_ASCII);
       txt->setSearchEngine("SM");
       txt->setSearchIndex("TM-TMINDEX");
       txt->setSearchInfo("ENU");
       txt->setContentFromClientFile("test.txt");
       txt->add();
       
       parts->addElement((dkDataObjectBase*)jpg);
       parts->addElement((dkDataObjectBase*)txt);

       cout << " about to get the dataId " << endl;
       dataId = pddo->dataId(DKPARTS); 

       cout << " Add parts to the member ddo " << endl;
       if (dataId == 0) { 
             
          dataId=pddo->addData(DKPARTS); 
          //pddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE,
          //    DK_CM_COLLECTION_XDO);   
           // add type property
          aValue = DK_CM_DATAITEM_TYPE_COLLECTION_XDO;
          pddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, aValue);
          // add nullable property
          aValue = TRUE;
          pddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, aValue);
       }  // end of creating dkparts attribute
                
       cout << " dataId is " << dataId << endl;
       any = (dkCollection*) parts;
       pddo->setData( dataId, any );
       pddo->update();

       delete pddo;

       cout << " retrieve the newly added ddo, with itemid = " << pid.getPrimaryId() << endl;
       cout << " default of Content = yes" << endl;   
       
       DKDDO* nddo = new DKDDO(&dsDL,pid);
       nddo->retrieve();

       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
       displayDDO(nddo);
                            
       cout << " retrieve the newly added ddo, with itemid = " << pid.getPrimaryId() << endl;
       cout << " ATTRONLY " << endl;       
       aValue = DK_CM_CONTENT_ATTRONLY;
       dsDL.setOption( DK_CM_OPT_RETRIEVE, aValue ); 

       nddo->retrieve();
       
       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
       displayDDO(nddo);

       delete nddo;       
       nddo = new DKDDO(&dsDL,pid);
                                  
       cout << " retrieve the newly added ddo, with itemid = " << pid.getPrimaryId() << endl;
       cout << " Content=yes again " << endl;       
       aValue = DK_CM_CONTENT_YES;
       dsDL.setOption( DK_CM_OPT_RETRIEVE, aValue ); 

       nddo->retrieve();
       cout << " pid value in nddo (before add to folder)" << ((DKPid)nddo->getPid()).getPrimaryId() << endl;
              
//       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
//       displayDDO(nddo);

       // Create a folder and add the ddo to it
       folder->addElement( nddo );
       aValue = DK_FOLDER;
       pfddo->addProperty(DK_CM_PROPERTY_ITEM_TYPE,aValue); 
       dataId=pfddo->addData(DKFOLDER); 
       aValue = DK_CM_COLLECTION_DDO;
       pfddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, aValue);
       // add nullable property
       aValue = TRUE;
       pfddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, aValue);
       pfddo->setData(dataId, folder);

       pfddo->add();  
       
       fpid = *((DKPid*)pfddo->getPidObject());
       cout << " the new added folder ddo itemid = " << fpid.getPrimaryId() << endl;
       
       delete pfddo;

       cout << " retrieve the newly added folder ddo, with itemid = " << fpid.getPrimaryId() << endl;
       cout << " default of Content = yes" << endl;   
       
       DKDDO* nfddo = new DKDDO(&dsDL,fpid);
       nfddo->retrieve();

       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
       displayDDO(nfddo);  
       
       cout << " retrieve the newly added folder ddo, with itemid = " << fpid.getPrimaryId() << endl;
       cout << " ATTRONLY " << endl;       
       aValue = DK_CM_CONTENT_ATTRONLY;
       dsDL.setOption( DK_CM_OPT_RETRIEVE, aValue ); 

       nfddo->retrieve();
       
       cout << " displaying the retrieved ddo " << endl;
       // display ddo 
       displayDDO(nfddo);

       // delete the folder and the member from the db    
       
       cout << " delete the folder from DL " << endl; 
       cout << " value of fpid " << fpid.getPrimaryId() << endl;
       DKString itemId = fpid.getId();
//       itemID = (PITEMID) ((char*) itemId);
       cout << " itemId = " << itemId << endl; 
       cout << " pid value in nfddo " << ((DKPid)nfddo->getPid()).getPrimaryId() << endl;
       nfddo->del(); 
       
       cout << " delete the document from DL " << endl;
       cout << " value of pid " << pid.getPrimaryId() << endl;
       itemId = pid.getId();
//       PITEMID itemID = (PITEMID) ((char*) itemId);
       cout << " itemId = " << itemId << endl; 
       // somehow nddo is gone...       
       nddo = new DKDDO(&dsDL,pid);
       nddo->retrieve();
//       pid = bddo->getPid();
       cout << " pid value in nddo " << ((DKPid)nddo->getPid()).getPrimaryId() << endl;
       nddo->del();
   

//       delete pddo;                     
//       cout << " before delete of memory " << endl;
       
//       cout << "before delete of jpid " << endl;
//       delete jpid;                             
//       cout << "before delete of tpid " << endl;
//       delete tpid;
       
//       cout << "before delete of folder" << endl; 
//       delete folder;                            
//       cout << "before delete of parts" << endl;
//       delete parts;                            
                                      
//       cout << "before delete of jpg blob " << endl;
//       delete jpg;                                  
//       cout << "before delete of txt blod" << endl; 
//       delete txt;                                 
         

// must delete the folder first !!!!
      
//       cout << "before delete of nfddo" << endl;
//       delete nfddo;                            
//       cout << " before delete of member" << endl;
//       delete nddo;

    cout << " before disconnect " << endl;
    dsDL.disconnect();
    cout << "datastore disconnected" << endl;
    }  // end of try
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    } // end of catch
    cout << "done ..." << endl;

}

void displayDDO(DKDDO* ddo) {
     DKString dataName;
     DKAny any; 
     DKAny* pAny;   
     DKBlobDL* blob;
//     DKParts parts;

     cout << "data count = " << ddo->dataCount() << endl;  
     unsigned short itemType = ddo->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);

     /* --------------------------------------------- */
     cout << " item id[" << ((DKPid*)ddo->getPidObject())->getPrimaryId() << "]  " << endl;
//     for (unsigned short j = 1; j <= ddo->dataCount(); j++) {
//         dataName = ddo->getDataName(j);
//         cout << " attr[" << j << "] = "
//              << dataName  << ", \t\t value "
//              << ddo->getData(j) << endl;
//     }  
     
     if (itemType == DK_CM_DOCUMENT) {  
     
         cout << " item is a document " << endl; 
     }                                                   
     else if (itemType == DK_CM_FOLDER) {  
     
         cout << " item is a folder " << endl; 
     }

       /* ------------------------------------------------*/     
      for (unsigned short j = 1; j <= ddo->dataCount(); j++) {
        
         dataName = ddo->getDataName(j);
         cout << " attr[" << j << "] = "
              << dataName  << ", \t\t value "
              << ddo->getData(j) << endl;

        if (dataName == DKPARTS) {
        cout << " dataname is DKPARTS " << endl;
              any = ddo->getData(j);
              if (any.value() != NULL) {
                DKParts* parts = (DKParts*) any.value();
                dkIterator* iter  = parts->createIterator();
                while(iter->more()) {   
                     pAny = iter->next();
                     blob = (DKBlobDL*) pAny->value();
            
                   if ( blob != NULL ) {
        
                   cout << "blob part id " 
                        << ((DKPidXDODL *)(blob->getPid()))->getPartId()
                        << " content class " 
                        << blob->getContentClass()
                        << " rep type "
                        << blob->getRepType()
                        << " aff type "
                        << blob->getAffiliatedType() 
                        << endl;
                     
//                     blob->retrieve();    // retrieve a part
                   }
              }   
                delete iter;
              } // end of if any == null
             
        } // end of if dataname == DKParts  
        else  if (dataName == DKFOLDER) {
           cout << " dataname is DKFOLDER " << endl;
           any = ddo->getData(j);
           if (any.value() != NULL) {
             
               DKFolder* folder = (DKFolder*) any.value();  
             
               dkIterator* iter  = folder->createIterator();
               DKDDO* mddo;
               while(iter->more()) {  
                  pAny = iter->next();
                  mddo = (DKDDO*) pAny->value();
                  if ( mddo != NULL ) {
                   mddo->retrieve();      
                   cout << "about to recurse through the folder " << endl;
                   displayDDO( mddo );  
                  }
               }                     
               delete iter;
             
           }
        }
      } // end of for loop
}
