/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TLogUtils.hpp
 * Description    : Test utility methods used in FVT
 *
 * Change History
 * Date        Name  Description     
 * ----------  ----- -----------------------------------------------------
 * 2000/07/24  CSP   created
 *------------------------------------------------------------------------
 */

#ifndef  _TLOGUTILS_HPP_
#define  _TLOGUTILS_HPP_

#include <stdio.h>

#include <DKConstant.h>
#include <DKEnvironment.hpp>
#include <DKString.hpp>
#include <DKException.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <dkXDO.hpp>
#include <DKPidXDO.hpp>
#include <DKAny.hpp>
#include <DKParts.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>

#define  _ID_ERRFILE_EXT              ".err"
#define  _ID_OUTFILE_EXT              ".log"
#define  _ID_DEFAULT_INFILE           "TestIn.txt"
#define  _ID_STR_COMMAND_DELIMITER    ", \t()="
#define  _ID_STR_ERROR                "*** error *** "
#define  _ID_STR_LEN                  356

class DKEXPORT TLogUtils
{
  public:
		TLogUtils();
		TLogUtils(int argc, const char* args[]);
		TLogUtils(TLogUtils* aParent);
		virtual ~TLogUtils();

		void             deletedkCollection(dkCollection* aCol);

		void             logmsg(const DKString& aMsg, bool fAppend = false);
		void             logmsg(const DKString& aMsg1, const DKString& aMsg2, bool fAppend = false);
		void             logmsgBuff(bool fAppend = false);
		void             logmsgdkResultSetCursor(dkResultSetCursor* lCursor);
		void             logmsgDKDDO(DKDDO* aDKDDO);
		void             logmsgdkXDO(dkXDO* adkXDO);
		void             logmsgdkCollection(dkCollection* adkCollection);
		void             logmsgObject(DKAny& anObject);

		const DKString   getItemName(const ushort shItemType);
		void             takeCommand(const DKString& aCmd);
		void             processCommandLineArgs(int argc, const char* args[]);
		void             readFromStdIn();
		void             readInputFile(const DKString& anInFile);
		void             printUsage();
		void             printOtherHelp();
		void             printCommandHelp();
		const DKString   getRHS(const DKString& aCmd);
		void             setVariableValue(const DKString& aName, const DKAny& aValue);
		void             setVariableValue(const DKString& aName, const DKString&  aValue);
		DKAny            getVariableValue(const DKString& aName);
		DKNVPair*        getValuePairFor(const DKString& aName);
		void             listVariables();
		void             logmsgError(const DKString& anErrMsg);
		void             logmsgDKException(DKException& aDKExp);
		void             logerr(const DKString& anErr1, const DKString& anErr2 = "");
		void             logErrorInfo();
		void             logmsgDual(const DKString& aMsg1, const DKString& aMsg2 = "");
		void             resetErrorInfo();

		virtual void            processCommand(const DKString& aCmd);
		virtual const DKString  getAvailableCommands();

		const DKString   stringValue(const DKString& aString);
		dkCollection*    getTokens(const DKString& aCmdLine, int numSkip = 0);

		static bool      isCommand(const DKString& aString);
		static bool      equalsIgnoreCase(const DKString& aS1, const DKString& aS2);
		static char*     newCopy(const DKString& aStr, int nSize = 0);
		static const DKString   trim(const DKString& aString);
		static       char*      toUpperCase(char* aString);

	protected:
		void             setParent(TLogUtils* aParent);

		char*            getMsgBuff();
		void             changeIndentTab(bool fIncrement);
		char*            getStrIndent();
		int              getCurIndent();
		void             setCurIndent(int nIndent = 0);
		void             enableDualOut(bool fEnable = true);
		bool             isDualOut();
		void             enableAppend(bool fEnable = true);
		bool             isAppend();
		FILE*            getOutStream();
		FILE*            getErrStream();
		const char*      getInFile();
		const char*      getOutFile();
		void             setOutStream(FILE* fileOut);
		void             setOutFile(const DKString& aFile);
		dkCollection*    getCollVars();

		// Error information
		int              nErrNumber;
		DKString         strCodeBase;
		DKString         strErrDesc;
		DKString         strErrLoc;
		DKString         strErrInput;

	private:
		void             init();

		char*            _strMsgBuff;
		char*            _strIndent;
		int              _nCurIndent;

		const char*      _strInFile;
		const char*      _strOutFile;
		FILE*            _out;
		FILE*            _err;
		bool             _fDualOut;
		bool             _fAppend;
		bool             _fChild;
		dkCollection*    _collVars;


};

#endif
