/**************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/**************************************************************************/

/******************************************************************************
 *
 * Source File Name = TCallbackV4.CPP
 *
 * TCallbackV4 - a class that represents a callback for VI400
 *      
 *****************************************************************************/
#include "TCallbackV4.hpp"
#include <DKDatastoreV4.hpp>
#include "DKParts.hpp"
#include "DKFolder.hpp"
#include <stdio.h>
#include <stdlib.h>

// constructors

TCallbackV4::TCallbackV4()
{
 _threshold = 5;
 _cancel = FALSE;
 _isDone = FALSE;
 _numCalled = 0;
 _exc = NULL;
//  cout << "*** callback created " << endl;
}

// destructor
TCallbackV4::~TCallbackV4()
{
    if (_exc != NULL)
    {
     delete _exc;
    }

}

/******************************************************************************
 * setThreshold - Sets the number of returned hits for each data callback
 *****************************************************************************/

void TCallbackV4::setThreshold(long nValue)
{
 _threshold = nValue;
}

/******************************************************************************
 * getThreshold - Gets the number of returned hits that the server returns
 *****************************************************************************/

long TCallbackV4::getThreshold()
{
 return _threshold;
}

/******************************************************************************
 * cancelQuery - Indicates user has canceled the query
 *****************************************************************************/

void TCallbackV4::cancelQuery()
{
 _cancel = TRUE;
}

/******************************************************************************
 * dataCallback - Return nHits from the backend datastore
 *****************************************************************************/

DKBoolean TCallbackV4::dataCallback(dkCollection* ddoList, DKBoolean isDone)
{
 DKBoolean fDone = FALSE;
 _numCalled++;
 
 if (_cancel == TRUE || isDone == TRUE)
 {
  fDone = TRUE;
  _isDone = TRUE;
 }
 else
 {
  cout << "*** " << _numCalled << " time(s) called ***" << endl;
  processResults(ddoList);
 }
 
 delete ddoList;
 return fDone;
}

/******************************************************************************
 * reportException - Report an exception from the backend datastore
 *****************************************************************************/

void TCallbackV4::reportException(DKException* exc)
{
    if (_exc != NULL)
    {
     delete _exc;
    }

    _exc = exc;
}

/******************************************************************************
 * getException - Get an exception from the backend datastore
 *****************************************************************************/

DKException* TCallbackV4::getException()
{
 return _exc;
}

/******************************************************************************
 * resetException - Call after the exception has been deleted, this method
 * will clear the exception variable.
 *****************************************************************************/

void TCallbackV4::resetException()
{
  _exc = NULL;
}

/******************************************************************************
 * isDone - 
 *****************************************************************************/
DKBoolean TCallbackV4::isDone()
{
 return _isDone;
}


/******************************************************************************
 * processResults - 
 *****************************************************************************/
void TCallbackV4::processResults(dkCollection* results)
{
   unsigned short itemType;
   DKDDO* item;
   unsigned short i;
   DKString dataName;
   DKAny any;
   DKParts* parts;
   DKFolder* folder;

   dkIterator* iter = results->createIterator();
   DKAny *element;
   cout << "  cardinality of results: " << results->cardinality() << endl;
   
   while(iter->more()) 
   {
      element = iter->next();
      item = (DKDDO*) element->value();
      
      if (!item) continue;
      
      itemType = item->getPropertyByName(DK_PROPERTY_ITEM_TYPE);
      switch(itemType) 
      {
        case DK_DOCUMENT:
             cout << "  Item is a document, " 
                  << "number of data item: " << item->dataCount() << endl;
                  
             for (i = 1; i <= item->dataCount(); i++) 
             {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", value = "
                      << item->getData(i) << endl;
                      
                 if (dataName == DKPARTS) 
                 {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      
                      if (parts != NULL) 
                      {
                         cout << "\t this document has " << 
                              parts->cardinality()  << " parts " << endl;
                      }
                 }   
             }
             break;
        case DK_FOLDER:
             cout << " Item is a folder " << endl;
             folder = 0;
             
             for (i = 1; i <= item->dataCount(); i++) 
             {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", value = "
                      << item->getData(i) << endl;
                      
                 if (dataName == DKPARTS) 
                 {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      
                      if (parts != NULL) 
                      {
                          cout << "\t this folder has " 
                               << parts->cardinality()  << " parts " << endl;
                          // iterate here to get parts
                      }
                 }   
                 
                 if (dataName == DKFOLDER) 
                 {
                     any    = item->getData(i);
                     folder = (DKFolder*) any.value();
                     
                     if (folder != NULL) 
                     {
                        cout << "\t this folder has " 
                             << folder->cardinality() 
                             << " document/folder " << endl;
                     }
                 }   
             }
             
             if (folder && folder->cardinality() > 0) 
             { 
                 cout << "     ---------- list of folder contents -----------" << endl;
                 processResults(folder); 
                 cout << "     ----------------- end of list ----------------" << endl;
             }
             else cout << " empty folder " << endl;
             break;
        default:
             cout << " Item is not recognized " << endl;
      }     
   }
   
   delete iter;
}
