/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import com.ibm.mm.sdk.common.*;
import java.beans.*;
import java.io.*;

/**
 * This sample demonstrates some of the functions of the CMBDataManagement bean.
 */
public class DemoDataMgt {
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    CMBSchemaManagement schemaBean = null;
    CMBDataManagement dataBean = null;
    CMBSearchResults resultBean = null;
    CMBQueryService queryBean = null;
    boolean isWindows = true;
    int docCounter = 0;

    DemoDataMgt() {
        try {
            System.out.println("Creating CMBSchemaManagement bean using " + "ser/class/jar specified in the classpath.");
            schemaBean = (com.ibm.mm.beans.CMBSchemaManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBSchemaManagement");
            dataBean = (com.ibm.mm.beans.CMBDataManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBDataManagement");
            resultBean = (com.ibm.mm.beans.CMBSearchResults) Beans.instantiate(null, "com.ibm.mm.beans.CMBSearchResults");
            queryBean = (com.ibm.mm.beans.CMBQueryService) Beans.instantiate(null, "com.ibm.mm.beans.CMBQueryService");
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void changeProperty() {
        if (ui.getBoolean("Enable the data trace capability?"))
            dataBean.setTraceEnabled(true);
        else
            dataBean.setTraceEnabled(false);
    }

    void run() {
        docCounter = 0; //reset for each run
        int count;
        isWindows = ui.getBoolean("Are you currently running on Win32 platform?");

        try {
            // get server names
            System.out.println("Getting server names...");
            String svNames[] = schemaBean.getServerNames();
            count = svNames.length;
            System.out.println("Done, server count is: " + count);
            for (int i = 0; i < count; ++i)
                System.out.println("Server[" + i + "] = " + svNames[i]);
            System.out.println("");

            // get search template names
            System.out.println("Getting search template names...");
            String stNames[] = schemaBean.getSearchTemplateNames();
            count = stNames.length;
            System.out.println("Done, search template count is: " + count + "....");

            CMBSearchTemplate stObj = null;
            for (int i = 0; i < count; ++i) {
                System.out.println("Template[" + i + "] = " + stNames[i]);

                System.out.println("Starting getting each search template and its criteria ...");
                stObj = schemaBean.getSearchTemplate(i);
                System.out.println("templateName = " + stObj.getName());
            }

            boolean runQry = false;
            String stName = "";
            while (!runQry) {
                stName = ui.getString("Please enter the search template name:");
                stObj = schemaBean.getSearchTemplate(stName);
                if (stObj == null)
                    System.out.println("Unknown template name, please try again");
                else
                    runQry = true;
            }

            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i = 0; i < critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
            }

            runQry = false;
            while (!runQry) {
                System.out.println("Selections:");
                System.out.println("1. Specify a search criterion");
                System.out.println("2. Run query ...");
                switch (ui.getInteger("Enter your selection:")) {
                    case 2 :
                        runQry = true;
                        break;
                    case 1 :
                        getSearchCriterion(stObj);
                        break;
                }
            }

            System.out.println("");

            stObj.addCMBSearchReplyListener(resultBean);
            stObj.setAsynchSearch(false);
            queryBean.setAsynchSearch(false);
            resultBean.setCurrentSearchTemplate(stObj);
            queryBean.runQueryWithCursor(stObj);

            resultBean.newResults(queryBean.getResults());
            CMBItem[] itemList = resultBean.getItem();

            //get the result from CMBSearchResults bean and display them
            displayResults(itemList);
        } catch (CMBException exc) {
            Object errData = exc.getErrorData();
            if (errData != null && errData instanceof Exception)
                 ((Exception) errData).printStackTrace();
            else
                exc.printStackTrace();
        }
    }

    private void getSearchCriterion(CMBSearchTemplate stObj) {
        try {
            String critName = ui.getString("Please enter criterion name: ");
            java.util.Vector critList = stObj.getSearchCriteria();
            CMBSTCriterion critObj = null;
            for (int i = 0; i < critList.size(); i++) {
                critObj = (CMBSTCriterion) critList.elementAt(i);
                if (critObj.getName().equals(critName))
                    break;
            }

            if (critObj == null) {
                System.err.println("Invalid criterion name, please try again...");
                return;
            }

            int critOp =
                ui.getInteger(
                    "Please enter search operator: "
                        + "\nEQ: 1\t\tNEQ: 2\t\tLEQ: 3\t\tLT: 4"
                        + "\nGEQ: 5\t\tGT: 6\t\tBETWEEN: 7\tNOTBETWEEN: 8"
                        + "\nIN: 9\t\tNOTIN: 10\tLIKE: 11\tNOTLIKE: 12\n");
            switch (critOp) {
                case 1 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_EQUAL);
                    break;
                case 2 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_EQUAL);
                    break;
                case 3 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS_EQUAL);
                    break;
                case 4 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS);
                    break;
                case 5 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER_EQUAL);
                    break;
                case 6 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER);
                    break;
                case 7 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_BETWEEN);
                    break;
                case 8 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_BETWEEN);
                    break;
                case 9 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_IN);
                    break;
                case 10 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_IN);
                    break;
                case 11 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LIKE);
                    break;
                case 12 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_LIKE);
                    break;
                default :
                    break;
            }
            if (critOp == 7 || critOp == 8) {
                String critVal1 = ui.getString("Please enter 1st search value: ");
                String critVal2 = ui.getString("Please enter 2nd search value: ");
                String[] critVals = new String[2];
                critVals[0] = critVal1;
                critVals[1] = critVal2;
                critObj.setValues(critVals);
            } else if (critOp == 9 || critOp == 10) {
                java.util.Vector vals = new java.util.Vector();
                boolean notDone = true;
                while (notDone) {
                    String val = ui.getString("Please enter value: (-1 if done entering)");
                    if (val.equals("-1"))
                        notDone = false;
                    else
                        vals.addElement(val);
                }
                String[] critVals = new String[vals.size()];
                vals.copyInto(critVals);
                critObj.setValues(critVals);
            } else {
                String critVal = ui.getString("Please enter search value: ");
                critObj.setValue(critVal);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    //Display all attribute values for the result list.
    //Also, pick the first item and display its data to either an editor
    //or in the browser.
    private void displayResults(CMBItem[] itemList) {
        if (itemList == null || itemList.length == 0) {
            System.out.println("THere is no 0 hit for the current query");
            return;
        }

        String path = "";
        String fullPath = "";
        if (isWindows) {
            path = "c:\\";
        } else
            path = "/tmp/";

        int nHits = itemList.length;
        for (int k = 0; k < nHits; k++) {
            System.out.println("getting item #" + k);
            CMBItem hit = (CMBItem) itemList[k];
            if (hit != null) {
                System.out.println("Hit item #" + k + ", pidString=" + hit.getPidString());

                try {
                    String[] values = hit.getAttrValues();
                    String[] names = hit.getAttrNames();
                    for (int l = 0; l < values.length; l++)
                        System.out.println("\t" + names[l] + ": " + values[l]);
                    if (hit.getServerType().equals(CMBBaseConstant.CMB_DSTYPE_DL) && docCounter < 1) {
                        docCounter++;
                        dataBean.setDataObject(hit);
                        CMBObject[] objs = dataBean.getContent();
                        for (int x = 0; x < objs.length && x < 5; x++) {
                            CMBObject content = objs[x];
                            byte[] data = content.getData();
                            String mimeType = content.getMimeType();
                            System.out.println("mimeType = " + mimeType);
                            String fileName = "";
                            String prog = "xv";
                            boolean isDisplayable = false;
                            if (mimeType.equals("image/gif")) {
                                fileName = "Dat" + System.currentTimeMillis() + ".gif";
                                fullPath = "file:///C|/" + fileName;
                                if (isWindows)
                                    prog = "C:\\Program Files\\Netscape\\Communicator\\Program\\netscape.exe";
                                isDisplayable = true;
                            } else if (mimeType.equals("text/plain")) {
                                fileName = "DAT" + System.currentTimeMillis() + ".txt";
                                fullPath = path + fileName;
                                if (isWindows)
                                    prog = "C:\\WINNT\\system32\\notepad.exe";
                                else
                                    prog = "editor";
                                isDisplayable = true;
                            } else
                                fileName = "DAT" + System.currentTimeMillis() + ".dat";
                            System.out.println("fileName=" + fileName);

                            if (isDisplayable) {
                                DataOutputStream ostream = new DataOutputStream(new FileOutputStream(path + fileName));
                                ostream.write(data, 0, data.length);
                                Runtime rt = Runtime.getRuntime();
                                Process p = null;
                                String cmd = new String(prog);
                                cmd = cmd + " " + fullPath;
                                try {
                                    //System.out.println("syn exc cmd = " + cmd);
                                    p = rt.exec(cmd);
                                    p.waitFor();
                                } catch (Exception e1) {
                                    e1.printStackTrace();
                                }
                                File wfile = new File(path + fileName);
                                if (wfile.exists())
                                    wfile.delete();
                                wfile = null;
                            }
                        }
                    }
                } catch (CMBException ex) {
                    ex.printStackTrace();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoDataMgt";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoConnect connDemo = new DemoConnect();
        DemoDataMgt demo = new DemoDataMgt();
        boolean loopControl = true;

        System.out.println("Going to connect to the server...");
        connDemo.connect();

        //always set connection to all the beans before using
        demo.schemaBean.setConnection(connDemo.connBean);
        demo.dataBean.setConnection(connDemo.connBean);
        demo.resultBean.setConnection(connDemo.connBean);
        demo.queryBean.setConnection(connDemo.connBean);

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. Change property values");
            System.out.println("2. Run the bean");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    demo.changeProperty();
                    break;
                case 2 :
                    demo.run();
                    break;
            }
        }

        connDemo.disconnect();
        System.out.println("Bye.");
    }
}