/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * Connects and disconnects from a server using the CMBConnection bean.
 */
public class TConnect {

    public static void main(String[] args) throws Exception {
        if (args.length < 4) {
            System.out.println(
                "Connects and disconnects from server. \n"
                    + "Syntax: \n"
                    + "  java TConnect <dstype> <server> <userid> <password> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "Examples: \n"
                    + "  java TConnect ICM icmnlsdb icmadmin password \n"
                    + "  java TConnect DB2 sample db2admin password \n"
                    + "  java TConnect Fed cmbdb cmbadmin password \n"
                    + "  java TConnect OD 9.112.20.3(ODPORT=2190) userid password");
           System.exit(0);
        }
        
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        
        CMBConnection connection = new CMBConnection();
        
        connect(connection, dstype, server, userid, password);
        disconnect(connection);
    }

    /**
     * Connects to a given server.
     * @param connection an instance of CMBConnection bean
     * @param dstype the datastore type.
     * @param server the server name.  A connection string can also be specified following the name, in parenthesis
     * @param userid the userid.
     * @param password the password.
     */
    public static void connect(CMBConnection connection, String dstype, String server, String userid, String password)
        throws Exception {
        
        // If the server name is followed by a parenthesized string,
        // use that string for the connect string.
        if (server.indexOf("(") > 0) {
            String connectString = server.substring(server.indexOf("(") + 1);
            server = server.substring(0, server.indexOf("("));
            if (connectString.endsWith(")")) {
                connectString = connectString.substring(0, connectString.length() - 1);
            }
            connection.setConnectString(connectString);
        }
        
        // Set properties on connection bean
        connection.setDsType(dstype);
        connection.setServerName(server);
        connection.setUserid(userid);
        connection.setPassword(password);

		// Use the connect method to connect.  This will
		// create the correct type of DKDatastore object (see Java API)
		// and call its connect method.
        System.out.println("Connecting to server");
        connection.connect();
    }

    /**
     * Disconnects from the server.
     * @param connection an instance of CMBConnection bean
     */
    public static void disconnect(CMBConnection connection) throws Exception {
        System.out.println("Disconnecting from server");
        
        // Use disconnect method to disconnect.  Note that calling
        // this method disconnects and destroys the underlying DKDatastore object.
        connection.disconnect();
    }

}