/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import java.util.*;

/**
 * Edits attributes of one or more items.  The list of items is chosen by searching.
 * The attributes to change are specified using name=value pairs.
 */
public class TEditAttributes {

    public static void main(String args[]) throws Exception {
        if (args.length < 6) {
            System.out.print(
                "Syntax: \n"
                    + "  java TEditAttributes <dstype> <server> <userid> <password> <entity> <condition> / <attrs> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <condition> is a search condition. \n"
                    + "  <attrs> are name=value syntax specifying the attributes. \n"
                    + "The condition and attributes are separated with a slash. \n"
                    + "Special syntax: \n"
                    + "  timestamp -- can be used as an attribute value and is substituted with real timestamp. \n"
                    + "Examples: \n"
                    + "  java TEditAttributes Fed cmbdb cmbadmin password Claim LastName LIKE \"Smith\" / LastName='Jones' \n"
                    + "  java TEditAttributes DB2 sample db2admin password EMP_PHOTO PHOTO_FORMAT LIKE 'gif' / PHOTO_FORMAT='image/gif' \n"
                    + "  java TEditAttributes DL libsrvrn frnadmin password NOINDEX Userid LIKE 'owings' / Userid='bowings' \n"
                    + "  java TEditAttributes ICM icmnlsdb icmadmin password Book @Title LIKE \"My Java Book\" / BkAuthor(FirstName='John' LastName='Smith') \n"
                    + "  java TEditAttributes ICM icmnlsdb icmadmin password Book @Title LIKE \"My Java Book\" / + BkAuthor(FirstName='Sam' LastName='Jones') \n"
                    + "");
            return;
        }

        // Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        String attributes = "";
        boolean doneWithCondition = false;
        for (int i = 5; i < args.length; i++) {
            if (!doneWithCondition) {
                if (args[i].equals("/")) {
                    doneWithCondition = true;
                } else {
                    condition = condition + " " + args[i];
                }
            } else {
                attributes = attributes + " " + args[i];
            }
        }

        // Create beans.
        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample)
        TConnect.connect(connection, dstype, server, userid, password);

        // Search to find items to edit
        TSearch.search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");

        // Edit the attributes of the items found
        for (int i = 0; i < searchResults.getCount(); i++) {
            CMBItem item = searchResults.getItem(i);
            updateAttributes(schemaManagement, dataManagement, item, attributes);
        }

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

    } // main

	/**
	 * Updates attributes of an item.
	 * @param schemaManagement an instance of CMBSchemaManagement bean.
	 * @param dataManagement an instance of CMBDataManagement bean.
	 * @param item an instance of CMBItem for the item to be edited.
	 * @param attribute a string containing the new attributes, in the form name=value name=value ...
	 *        Names can be compound names, describing the child component and the attribute
	 *        within the child component, separated with ".".
	 */
    public static void updateAttributes(
        CMBSchemaManagement schemaManagement,
        CMBDataManagement dataManagement,
        CMBItem item,
        String attributes)
        throws CMBNoConnectionException, CMBException {
        System.out.println("Updating attributes of item " + item.getName());
        CMBEntity entityBean = schemaManagement.getEntity(item.getEntityName());

        // Parse up the attrs string, setting attribute values on item
        StringTokenizer tokenizer = new StringTokenizer(attributes, "+-='() ", true);
        String name = "";
        short type = 0;
        String value = "";
        boolean invalue = false;
        boolean instring = false;
        boolean attrready = false;
        while (tokenizer.hasMoreTokens()) {
            String token = tokenizer.nextToken();
            if (token.equals("=")) {
                invalue = true;
            } else if (token.equals("'")) {
                if (instring) {
                    instring = false;
                    if (invalue) {
                        invalue = false;
                        attrready = true;
                    }
                } else {
                    instring = true;
                }
            } else if (token.equals(" ")) {
                if (instring) {
                    if (invalue) {
                        value = value + " ";
                    } else {
                        name = name + " ";
                    }
                }
            } else {
                if (instring) {
                    if (invalue) {
                        value = value + token;
                    } else {
                        name = name + token;
                    }
                } else if (invalue) {
                    value = token;
                    invalue = false;
                    attrready = true;
                } else {
                    name = token;
                }
            }
            if (attrready) {
                System.out.println("  Name:  " + name);
                if (value.equals("timestamp")) {
                    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat("yyyy-MM-dd-HH.mm.ss.SSSSSS");
                    Date currentTime = new Date();
                    String dateString = formatter.format(currentTime);
                    if (dateString.length() == 23)
                        dateString = dateString + "000";
                    value = dateString;
                }
                System.out.println("  Value: " + value);
                setItemAttr(entityBean, item, name, value);
                attrready = false;
                name = "";
                value = "";
            }
        }

        // Update the item on the server
        dataManagement.setDataObject(item);
		if (dataManagement.isSupported("checkOut")) { // some servers require checkout
			dataManagement.checkOut();
		}
        dataManagement.updateItem();
		if (dataManagement.isSupported("checkIn")) {
			dataManagement.checkIn();
		}

    } // updateAttributes
    
	/**
	 * Set an attribute of an item.
	 * @param entityBean instance of CMBEntity for the item
	 * @param item instance of CMBItem for the item
	 * @param name name of attribute to set
	 * @param value value of attribute to set
	 */
    public static void setItemAttr(CMBEntity entityBean, CMBItem item, String name, String value) throws CMBException {

        if (name.indexOf(".") >= 0) {
            setSubItemAttr(entityBean, item, name, value);
        } else {
            String[] attrNames = entityBean.getAttributeNames();
            short[] attrTypes = entityBean.getAttributeTypes();
            boolean valueSet = false;
            for (int i = 0; i < attrNames.length; i++) {
                if (attrNames[i].equalsIgnoreCase(name)) {
                    try {
                        item.setAttrValue(name, value);
                    } catch (CMBException e) { // thrown if attribute doesn't exist
                        item.addAttr(attrNames[i], value, attrTypes[i]);
                    }
                    valueSet = true;
                }
            }
            if (!valueSet) {
                throw new IllegalArgumentException("Attribute specified is not an attribute of the item: " + name);
            }
        }
    } // setItemAttr

	/**
	 * Set an attribute of a child component.
	 * @param entityBean instance of CMBEntity for the child component type
	 * @param item instance of CMBItem for the parent component
	 * @param name name of attribute to set
	 * @param value value of attribute to set
	 */
    static void setSubItemAttr(CMBEntity entityBean, CMBItem item, String name, String value) throws CMBException {

        String[] attrNames = entityBean.getAttributeNames();
        short[] attrTypes = entityBean.getAttributeTypes();
        String collectionName = name.substring(0, name.indexOf("."));

        // find the subentity for the collection attribute
        CMBEntity[] subEntities = entityBean.getSubEntities();
        CMBEntity subEntityBean = null;
        for (int i = 0; i < subEntities.length; i++) {
            if (subEntities[i].getName().equalsIgnoreCase(collectionName)) {
                subEntityBean = subEntities[i];
            }
        }
        if (subEntityBean == null) {
            throw new IllegalArgumentException("Attribute specified is not a collection attribute of the item: " + collectionName);
        }

        // Find the collection attribute, or create one if need be
        // Note: This logic only handles a single child component in the collection.
        CMBItem[] collection = null;
        try {
            collection = item.getAttrItems(collectionName);
        } catch (CMBException e) { // thrown if item does not exist
        }
        if (collection == null) {
            collection = new CMBItem[1];
            collection[0] = new CMBItem();
            collection[0].setServerName(item.getServerName());
            collection[0].setServerType(item.getServerType());
            collection[0].setEntityName(collectionName);
            item.addAttr(collectionName, collection, CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION);
        }

        // Set the appropriate attribute value in the subitem
        String attrName = name.substring(name.indexOf(".") + 1);
        setItemAttr(subEntityBean, collection[0], attrName, value);

    } // setSubItemAttr

}