/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import java.util.*;

/**
 * Perform a search using EIP JavaBeans.  Also, displays details on items found.  
 * Details include attributes, child components, folder contents, parts
 */
public class TSearch {
	
    public static void main(String[] args) throws Exception {
        if (args.length < 5) {
            System.out.println(
                "Searches a server for items, using a query string. \n"
                    + "Syntax: \n"
                    + "  java TSearch <dstype> <server> <userid> <password> <entity> <condition> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, table) to search. \n"
                    + "  <condition> the condition portion of the query string.  The syntax \n"
                    + "              of the condition varies depending on server type. \n"
                    + "Examples: \n"
                    + "  java TSearch ICM icmnlsdb icmadmin password Book @Title LIKE \\\"%\\\" \n"
                    + "  java TSearch DL libsrvrn frnadmin password NOINDEX Userid LIKE '%' \n"
                    + "  java TSearch Fed cmbdb cmbadmin password NoIndex Userid LIKE '%' \n"
                    + "  java TSearch DB2 sample db2admin password EMP_PHOTO PHOTO_FORMAT LIKE 'gif' \n"
                    + "  java TSearch OD odserver userid password Balance CustName like '%BLONDIE%'");
            System.exit(0);
        }

        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        for (int i = 5; i < args.length; i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

        // Search and display details on all hits
        search(queryService, dstype, entity, condition, false);
        searchResults.newResults(queryService.getResults());
        System.out.println("There are " + searchResults.getCount() + " items found.");
        for (int i = 0; i < searchResults.getCount(); i++) {
            CMBItem item = searchResults.getItem(i);
            System.out.println("  Item[" + i + "] = " + item.getName());
            printItemDetails(dataManagement, item, 1, true);
        }

        // Disconnect (using TConnect sample)
        TConnect.disconnect(connection);

    }

    /**
     * Create a query string and run a search using the query service bean.
     * @param queryService an instance of CMBQueryService bean.
     * @param dstype the datastore type (ICM, Fed, DL, OD, etc)
     * @param entity the name of the type of entity to search on the server
     * @param condition the condition part of the query string
     * @param asynchSearch if true, search is performed asynchronously
     */
    public static void search(
        CMBQueryService queryService,
        String dstype,
        String entity,
        String condition,
        boolean asynchSearch)
        throws CMBInvalidQueryException, CMBConnectFailedException, CMBException {

        // Determine the query type and formulate the query string for
        // the given dstype, entity, and condition.  The query string
        // syntax differs by datastore type.  Also, some datastores
        // specify the entity using a query parameter.
        short queryType;
        String queryString;
        CMBQueryService.QueryParameter[] queryParams;
        if (dstype.equals("Fed")) {
            queryType = CMBBaseConstant.CMB_QS_TYPE_FEDERATED;
            queryString = "PARAMETRIC_SEARCH=(ENTITY=" + entity;
            if (condition.length() > 0) {
                queryString += ",COND=(" + condition + ")";
            }
            queryString += ")";
            queryParams = null;
        } else if (dstype.equals("DL") || dstype.equals("V4")) {
            queryType = CMBBaseConstant.CMB_QS_TYPE_PARAMETRIC;
            queryString = "SEARCH=(INDEX_CLASS=" + entity;
            if (condition.length() > 0) {
                queryString += ",COND=(" + condition + ")";
            }
            queryString += ")";
            queryParams = null;
        } else if (dstype.equals("OD")) {
            queryType = CMBBaseConstant.CMB_QS_TYPE_SQL;
            queryString = "";
            if (condition.length() > 0) {
                queryString += " WHERE " + condition;
            }
            queryParams = new CMBQueryService.QueryParameter[2];
            queryParams[0] = queryService.new QueryParameter("APPL_GROUP", entity);
            queryParams[1] = queryService.new QueryParameter("CONTENT", "ATTRONLY");
        } else if (dstype.equals("ICM")) {
            queryType = CMBBaseConstant.CMB_QS_TYPE_XPATH;
            queryString = "/" + entity;
            if (condition.length() > 0) {
                queryString += "[" + condition + "]";
            }
            queryParams = null;
        } else {
            queryType = CMBBaseConstant.CMB_QS_TYPE_SQL;
            queryString = "SELECT * FROM " + entity;
            if (condition.length() > 0) {
                queryString += " WHERE " + condition;
            }
            queryParams = null;
        }
        
        // Set properties on the query service bean
        System.out.println(queryString);
        if (queryParams != null) {
            queryService.setQueryString(queryString, queryType, queryParams);
        } else {
            queryService.setQueryString(queryString, queryType);
        }
        queryService.setAsynchSearch(asynchSearch);

        // Perform the search using the runQuery method
        System.out.println("Performing search");
        queryService.runQuery();
    }

    /**
     * Print information about an item in the search results.
     * This will also print child components and folder contents.
     * @param dataManagement an instance of CMBDataManagement bean
     * @param item an instance of CMBItem for the item
     * @param indent amount to indent printed lines
     * @param withExtraInfo if true, additional information about parts, and other details not normally displayed in search results is displayed
     */
    public static void printItemDetails(CMBDataManagement dataManagement, CMBItem item, int indent, boolean withExtraInfo)
        throws CMBException, Exception {
        
        String space = "";
        for (int i = 0; i < indent; i++) {
            space = space + "  ";
        }

        // Print attribute values.  This will recurse for any attribute values
        // that are item collections (e.g. child components)
        String[] names = item.getAttrNames();
        short[] types = item.getAttrTypes();
        String[] values = item.getAttrValues();
        for (int i = 0; i < names.length; i++) {
            if (types[i] == CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION) {
                CMBItem[] subitems = item.getAttrItems(names[i]);
                System.out.println(space + "  " + names[i] + ": Collection of " + subitems.length + " items:");
                for (int j = 0; j < subitems.length; j++) {
                    System.out.println(space + "    Subitem[" + j + "] = " + subitems[j].getName());
                    printItemDetails(dataManagement, subitems[j], indent + 2, false);
                }
            } else {
                System.out.println(space + "  " + names[i] + ": " + values[i] + "  ");
            }
        }

        // Print the type of item.
        if (item.getItemType() == CMBItem.CMB_TYPE_FOLDER) {
        	System.out.println(space + "  Item is a folder.");
        } else if (item.getItemType() == CMBItem.CMB_TYPE_DOCUMENT) {
            System.out.println(space + "  Item is a document.");
        } else if (item.getItemType() == CMBItem.CMB_TYPE_ITEM) {
            System.out.println(space + "  Item is not a document or folder.");
        } else {
            System.out.println(space + "  Item type is unknown.");
        }

        // Print extra detailed information about the document.
        if (withExtraInfo) {
	        dataManagement.setDataObject(item);

	        // Print if the item is checked-out.
	        if (dataManagement.isSupported("isCheckedOut")) {
	            if (dataManagement.isCheckedOut()) {
	                System.out.println(space + "  Item is checked out by " + dataManagement.getLockedBy());
	            } else {
	                System.out.println(space + "  Item is not checked out.");
	            }            	
	        }
            
            // Content parts.  These are represented with CMBObject objects.
            int contentCount = dataManagement.getContentCount();
            System.out.println(space + "  There are " + contentCount + " content parts.");
            for (int i = 0; i < contentCount; i++) {
                System.out.print(space + "    Content Part[" + i + "] =");
                CMBObject part = dataManagement.getContent(i);
                System.out.print(" length: " + part.getSize());
                System.out.print(" mimetype: " + part.getMimeType());
                // Use part.getData() here to retrieve the data (as a byte array)
                System.out.println();
            }
            
            // Annotation parts.  These are represented with CMBAnnotation objects,
            // although complex annotation data is represented with CMBObject object.
            int annotationCount = dataManagement.getAnnotationCount();
            System.out.println(space + "  There are " + annotationCount + " annotation parts.");
            for (int i = 0; i < annotationCount; i++) {
                System.out.print(space + "    Annotation Part[" + i + "] =");
                CMBAnnotation annotation = dataManagement.getAnnotation(i);
                // Note: Depending on the origin of the annotation, not all
                //       properties may be filled in or used.
                System.out.print(" page: " + annotation.getPageNumber());
                System.out.print(" x: " + annotation.getX());
                System.out.print(" y: " + annotation.getY());
                System.out.print(" color: " + annotation.getColor());
                if (annotation.getAnnotationType() == CMBAnnotation.CMB_ANNOTATION_TYPE_STRING) {
                    System.out.print(" text: " + annotation.getStringData());
                } else {
                    System.out.print(" annotation type is binary");
                }
                CMBObject part = annotation.getComplexData();
                if (part != null) {
	                System.out.print(" length: " + part.getSize());
 	                System.out.print(" mimetype: " + part.getMimeType());
  	 	            // Use part.getData() here to retrieve the data (as a byte array)
                }
                System.out.println();
            }
            
            // Notelog parts.  These are represented with CMBObject objects.
            int notelogCount = dataManagement.getNoteLogCount();
            System.out.println(space + "  There are " + notelogCount + " notelog parts.");
            for (int i = 0; i < notelogCount; i++) {
                System.out.print(space + "    Notelog Part[" + i + "] =");
                CMBObject part = dataManagement.getNoteLog(i);
                System.out.print(" length: " + part.getSize());
                System.out.print(" mimetype: " + part.getMimeType());
                // Use part.getData() here to retrieve the data (as a byte array)
                System.out.println();
            }
		
			// Folder contents            
	        if (item.getItemType() == CMBItem.CMB_TYPE_FOLDER) {
	            Vector vector = item.listFolderItems();
	            System.out.println(space + "  Folder contains " + vector.size() + " items:");
	            for (int j = 0; j < vector.size(); j++) {
	                CMBItem subitem = item.getFolderItem(j);
	                System.out.print(space + "    ");
	                try {
		                System.out.print(subitem.getName() + " [ ");
		                names = subitem.getAttrNames();
		                values = subitem.getAttrValues();
		                for (int k = 0; k < names.length; k++) {
		                    System.out.print(names[k] + ": " + values[k] + "  ");
		                }
		                System.out.println(" ]");
	                } catch (CMBException e) {
	                	System.out.println("Error getting item attributes, item="+subitem);
	                }
	            }
	        }
            
        }
    }

}