/*
 * @copyright(disclaimer) 
 * Licensed Materials - Property of IBM
 * IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
 * 
 * US Government Users Restricted Rights                                  
 *  Use, duplication or disclosure restricted by GSA ADP Schedule          
 *  Contract with IBM Corp.     
 * 
 * DISCLAIMER OF WARRANTIES :                                             
 *                                                                        
 * Permission is granted to copy and modify this  Sample code, and to           
 * distribute modified versions provided that both the copyright        
 * notice,- and this permission notice and warranty disclaimer  appear in all copies and modified  
 * versions. 
 * 
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. 
 * IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM     
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN 
 * SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF 
 * NON-INFRINGEMENT AND THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 * PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
 * SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT 
 * OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, 
 * DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION 
 * OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO 
 * EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE 
 * LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, 
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED 
 * AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF 
 * IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGES.                                     
 *                                                                  
 * @endCopyright
 */
import java.awt.*;
import java.awt.geom.*;   
import java.util.*;
import javax.swing.*;


/******************************************************************************
 * Modifications                                                              *
 *                                                                            *
 *  Date  Number  Name  Description                                           *
 * ------ ------ ------ ------------------------------------------------------*
 * 033001         jgui  Create a generic MenuBar that can be added a banner   *
 * 041601         cmh   Copy code to make a ToolBar with a banner             *
 * 042001         cmh   Modify code to handle BiDi                            *
 * 042401         cmh   Modify to make ToolBar with bottom-up gradient        *
 ******************************************************************************/


/*****************************************************************************************
 * Discription : 
 *
 * This class was created for setting a banner(an image) to any normal JToolBar. The 
 * banner and the gradient background will keep the same L & F when resize the UI frame. 
 * Usage: Change the JToolBar to BannerToolBar and use it as normal JMenuBar
 *        1, SajCmnBannerToolBar _menuBar = new BannerToolBar(banner);
 *        2, SajCmnBannerToolBar _menuBar = new BannerToolBar(banner,color,150,100);
 *        3, SajCmnBannerToolBar _menuBar = new BannerToolBar();
 *           _toolBar.setProductBanner(banner);
 ******************************************************************************************/
 
public class BannerToolBar extends JToolBar 
{  
    // Do not remove - needed for copyright protection
    private static String COPYRIGHT = ClientPane.IBM_LONG_COPYRIGHT;
    boolean ltr = ClientPane.isLeftToRight();
    private Image productBanner = null;        //default case: there is no image
    private Color gradientColor = new Color(153, 162,177); //default gradient color 
    private float startPosition  = -1 ;    // no setting. will paint from 1/3 width position 
                                       //It is better to get 1/3 width when paint because of resizing
	private float rightDistance  = 0;     // Default case: paint to the end. 
	
	public BannerToolBar()
	{
		super();
		gradientColor = getBackground().darker();
	}
	
	public BannerToolBar(Image newBanner)
	{   
	    super();
	    productBanner = newBanner;
	}
	
	public BannerToolBar(Color newColor, float startFromHere, float distance)
	{
	    super();
	    //Need to take care of "null"
	    //1,Use default value. 2, Eliminate possible exception for painting null image with null color
	    if(newColor  != null)
	        gradientColor = newColor;
	    if(startFromHere >= 0 )
	        startPosition = startFromHere;
	    if(distance >= 0 )
	        rightDistance = distance;
	}
	
	   
    /***********************************************************
     * Set product banner   
     * paintComponent() will pick it up to paint it in MenuBar
     * *********************************************************/ 
     public void setProductBanner(Image newImage)
     {
        productBanner = newImage;
     }

     /***********************************************************
      * Get product banner (an Image object)  
      * *********************************************************/ 
      public Image getProductBanner()
      {
         return productBanner ;
      }

     /*************************************************************************
      * Set gradient color for the MenuBar.   
      * This will change the default gradient color(dark side)provided by UCD
      * ***********************************************************************/ 
      public void setGradientColor(Color newColor)
      {
         gradientColor=newColor;
      }


     /*************************************************************************
      * Get gradient color for the MenuBar.   
      * ***********************************************************************/ 
      public Color getGradientColor()
      {
         return gradientColor ;
      }



     /*************************************************************************
      * Set start Postion from where we start to paint the gradient color.   
      * ***********************************************************************/ 
      public void setStartPosition(float startFromHere)
      {
         startPosition = startFromHere ;
      }

     
      public float getStartPosition()
      {
         return startPosition ;
      }

     /*****************************************************************************
      * set the end Postion (measured from the right bound) of the gradient color.   
      * ***************************************************************************/ 
      public void setRightDistance(float distance)
      {
         rightDistance = distance ;
      }

     /*****************************************************************************
      * Get the end Postion (measured from the right bound) of the gradient color. 
      * ***************************************************************************/ 
      public float getRightDistance()
      {
         return rightDistance ;
      }



     /***********************************************************
      * This function is used to paint the product banner 
      * and the gradient background.
      * *********************************************************/ 
	  protected void paintComponent(Graphics g)
	  {   
	        	    
		    if(startPosition == -1)  {// no setting 
		        if (ltr)  {
		            startPosition = (float) (this.getWidth()/3); //default:from 1/3 width
		        }
		        else {
		            // BiDi, start from left & paint to right
		            startPosition = 0;
		            if (rightDistance == 0) // default setting
		                rightDistance = (float) (this.getWidth()/3);
		        }
		    }
	
		    //Paint the component.   
			if (gradientColor != null) {
	            Graphics2D g2 = (Graphics2D)g;
	            GradientPaint gp = null;
	        if (ltr) {
	            // right-to-left gradient
	            gp = new GradientPaint(startPosition, 0, this.getBackground(),
					(float)(this.getWidth()-rightDistance), (float)this.getHeight(), gradientColor );
			}
		    else {
		        // left-to-right gradient
	            gp = new GradientPaint(startPosition, 0, gradientColor,
					rightDistance, (float)this.getHeight(), this.getBackground() );
			}
			    // Bottom-up gradient
			    //gp = new GradientPaint(0,this.getHeight()/3,this.getBackground(), 0, this.getHeight(),gradientColor);
	            g2.setPaint(gp);
	            Rectangle2D rect = new Rectangle2D.Double(0, 0, getWidth(), getHeight());
	    	    g2.fill(rect);
	    	} else {
	    	    super.paintComponent(g);
	    	}
	    	//Paint product Banner when it exists.
	        if(productBanner != null)
	        {
	          	ImageIcon bannerIcon = null;
	    	    bannerIcon =new ImageIcon(productBanner);
	    	    if (ltr) {
	    	        bannerIcon.paintIcon(this, g, getWidth() - bannerIcon.getIconWidth(), 0);
	    	    }
	    	    else {
	    	        // BiDi - start banner from left
	    	        bannerIcon.paintIcon(this, g, 0, 0);
	    	    }
			}
	    	
			
		}  // End of paintComponent

	    
} //End of BannerMenuBar class	    
