/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

 // Perform create and delete Text Search Index
 /*
  * Filename       : TCreateDelIndexTS.java
  * Description    : Perform index creation/deletion functions
  */


import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TCreateDelIndexTS class
public class TCreateDelIndexTS implements DKConstantDL {

  // Main method
  public static void main(String argv[]){
    
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 2){
          System.out.println("Invoke Syntax: " );
          System.out.println("  java TCreateDelIndexTS <srchSrv> <srchIndex>" );
          return;
      }
      
      String srchSrv  = argv[0];
      String strIndex = argv[1];
       
    DKDatastoreTS dsTS = null;
    try {
      DKSearchIndexDefTS pEnt = null;
      DKSearchIndexDefTS pEntOut = null;
      int iIndexType = 0;
      String strIndexType = "";
      String strIndexName = "";
      int iIndexADD = 0;
      String strIndexADD = "";
      dsTS = new DKDatastoreTS();
      DKDatastoreDefTS dsDef = null;

      System.out.println("connecting to datastore"); 
      dsTS.connect(srchSrv,"","","");
      System.out.println("datastore connected srchSrv " + srchSrv); 
      
      dsDef = (DKDatastoreDefTS)dsTS.datastoreDef();
      pEnt = new DKSearchIndexDefTS((dkDatastore)dsTS);
      System.out.println(" create index " + strIndex);
      
      pEnt.setName(strIndex);
      pEnt.setIndexType(DK_TS_INDEX_TYPE_PRECISE);
      pEnt.setLibraryId("LIBSUM");
      pEnt.setLibraryClientServices("IMLLSCDL");
      pEnt.setLibraryServerServices("IMLLSSDL");
      String strDefaultDir = "e:\\tsindex\\";
      String strIndexFileDir = strDefaultDir + "index\\" + strIndex;
      pEnt.setIndexDataArea(strIndexFileDir);
      String strWorkFileDir = strDefaultDir + "work\\" + strIndex;
      pEnt.setIndexWorkArea(strWorkFileDir);
      dsDef.add((dkEntityDef)pEnt);
      System.out.println(" created index " + strIndex);
      System.out.println(" get index info " + strIndex );

      pEntOut = (DKSearchIndexDefTS)dsDef.getEntity(strIndex);
      System.out.println("*Index name " + pEntOut.getName());
      iIndexType = pEntOut.getType();
      if (iIndexType == DK_TS_INDEX_TYPE_LINGUISTIC)
    	  strIndexType = "LINGUISTIC";
      else if (iIndexType == DK_TS_INDEX_TYPE_PRECISE)
    	  strIndexType = "PRECISE";
      else if (iIndexType == DK_TS_INDEX_TYPE_GTR)
          strIndexType = "GTR";
      System.out.println("*Index type " + strIndexType);
      iIndexADD = pEntOut.getIndexFeature();
      
      if (iIndexADD == DK_TS_INDEX_FEATURE_CASE_ENABLED)
       	  strIndexADD = "CASE_ENABLED";
      if (iIndexADD != 0)
       	  System.out.println("*Index feature " + strIndexADD);
      
      System.out.println("*Index ccsid " + pEntOut.getIndexCCSID());
      System.out.println("*Index libid " + pEntOut.getLibraryId());
      System.out.println("*Index LSCE " + pEntOut.getLibraryClientServices());
      System.out.println("*Index LSSE " + pEntOut.getLibraryClientServices());
      System.out.println("*Index DataArea " + pEntOut.getIndexDataArea());
      System.out.println("*Index WorkArea " + pEntOut.getIndexWorkArea());
      System.out.println("*Index Title " + pEntOut.getIndexTitle());
      System.out.println("*Index Title CCSID " + pEntOut.getIndexTitleCCSID());
      System.out.println("*Index Desc " + pEntOut.getIndexDescription());
      System.out.println("*Index Desc CCSID " + pEntOut.getIndexDescriptionCCSID());
      dsTS.disconnect();
      System.out.println("datastore disconnected"); 
      dsTS.destroy();
    }
    catch (DKException exc){
     try {
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
    catch (Exception exc){
     try {
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }
}

