/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TxdoAsyncRetDL.java
 * Description    : test the asynchronous retrieval of Blob.
 *
 * This test case requires you to provide a existing itemId & partId, it will 
 * use regular retrieve() and save the content, then do retrieveAsync(),
 * retrieveAsyncNext() to retrieve content and save the content.  Compare
 * both content to verify both content are the same.
 *
 */
// replace 'import com.ibm.mm.sdk.server.*' with 'import com.ibm.mm.sdk.client.*' 
// for client testing
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.internal.*;
import java.io.*;
import java.lang.*;

public class TxdoAsyncRetDL
       implements DKConstantDL 
{
  // Main method
  public static void main(String[] args)
  {
    //replace following with your library server name, userid, pasword
    String STR_LIB_SERVER = "LIBSRVRH";
    String STR_USER_ID    = "FRNADMIN";
    String STR_PASSWORD   = "PASSWORD";

    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    int    partId = 1;
    String itemId = "RWNCRQSLE$KWDJH0";
    String repType = "FRN$NULL";
    int size = 0,         //object total size
	readLen = 1000,   //read length
	lenLeft = 0;      //length left to be retrieved
    String content1 = "",
	   content2 = ""; 
    int i = 0;

    if (args.length == 3)
    {
      partId = (int)Integer.parseInt(args[0], 10);
      repType = args[1];
      itemId = args[2];
      System.out.println("You enter: java TxdoAsyncRetDL " +
      partId + " " + repType + " " + itemId);
    }
    if (args.length == 2)
    {
      partId =(int)Integer.parseInt(args[0], 10);
      repType = args[1];
      System.out.println("You enter: java TxdoAsyncRetDL " +
      partId + " " + repType );
      System.out.println("The supplied default itemId = " + itemId);
    }
    if (args.length == 1)
    {
      partId =(int)Integer.parseInt(args[0], 10);
      System.out.println("You enter: java TxdoAsyncRetDL " + partId);
      System.out.println("The supplied default repType = " + repType);
      System.out.println("The supplied default itemId = " + itemId);
    }
    if (args.length == 0)
    {
      System.out.println("invoke: java TxdoAsyncRetDL <fileName> <partId> <repType> <itemId>");
      System.out.println("No parameter, following defaults will be provided:");
      System.out.println("     default partId = " + partId);
      System.out.println("     default repType = " + repType);
      System.out.println("     default itemId = " + itemId);
    }
    DKDatastoreDL dsDL = null;
    try {
    dsDL = new DKDatastoreDL();  
    System.out.println("connecting to datastore"); 
    dsDL.connect(STR_LIB_SERVER, STR_USER_ID, STR_PASSWORD,"");
    System.out.println("datastore connected"); 
    
    DKBlobDL axdo = new DKBlobDL(dsDL);
    DKPidXDODL  apid = new DKPidXDODL();
    apid.setPartId(partId);
    apid.setItemId(itemId);
    apid.setRepType(repType);
    axdo.setPidObject(apid);
    System.out.println("  itemid=" + axdo.getItemId());
    System.out.println("  partId=" + axdo.getPartId()); 
    System.out.println("  repType=" + axdo.getRepType()); 
 
    //====== do regular retrieve =======
    //mimetype is from DL
    System.out.println("before retrieve, content length = " + axdo.length());
    System.out.println("about to call retrieve()");
    axdo.retrieve();
    System.out.println("after retrieve, content length = " + axdo.length());
    //System.out.println("about to do open()...");
    //axdo.setInstanceOpenHandler("notepad ",true);   //use lviewpro for image
    //axdo.open();
    content1 = new String(axdo.getContent());

    //====== do asynchronous retrieve with concatenate to buffer area=======
    System.out.println("===============================================================");
    System.out.println("++do asynchronous retrieve with concatenate to buffer area++");
    System.out.println("++ refresh content ++");
    axdo.setNull();
    size = axdo.getSize(); 
    System.out.println("before retrieve, content size =" + size);

    System.out.println("about to do retrieveAsync()...");
    lenLeft = axdo.retrieveAsync(readLen, true);	 
    i = i+1;
    System.out.println("length of retrieve content piece " + i + "=" + axdo.length());
    System.out.println("lenLeft=" + lenLeft);   
    while (lenLeft > 0)
    {	 
	lenLeft = axdo.retrieveAsyncNext();
	i = i+1;
	System.out.println("after retrieve content piece " + i + " lenLeft=" + lenLeft);
	System.out.println("length of content = " + axdo.length());
    }	  
    System.out.println("about to do retrieveAsyncEnd()...");
    axdo.retrieveAsyncEnd();
    content2 = new String(axdo.getContent());
    //compare result should be true
    System.out.println("compare content1 & content2 = " + content1.equals(content2)); 

    //====== do asynchronous retrieve without concatenate to buffer area=======
    System.out.println("===============================================================");
    System.out.println("++do asynchronous retrieve without concatenate to buffer area++");
    System.out.println("++ refresh content ++");
    i = 0;
    axdo.setNull();
    size = axdo.getSize(); 
    System.out.println("before retrieve, content size =" + size);

    System.out.println("about to do retrieveAsync()...");
    lenLeft = axdo.retrieveAsync(readLen, false);	 
    i = i+1;
    System.out.println("length of retrieve content piece " + i + "=" + axdo.length());
    System.out.println("lenLeft=" + lenLeft);   
    while (lenLeft > 0)
    {	 
	lenLeft = axdo.retrieveAsyncNext();
	i = i+1;
	System.out.println("after retrieve content piece " + i + " lenLeft=" + lenLeft);
	System.out.println("length of content = " + axdo.length());
    }	  
    System.out.println("about to do retrieveAsyncEnd()...");
    axdo.retrieveAsyncEnd();
    content2 = new String(axdo.getContent());
    //compare result should be false
    System.out.println("compare content1 & content2 = " + content1.equals(content2)); 

    dsDL.disconnect();
    System.out.println("datastore disconnected"); 
    dsDL.destroy();
    }
    catch (DKException exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }        
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }        
     exc.printStackTrace();
    }
  }
}
