/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/


/*
 * Filename       : TADLstDemo.java
 * Description    : test add, delete, list search template for FedDemo
 *                : Need to run the following program first
 *                 java TADLServers <feddb> <userid> <pw> add
 *                 java TADLsmDemo <feddb> <userid> <pw> add
 *
 *
 */

import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

public class TADLstDemo implements DKConstantFed 
{

	// Main method
	public static void main(String argv[]){

		// Indicate this is a Console Application on Windows NT/95 for error
		// message display in dos console code page. If this was a Windows
		// GUI (Graphic User Interface) Application this would not be necessary
		// because DK_CM_SS_WINDOWS is the default.
		DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
        
        // checking input parameters
        String libSrv = "",    // library server
               userid = "",    // userid 
               pw     = "",    // pw
               cmd    = "";    // command: add, del, or list
        
        //-------------------------------------------------------------- 
        // Checking for input parameters
        //-------------------------------------------------------------- 
        if (argv.length < 4) {
           System.out.println("Usage Syntax: ");
           System.out.println("\t java TADLstDemo <feddb> <userid> <pw> <add/del/list>" );
           return;
        }
        else {
           libSrv = argv[0];
           userid = argv[1];
           pw     = argv[2];
           cmd    = argv[3];
        }   

        
        
		try {
            // create federated datastore
            System.out.println(" *** connecting to datastore : " + libSrv);
            DKDatastoreFed ds = new DKDatastoreFed();
            ds.connect(libSrv, userid, pw, "");
            System.out.println(" *** datastore connected *** "); 

            // get the Def
            dkDatastoreDef  defFed = ds.datastoreDef();
		    
            // set up search criteria memory variable.
    		short[] ops = { DKOpCode.EQ, DKOpCode.NEQ, DKOpCode.LIKE, DKOpCode.NOTLIKE, DKOpCode.IN, DKOpCode.NOTIN };
            
//========== Create search criteria for FedTitle
            DKSearchCriteria scTitle = new DKSearchCriteria("FedDemo", "FedTitle");
                scTitle.setName("Title");
                scTitle.setDisplayName("Document Title");
            	scTitle.setDefaultOperator(DKOpCode.LIKE);
		        scTitle.setValidOperators(ops);
            	scTitle.setDisplayWidth((short)40);
		        scTitle.setQueryable(true);
		        scTitle.setDisplayable(true);
 
//========== Create search criteria for FedUserID
            DKSearchCriteria scUserID = new DKSearchCriteria("FedDemo", "FedUserId");
                scUserID.setName("UserID");
                scUserID.setDisplayName("Document UserID");
            	scUserID.setDefaultOperator(DKOpCode.LIKE);
		        scUserID.setValidOperators(ops);
            	scUserID.setDisplayWidth((short)40);
		        scUserID.setQueryable(true);
		        scUserID.setDisplayable(true);

//========== Create search criteria for FedDate
            DKSearchCriteria scDate = new DKSearchCriteria("FedDemo", "FedDate");
                scDate.setName("Date");
                scDate.setDisplayName("Document Date");
            	scDate.setDefaultOperator(DKOpCode.LIKE);
		        scDate.setValidOperators(ops);
            	scDate.setDisplayWidth((short)40);
		        scDate.setQueryable(true);
		        scDate.setDisplayable(true);
		        scDate.setOperator(DKOpCode.NEQ);
		        scDate.setValue("NULL");

//========== Create template with one criteria.
            DKNVPair opt[] = new DKNVPair[1];
            opt[0] = new DKNVPair("MAX_RESULTS", "6");
		    DKSearchTemplateFed term = new DKSearchTemplateFed(ds);
    		    term.setName("DEMO Term");
		        term.setDescription("Search template on FedUserID of FedDemo.");
		        term.setFolderName("DavidFolderDemoTerm");
		        term.setOptionList(opt);
		        term.setEntityName("FedDemo");
		    try {
		        if (cmd.equalsIgnoreCase("add")) {
		            term.add();     // Add search template to persistent datastore
		            term.addUser(ds.userName(), (short) DK_FED_ST_ACCESS_ADMIN);
		            term.add(scUserID);
		            System.out.println("Successfully add search template with criteria FedUserID");
		        }
		        else if (cmd.equalsIgnoreCase("del")) {
		            term.retrieve();
		            term.del();
		            System.out.println("Successfully delete search template with criteria FedUserID");
		        }
		    }
		    catch (Exception e) {
		        e.printStackTrace();
		    }
		        
//========== Create template with FedUserID OR FedTitle
		    DKSearchTemplateFed or2 = new DKSearchTemplateFed(ds);
    		    or2.setName("DEMO OR2");
		        or2.setDescription("Search template on FedUserID OR FedTitle of FedDemo.");
		        or2.setFolderName("DavidFolderDemoOR2");
		        or2.setParametricOp(DK_CM_OP_ALL_ORS);
		        or2.setEntityName("FedDemo");
		        or2.setOptionList(opt);
		    
		    try {
		        if (cmd.equalsIgnoreCase("add")) {
		            or2.add();     // Add search template to persistent datastore
		            or2.addUser(ds.userName(), (short) DK_FED_ST_ACCESS_ADMIN);
		            or2.add(scUserID);
		            or2.add(scTitle);
		            System.out.println("Successfully add search template with  FedUserID OR FedTitle");
		        }
		        else if (cmd.equalsIgnoreCase("del")) {
		            or2.retrieve();
		            or2.del();
		            System.out.println("Successfully delete search template with FedUserID OR FedTitle");
		        }
		    }
		    catch (Exception e) {
		        e.printStackTrace();
		    }
        
		        
//========== Create template with FedDate AND FedTitle
		    DKSearchTemplateFed and2 = new DKSearchTemplateFed(ds);
    		    and2.setName("DEMO AND2");
		        and2.setDescription("Search template on FedDate AND FedTitle of FedDemo.");
		        and2.setFolderName("DavidFolderDemoAND2");
		        and2.setParametricOp(DK_CM_OP_ALL_ANDS);
		        and2.setEntityName("FedDemo");
		        and2.setOptionList(opt);
		    
		    try {
		        if (cmd.equalsIgnoreCase("add")) {
		            and2.add();     // Add search template to persistent datastore
		            and2.addUser(ds.userName(), (short) DK_FED_ST_ACCESS_ADMIN);
		            and2.add(scDate);
		            and2.add(scTitle);
		            System.out.println("Successfully add search template with  FedDate AND FedTitle");
		        }
		        else if (cmd.equalsIgnoreCase("del")) {
		            and2.retrieve();
		            and2.del();
		            System.out.println("Successfully delete search template with FedDate AND FedTitle");
		        }
		    }
		    catch (Exception e) {
		        e.printStackTrace();
		    }

//========== Print out list of search templates.
            dkCollection coll = ds.listSearchTemplates();
            dkIterator iter = null;
            
            if (coll != null) {
                iter = coll.createIterator();
                System.out.println(" ");
                System.out.println("-- list Search template -- ");
                DKSearchTemplateFed stObj;

                while (iter.more()) {
                    stObj = (DKSearchTemplateFed) iter.next();
                    System.out.println("\n *** Search Template Name : " + stObj.getName());
                    System.out.println("    Description: " + stObj.getDescription());
                    System.out.println("    Folder Name: " + stObj.getFolderName());
                    System.out.println("    Entity Name: " + stObj.getEntityName());
                    System.out.print  (" Parametric OP : ");
                    if (stObj.getParametricOp() == DK_CM_OP_ALL_ANDS)
                        System.out.println("DK_CM_OP_ALL_ANDS");
                    else if (stObj.getParametricOp() == DK_CM_OP_ALL_ORS)
                        System.out.println("DK_CM_OP_ALL_ORS");
                        
                    dkCollection scColl = stObj.listCriteria();
                    dkIterator scIter = scColl.createIterator();
                    while (scIter.more()) {
                        DKSearchCriteria scObj = (DKSearchCriteria) scIter.next();
                        System.out.println("     ++ Criteria Name: " + scObj.getName());
                        System.out.println("         Display Name: " + scObj.getDisplayName());
                        System.out.println("          Entity Name: " + scObj.getEntityName());
                        System.out.println("            Attr Name: " + scObj.getAttrName());
                        System.out.println("           Default Op: " + scObj.getDefaultOperator());
                        System.out.println("        Display Width: " + scObj.getDisplayWidth());
		                System.out.println("            Queryable: " + scObj.isQueryable());
		                System.out.println("          Displayable: " + scObj.isDisplayable());
		                System.out.println(" ");
		            }
		        }
		    }
		    else 
		        System.out.println("No Search template defines");
		    ds.disconnect();
		    System.out.println("Disconnect. Done.");
        }    
		catch (DKException exc) {
			System.out.println("Exception name " + exc.name());
			System.out.println("Exception message " + exc.getMessage());
			exc.printStackTrace();
    	}
		catch (Exception exc) {
			System.out.println("Exception message " + exc.getMessage());
			exc.printStackTrace();
    	}

	}

}

