/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/* -------------------------------------------------------------------------
 * Class Name  = TQSetup.java
 * Description = Setup required things before running queries
 *                  - Add/Remove/List back end servers (Note that back-end
 *                    servers can be added/removed/listed for only 
 *                    DKDatastoreFed)
 * 
 *               This program uses TdkDatastore.java, TLogUtils.java, 
 *               and TdkCallback.java (TdkCallback is used by TdkDatastore)
 *
 * Author      = Chandra S Potula
 * Create Date = 07/04/2000
 * 
 * Change History
 * -------------------------------------------------------------------------
 */

import com.ibm.mm.sdk.common.*;

import java.io.*;
import java.util.*;

public class TQSetup extends TdkDatastore implements DKConstantFed {
	// these constants are used for schemaMapping in 
	// add() and getProcessType()
	public static final short ID_ADD_ATTR = 1;
	public static final short ID_DEL_ATTR = 2;
	public static final short ID_ADD_SM   = 3;
	public static final short ID_DEL_SM   = 4;

public TQSetup() {
	super();
}
public void processCommand(String aCmd) {
	try {
		String args[] = getParms(aCmd);

		if(args != null && args.length > 0) {
			if(     args[0].equalsIgnoreCase("addServer"             )) {
				addServer(args[1]);
			}
			else if(args[0].equalsIgnoreCase("removeServer") ||
					args[0].equalsIgnoreCase("delServer")                 ) {
				removeServer(args[1]);
			}
			else if(args[0].equalsIgnoreCase("listServerInfo"        )) {
				listServerInfo(args[1]);
			}
			else if(args[0].equalsIgnoreCase("listServersInfo"       )) {
				listServersInfo();
			}
			else if(args[0].equalsIgnoreCase("listServerNames"       )) {
				listServerNames();
			}
			else if(args[0].equalsIgnoreCase("listServers"           )) {
				listServers();
			}
			else if(args[0].equalsIgnoreCase("listAvailableServers"  )) {
				listAvailableServers();
			}
			else if(args[0].equalsIgnoreCase("selectServers"         )) {
				selectServers(args);
			}
			else if(args[0].equalsIgnoreCase("assocSearchServer"     )) {
				assocSearchServer(args[1], args[2]);
			}
			else if(args[0].equalsIgnoreCase("disassocSearchServer"  )) {
				disassocSearchServer(args[1], args[2]);
			}
			else if(args[0].equalsIgnoreCase("listAllAssocSearchServerNames" )) {
				listAllAssocSearchServerNames();
			}
			else if(args[0].equalsIgnoreCase("listEntityNames"       )) {
				listEntityNames();
			}
			else if(args[0].equalsIgnoreCase("addEntity"             )) {
				if(args.length > 3) {
					addEntity(args[1], args[2], args[3]);
				}
				else {
					addEntity(args[1], args[2]);
				}
			}
			else if(args[0].equalsIgnoreCase("delEntity"             )) {
				delEntity(args[1]);
			}
			else if(args[0].equalsIgnoreCase("delSchemaMappingOnEntity")) {
				delSchemaMappingOnEntity(args[1]);
			}
			else if(args[0].equalsIgnoreCase("listEntityAttrNames"   )) {
				if(args.length > 1) {
					listEntityAttrNames(args[1]);
				}
				else {
					listEntityAttrNames();
				}
			}
			else if(args[0].equalsIgnoreCase("listEntityAttrInfo"   )) {
				if(args.length > 1) {
					listEntityAttrInfo(args[1]);
				}
				else {
					listEntityAttrInfo();
				}
			}
			else if(args[0].equalsIgnoreCase("listMappingNames"      )) {
				listMappingNames();
			}
			else if(args[0].equalsIgnoreCase("listSchemaMappingNames")) {
				if(args.length > 1) {
					listSchemaMappingNames(args[1]);
				}
				else {
					listSchemaMappingNames();
				}
			}
			else if(args[0].equalsIgnoreCase("listSchemaMappingInfo" )) {
				if(args.length > 1) {
					listSchemaMappingInfo(args[1]);
				}
				else {
					listSchemaMappingInfo();
				}
			}
			else if(args[0].equalsIgnoreCase("mapAttr"                )) {
				args = getTokens(aCmd, 1, STR_COMMAND_DELIMITER + ".->");
				replaceVariables(args, 0);
				mapAttr(args);
			}
			else if(args[0].equalsIgnoreCase("mapEntity"              )) {
				args = getTokens(aCmd, 1, STR_COMMAND_DELIMITER + ".->");
				replaceVariables(args, 0);
				mapEntity(args);
			}
			else {
				super.processCommand(aCmd);
			}
		}
	}
	catch(Exception exp) {
		logmsgThrowable(exp);
	}
}
public void listEntityNames() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listEntityNames();
	logmsgList(lstrNames);
}
public void listMappingNames() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listMappingNames();
	logmsgList(lstrNames);
}
public void listSchemaMappingNames() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listEntityNames();
	for(int i=0; i<lstrNames.length; i++) {
		listSchemaMappingNames(lstrNames[i]);
	}
}
public void listSchemaMappingNames(String anEntityName) throws DKException, Exception {
	logmsg(anEntityName);
	dkEntityDef lEntityDef = getdkDatastore().datastoreDef().getEntity(anEntityName);

	if(lEntityDef == null) {
		logmsgError("Entity not found with that name");
	}
	else if(!(lEntityDef instanceof DKEntityDefFed)) {
		logmsgError("Entity is not DKEntityDefFed type");
	}
	else {
		String[] lstrNames = ((DKEntityDefFed)lEntityDef).listSchemaMappingNames();
		logmsgList(lstrNames, STR_TAB + STR_TAB);
	}
}
public void delSchemaMappingOnEntity(String anEntityName) throws DKException, Exception {
	dkEntityDef lEntityDef = getdkDatastore().datastoreDef().getEntity(anEntityName);

	if(lEntityDef == null) {
		logmsgError("Entity not found with that name");
	}
	else if(!(lEntityDef instanceof DKEntityDefFed)) {
		logmsgError("Entity is not DKEntityDefFed type");
	}
	else {
		DKEntityDefFed  lFedEntity       = ((DKEntityDefFed)lEntityDef);
		String[]        lstrSMNames      = lFedEntity.listSchemaMappingNames();
		dkSchemaMapping ldkSchemaMapping = null;
		String          lName;

		for(int i=0; i<lstrSMNames.length; i++) {
			logmsg(lstrSMNames[i]);
			ldkSchemaMapping = lFedEntity.getSchemaMapping(lstrSMNames[i]);
			lName            = ldkSchemaMapping.getName();
			ldkSchemaMapping.del();
			logmsg(lName + " deleted");
		}
	}
}
public void listSchemaMappingInfo() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listEntityNames();
	for(int i=0; i<lstrNames.length; i++) {
		listSchemaMappingInfo(lstrNames[i]);
	}
}
public void listSchemaMappingInfo(String anEntityName) throws DKException, Exception {
	logmsg(anEntityName);
	dkEntityDef lEntityDef = getdkDatastore().datastoreDef().getEntity(anEntityName);

	if(lEntityDef == null) {
		logmsgError("Entity not found with that name");
	}
	else if(!(lEntityDef instanceof DKEntityDefFed)) {
		logmsgError("Entity is not DKEntityDefFed type");
	}
	else {
		String          lstrIndentTab    = getIndentTab();
		DKEntityDefFed  lFedEntity       = ((DKEntityDefFed)lEntityDef);
		String[]        lstrSMNames      = lFedEntity.listSchemaMappingNames();
		dkSchemaMapping ldkSchemaMapping = null;
		dkCollection    lcollAttrMapping = null;
		dkAttrMapping   ldkAttrMapping   = null;

		try {
			changeIndentTab(true);
			for(int i=0; i<lstrSMNames.length; i++) {
				logmsg(lstrSMNames[i]);
				ldkSchemaMapping = lFedEntity.getSchemaMapping(lstrSMNames[i]);

				//logmsgList(ldkSchemaMapping.listAttrMappingNames());

				changeIndentTab(true);
				if(ldkSchemaMapping.getMappedEntityType() == DK_FED_MAPPED_TEXT_ENTITY) {
					// it is TEXT_ENTITY (which is Entity mapping only)
					// with associated server/entity(s)
					logmsg(sizeStr(lFedEntity.getName()                  + "-"   + 
							ldkSchemaMapping.getAssocMappedEntityName(), 25) + "-> " +
							ldkSchemaMapping.getServerName().trim()          + "-"   +
							ldkSchemaMapping.getAssocServerName()            + "."   +
							ldkSchemaMapping.getEntityName().trim()          + "-"   +
							ldkSchemaMapping.getAssocEntityName().trim());
				}
				else if(ldkSchemaMapping.getMappedEntityType() == DK_FED_MAPPED_IMAGE_ENTITY) {
					// it is IMAGE_ENTITY
				}
				else {
					// it is ENTITY (which has attribute mapping
					lcollAttrMapping = ldkSchemaMapping.listAttrMappings();
					dkIterator lIter = lcollAttrMapping.createIterator();
					while(lIter.more()) {
						ldkAttrMapping        = (dkAttrMapping)lIter.next();
						short  nMapToAttr     = ldkAttrMapping.numberOfMapToAttr();
						short  nMappedAttr    = ldkAttrMapping.numberOfMappedAttr();
						String lFedEntityName = stringConst(lFedEntity.getName());
						String lFedAttrName;
						String lBEServerName  = stringConst(ldkSchemaMapping.getServerName());
						String lBEEntityName  = stringConst(ldkSchemaMapping.getEntityName());
						String lBEAttrName;

						for(short nI=1; nI<=nMappedAttr; nI++) {
							lFedAttrName = stringConst(ldkAttrMapping.getMappedAttrName(nI));
							for(short nJ=1; nJ<=nMapToAttr; nJ++) {
								lBEAttrName = stringConst(ldkAttrMapping.getMapToAttrName(nJ));
								logmsg(sizeStr(lFedEntityName + "."   + 
										lFedAttrName,25)          + "-> " + 
										lBEServerName             + "."   +
										lBEEntityName             + "."   +
										lBEAttrName);
							}
						}
					}
				}
				changeIndentTab(false);
			}
			changeIndentTab(false);
		}
		finally {
			setIndentTab(lstrIndentTab);
		}
	}
}
public void listEntityAttrNames() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listEntityNames();
	for(int i=0; i<lstrNames.length; i++) {
		listEntityAttrNames(lstrNames[i]);
	}
}
public void listEntityAttrNames(String aName) throws DKException, Exception {
	logmsg(aName);
	String[] lstrNames = getdkDatastore().listEntityAttrNames(aName);
	logmsgList(lstrNames, STR_TAB + STR_TAB);
}
public void listEntityAttrInfo() throws DKException, Exception {
	String[] lstrNames = getdkDatastore().listEntityNames();
	for(int i=0; i<lstrNames.length; i++) {
		listEntityAttrInfo(lstrNames[i]);
	}
}
public void listEntityAttrInfo(String aName) throws DKException, Exception {
	dkEntityDef lEntity     = getEntity(aName);
	lEntity     = getEntity(aName);
	lEntity     = getEntity(aName);
	String[]    lstrNames   = getdkDatastore().listEntityAttrNames(aName);

	if(lEntity != null) {
		dkAttrDef lAttr       = null;
		String    lEntityInfo = sizeStr(aName, 15) + " Type:" + sizeStr(lEntity.getType(), 4) + lEntity.getDescription();
		String    lAttrInfo   = "";

		logmsg(lEntityInfo);

		changeIndentTab(true);
		for(int i=0; lstrNames != null && i<lstrNames.length; i++) {
			try {
				lAttr     = lEntity.getAttr(lstrNames[i]);
				lAttrInfo = sizeStr(lstrNames[i], 20) + 
						" Type:" + sizeStr(attrTypeToStr(lAttr.getType()), 15) + 
						"  StringType:" + sizeStr(stringTypeToStr((short)lAttr.getStringType()), 15) + 
						lAttr.getDescription();
			}
			catch(Exception e1) {
				lAttrInfo = lstrNames[i];
			}
			logmsg(lAttrInfo);
		}
		changeIndentTab(false);
	}
	else {
		logmsg(aName);
		logmsgList(lstrNames, STR_TAB + STR_TAB);
	}
}
public boolean removeServer(String aServerName) throws DKException, Exception {
	boolean rc = false;

	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(aServerName != null && lDSAdminFed != null) {
		lDSAdminFed.removeServer(aServerName.trim());
		rc = true;
		logmsg("-->server successfully removed : " + aServerName);
	}
	else {
		logmsg(STR_ERROR + "error removing server");
	}
	return rc;
}
public boolean selectServers(String[] aSelection) throws DKException, Exception {
	boolean rc = false;
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();

	if(aSelection != null && lDSAdminFed != null) {
		String lSrvrNames[] = lDSAdminFed.listServerNames();
		for(int nSel=1; nSel<aSelection.length; nSel++) {
			// skip if this server name is already found
			if(!aSelection[nSel].equals("")) {
				for(int nAvl=0; nAvl<lSrvrNames.length; nAvl++) {
					if(aSelection[nSel].equalsIgnoreCase(lSrvrNames[nAvl])) {
						for(int nRemaining=nSel; nRemaining<aSelection.length; nRemaining++) {
							// check if duplicate servernames specified in selection list
							if(aSelection[nSel] == aSelection[nRemaining]) {
								aSelection[nRemaining] = "";
							}
						}
						aSelection[nSel] = "";
						lSrvrNames[nAvl] = "";
					}
				}
			}

			if(!aSelection[nSel].equals("")) {
				// This server is not there so add it
				addServer(aSelection[nSel]);
			}
		}
		for(int nAvl=0; nAvl<lSrvrNames.length; nAvl++) {
			if(!lSrvrNames[nAvl].equals("")) {
				// This server is already there but not selected so remove it
				removeServer(lSrvrNames[nAvl]);
			}
		}
		rc = true;
	}
	return rc;
}
public boolean addServer(String aServerName) throws DKException, Exception {
	boolean rc = false;

	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(aServerName != null && lDSAdminFed != null) {
		String            lstrName    = aServerName.trim();
		DKServerMapDefFed lServerMap  = new DKServerMapDefFed();
		String[]          lServerInfo = getServerInfo(lstrName);
		if(lServerInfo != null) {
			int nPortNumber = toNumber(lServerInfo[ _SRVR_RemotePort]).intValue();

			lServerMap.setName          (lServerInfo[ _SRVR_Name          ]         );
			lServerMap.setServerType    (lServerInfo[ _SRVR_Type          ]         );
			lServerMap.setRemoteHost    (lServerInfo[ _SRVR_RemoteHost    ]         );
			lServerMap.setRemotePort    (nPortNumber                                );
			lServerMap.setConnectString (lServerInfo[ _SRVR_ConnectString ]         );
			lServerMap.setDescription   (lServerInfo[ _SRVR_Description   ]         );
			lServerMap.setRefreshTime   (new DKTimestamp(System.currentTimeMillis()));

			lDSAdminFed.addServer(lServerMap);
			rc = true;
			logmsg("-->Server successfully added : " + lstrName);
		}
		else {
			logmsg(STR_ERROR + "server information is not defined in this program");
		}
	}
	else {
		logmsg(STR_ERROR + "servers cannot be added to this datastore OR invalid Server name");
	}
	return rc;
}
public void listServerInfo(String aServerName) throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(aServerName != null && lDSAdminFed != null) {
		DKServerMapDefFed lServerMap = lDSAdminFed.getServer(aServerName.trim());

		try {
			String lHost = lServerMap.getRemoteHost();
			if(lHost != null) {
				lHost = lHost.trim();
			}

			logmsg("------------------------------------------------------------");
			logmsg("Server Name       : " + lServerMap.getName());
			logmsg("    Type          : " + lServerMap.getServerType());
			logmsg("    RemoteHost    : " + lHost);
			logmsg("    RemotePort    : " + lServerMap.getRemotePort());
			logmsg("    ConnectString : " + lServerMap.getConnectString());
			logmsg("    Description   : " + lServerMap.getDescription());
			logmsg("    RefreshTime   : " + lServerMap.getRefreshTime());
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
	else {
		logmsg(STR_ERROR + "listing Server information");
	}
}
public void listServerNames() throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		String lSrvrNames[] = lDSAdminFed.listServerNames();
		logmsg("Selected back-end servers:");
		for(short nInd=0; nInd<lSrvrNames.length; nInd++) {
			logmsg("    " + sizeStr((short)(nInd+1)) + lSrvrNames[nInd]);
		}
	}
}
public void listServers() throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		String lSrvrNames[] = lDSAdminFed.listServerNames();
		logmsg("Selected back-end servers:");
		DKServerMapDefFed lServerMap = null;
		for(short nInd=0; nInd<lSrvrNames.length; nInd++) {
			lServerMap = getDatastoreAdminFed().getServer(lSrvrNames[nInd]);
			logmsg("    " + sizeStr((short)(nInd+1)) + sizeStr(lSrvrNames[nInd], 32) +
				sizeStr(lServerMap.getServerType(), 4) + lServerMap.getRemoteHost().trim());
		}
	}
}
public void listServersInfo() throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		String[] lstrNames =  lDSAdminFed.listServerNames();
		for(int nInd=0; nInd<lstrNames.length; nInd++) {
			listServerInfo(lstrNames[nInd]);
		}
	}
}
public void listAvailableServers() throws DKException, Exception {
	logmsg("All available back-end servers:");
	for(short nInd=0; nInd<_serverList.length; nInd++) {
		String[] lInfo = getTokens(_serverList[nInd][1]);
		logmsg("    " + sizeStr((short)(nInd + 1)) + sizeStr(_serverList[nInd][0], 32) + sizeStr(lInfo[0], 4) + lInfo[1]);
	}
}
public void assocSearchServer(String aBackEndServer, String aSearchServer) 
		throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		lDSAdminFed.assocSearchServer(aBackEndServer, aSearchServer);
	}
}
public void disassocSearchServer(String aBackEndServer, String aSearchServer) 
		throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		lDSAdminFed.disassocSearchServer(aBackEndServer, aSearchServer);
	}
}
public void listAllAssocSearchServerNames() 
		throws DKException, Exception {
	DKDatastoreAdminFed lDSAdminFed = getDatastoreAdminFed();
	if(lDSAdminFed != null) {
		String[] lstrNames = lDSAdminFed.listAllAssocSearchServerNames();
		logmsgList(lstrNames);
	}
}
public static String[] getServerInfo(String aServerName) {
	String[] rc = null;

	if(aServerName != null && aServerName.length() > 0) {
		for(int nInd=0; nInd<_serverList.length; nInd++) {
			if(_serverList[nInd][0].equalsIgnoreCase(aServerName)) {
				String[] lInfo = getTokens(_serverList[nInd][1]);
				rc    = new String[lInfo.length + 1];

				// put server name also in the list (because it is case
				// sensitive for some servers
				rc[0] = _serverList[nInd][0];
				for(int nInd2=0; nInd2<lInfo.length; nInd2++) {
					rc[nInd2 + 1] = lInfo[nInd2];
				}
				break;
			}
		}
	}

	return rc;
}
public dkAttrMapping createAttrMapping(dkAttrDef aFromAttr, dkAttrDef aToAttr)
		throws DKException, Exception {
	dkAttrMapping lAttrMapping = new DKAttrMappingFed();
	lAttrMapping.addMappedAttr(aFromAttr);
	lAttrMapping.addMapToAttr (aToAttr);

	return lAttrMapping;
}
public dkEntityDef addEntity(String aName, String aDescription)
		throws DKException, Exception {
	return addEntity(aName, aDescription, "normal");
}
public dkEntityDef addEntity(String aName, String aDescription, String aEntityType)
		throws DKException, Exception {
	short nEntityType = DK_FED_MAPPED_ENTITY;

	if(aEntityType.equalsIgnoreCase("text")) {
		nEntityType = DK_FED_MAPPED_TEXT_ENTITY;
	}
	else if(aEntityType.equalsIgnoreCase("image")) {
		nEntityType = DK_FED_MAPPED_IMAGE_ENTITY;
	}

	return addEntity(aName, aDescription, nEntityType);
}
public dkEntityDef addEntity(String aName, String aDescription, short nEntityType)
		throws DKException, Exception {
	dkEntityDef lEntityDef = newEntity(aName, aDescription, nEntityType);
	if(lEntityDef != null) {
		lEntityDef.add();
	}
	return lEntityDef;
}
public dkEntityDef newEntity(String aName, String aDescription, short nEntityType)
		throws DKException, Exception {
	dkEntityDef lEntityDef = getdkDatastore().datastoreDef().createEntity();
	if(lEntityDef != null) {
		lEntityDef.setName(aName);
		lEntityDef.setDescription(aDescription);
		lEntityDef.setType(nEntityType);
	}
	return lEntityDef;
}
public boolean delEntity(String aName) throws DKException, Exception {
	boolean rc = false;

	dkEntityDef lEntityDef = getdkDatastore().datastoreDef().getEntity(aName);
	if(lEntityDef != null) {
		lEntityDef.del();
		rc = true;
	}
	else {
		logmsgError("Entity not found");
	}
	return rc;
}
public dkEntityDef getEntity(String aName) {
	// return enitity if it exists otherwise return null
	dkEntityDef rc = null;

	try {
		rc = getdkDatastore().datastoreDef().getEntity(aName);
	}
	catch(Exception e) {
	}

	return rc;
}
private short getProcessType(String aType) {
	short rc = ID_ADD_SM;
	if(aType != null) {
		if(     aType.equalsIgnoreCase("delAttr")) {
			rc = ID_DEL_ATTR;
		}
		else if(aType.equalsIgnoreCase("addAttr")) {
			rc = ID_ADD_ATTR;
		}
		else if(aType.equalsIgnoreCase("del")) {
			rc = ID_DEL_SM;
		}
		else {
			// "add"
			// default is add schema mapping
			rc = ID_ADD_SM;
		}
	}
	return rc;
}
public boolean mapEntity(String args[]) {
	boolean rc = false;
	try {
		if(args.length < 6) {
			logmsgError("invalid syntax");
			logmsg("-->usage : mapEntity(FedEntity-AssocFedEntity->BEServer-AssocServer.BEEntity-AssocEntity, add/del)");
		}
		else {
			short   nProcType           = ID_ADD_SM;

			String  lFedEntityName      = args[0];
			String  lAssocFedEntityName = args[1];
			String  lBEServerName       = args[2];
			String  lAssocServerName    = args[3];
			String  lBEEntityName       = args[4];
			String  lAssocEntityName    = args[5];

			DKEntityDefFed lFedEntity = null;
	
			if(args.length > 6) {
				nProcType   = this.getProcessType(args[6]);
				if(nProcType != ID_ADD_SM && nProcType != ID_DEL_SM) {
					logmsg("-->Invalid process type!  Only add/del allowed");
					logmsg("-->Trying to add EntitySchemaMapping");
				}
			}
			if(nProcType != ID_DEL_SM) {
				nProcType = ID_ADD_SM;
			}

			lFedEntity = (DKEntityDefFed)getEntity(lFedEntityName);
			if(lFedEntity == null) {
				lFedEntity = (DKEntityDefFed)addEntity(lFedEntityName, lFedEntityName + " to map to " + 
						lBEEntityName + " at " + lBEServerName, "text");
			}

		
			// no need to connect to native server and get entity because to
			// create SchemaMapping we don't require instance of Entity
			/*
			dkEntityDef    lBEEntity  = null;
			try {
				logmsg("-->Connecting to native server");
				setMapServer(lBEServerName);
			}
			catch(Exception e1) {
				logmsgThrowable(e1);
			}
			queryMapServer(true);
			lBEEntity = getEntity(lBEEntityName);
			queryMapServer(false);
			*/

			// Look in Datastore for schema mapping between specified entities
			dkCollection    lcollSM        = lFedEntity.listSchemaMappings();
			dkIterator      lIterCollSM    = lcollSM.createIterator();
			dkSchemaMapping ldkSMBE        = null;
			while(ldkSMBE == null && lIterCollSM.more()) {
				ldkSMBE = (dkSchemaMapping)lIterCollSM.next();
				if( ldkSMBE.getMappedEntityName().trim().equals(lFedEntityName  ) &&
						ldkSMBE.getServerName().      trim().equals(lBEServerName   ) &&
						ldkSMBE.getEntityName().      trim().equals(lBEEntityName   )) {
					break;
				}
				ldkSMBE = null;
			}

			if(nProcType == ID_ADD_SM) {
				// add Schema mapping
				boolean fNewSM = false;

				if(ldkSMBE == null) {
					ldkSMBE = new DKSchemaMappingFed(getdkDatastore());
					ldkSMBE.setMappedEntityName      ( lFedEntityName                 );
					ldkSMBE.setServerName            ( lBEServerName                  );
					ldkSMBE.setServerType            ( getMapServer().datastoreType() );
					ldkSMBE.setEntityName            ( lBEEntityName                  );
					ldkSMBE.setMappedEntityType      ( lFedEntity.getType()           );

					// Schema mapping needs attributes to add SM!!!
					// lets create some native dummy attributes...
					dkAttrMapping ldkAttrMapping = new DKAttrMappingFed();
					dkAttrDef     lFedAttr       = new DKAttrDefFed();
					dkAttrDef     lBEAttr        = new DKAttrDefFed();
	
					lFedAttr.setEntityName ( lFedEntityName );
					lFedAttr.setName       ( "DummyFed"     );
					lBEAttr.setEntityName  ( lBEEntityName  );
					lBEAttr.setName        ( "DummyBE"      );

					ldkAttrMapping.addMappedAttr( lFedAttr );
					ldkAttrMapping.addMapToAttr ( lBEAttr  );

					ldkSMBE.addAttrMapping(ldkAttrMapping);
					fNewSM = true;
				}

				ldkSMBE.setAssocMappedEntityName ( lAssocFedEntityName            );
				ldkSMBE.setAssocServerName       ( lAssocServerName               );
				ldkSMBE.setAssocEntityName       ( lAssocEntityName               );

				if(fNewSM) {
					ldkSMBE.add();
				}
				else {
					ldkSMBE.update();
				}
				rc = true;
			}
			else {
				// delete Schema mapping
				if(ldkSMBE != null) {
					ldkSMBE.del();
					rc = true;
				}
				else {
					logmsgError("specified Entity mapping not found");
				}
			}
		}
	}
	catch(Exception e) {
		queryMapServer(false);
		logmsgThrowable(e);
	}
	return rc;
}
public boolean mapAttr(String args[]) {
	boolean rc = false;
	try {
		String         lFedEntityName = null;
		String         lFedAttrName   = null;
		String         lBEServerName  = null;
		String         lBEEntityName  = null;
		String         lBEAttrName    = null;

		DKEntityDefFed lFedEntity     = null;
		dkAttrDef      lFedAttr       = null;
		dkEntityDef    lBEEntity      = null;
		dkAttrDef      lBEAttr        = null;

		short          nProcType      = ID_ADD_SM;
		boolean        fNewSM         = false;

		try {
			lFedEntityName = args[0];
			lFedAttrName   = args[1];
	
			if(args.length <= 3) {
				nProcType   = this.getProcessType(args[2]);
				if(nProcType != ID_DEL_ATTR) {
					String lTmp = args[3];
				}
			}
			else {
				lBEServerName  = args[2];
				lBEEntityName  = args[3];
				lBEAttrName    = args[4];
				if(args.length > 5) {
					nProcType   = this.getProcessType(args[5]);
				}
			}
		}
		catch(IndexOutOfBoundsException iobe) {
			logmsgError("invalid syntax");
			logmsg("-->usage : \nmapAttr(FedEntity.FedAttr->BEServer.BEEntity.BEAttr, addAttr/delAttr/add/del)");
			throw iobe;
		}
		
		lFedEntity = (DKEntityDefFed)getEntity(lFedEntityName);
		if(lFedEntity == null) {
			lFedEntity = (DKEntityDefFed)addEntity(lFedEntityName, lFedEntityName + " to map to " + 
					lBEEntityName + " at " + lBEServerName, "normal");
		}

		try {
			lFedAttr = lFedEntity.getAttr(lFedAttrName);
		}
		catch(Exception e1) {
		}

		if(nProcType == ID_DEL_ATTR) {
			if(lFedAttr == null) {
				logmsgError("Attribute not found");
			}
			else {
				lFedEntity.del(lFedAttr);
				rc = true;
			}
		}
		else {
			try {
				setMapServer(lBEServerName);
			}
			catch(Exception e2) {
				logmsgThrowable(e2);
			}

			queryMapServer(true);

			lBEEntity = getEntity(lBEEntityName);
			if(lBEEntity == null) {
				//logmsgError("getting native entity : " + lBEEntityName);

				if(lFedEntity != null) {
					// create dummy BEEntity from FedEntity for mapping
					lBEEntity = newEntity(lFedEntityName, "", lFedEntity.getType());
					logmsg("-->Created dummy Entity for SchemaMapping");
				}
			}
			else {
				try {
					lBEAttr   = lBEEntity.getAttr(lBEAttrName);
				}
				catch(DKException dke) {
					//logmsgThrowable(dke);
				}
				if(lBEAttr == null) {
					//logmsgError("attr not found using " + lBEEntity.getClass().getName() + ".getAttr(" + lBEAttrName + ")");

					dkCollection lCol     = lBEEntity.listAttrs();
					dkIterator   lIterCol = lCol.createIterator();
					while(lBEAttr == null && lIterCol.more()) {
						lBEAttr = (dkAttrDef)lIterCol.next();
						if(!lBEAttr.getName().equals(lBEAttrName)) {
							lBEAttr = null;
						}
					}
				}
			}

			if(lBEAttr == null) {
				//logmsgError("getting native attr   : " + lBEAttrName);
				if(lBEEntity != null && lFedAttr != null) {
					// create dummy BEAttr from FedAttr for mapping
					lBEAttr = lBEEntity.createAttr();
					lBEAttr.setEntityName ( lBEEntityName          );
					lBEAttr.setName       ( lBEAttrName            );
					lBEAttr.setType       ( lFedAttr.getType()     );
					logmsg("-->Created dummy Attribute for SchemaMapping");
				}
			}

			if(lFedAttr == null) {
				if(lBEAttr == null) {
					logmsgError("creating FedAttr because native Attr not found");
				}
				else {
					// Fed Attr doesn't exist.  Let's create it by creating a 
					// a copy of Back-End server Attr
					lFedAttr = lFedEntity.createAttr();
	
					lFedAttr.setEntityName ( lFedEntityName          );
					lFedAttr.setName       ( lFedAttrName            );
					lFedAttr.setType       ( lBEAttr.getType()       );
					lFedAttr.setId         ( lBEAttr.getId()         );
					lFedAttr.setSize       ( lBEAttr.getSize()       );
					lFedAttr.setScale      ( lBEAttr.getScale()      );
					lFedAttr.setPrecision  ( lBEAttr.getPrecision()  );
					lFedAttr.setNullable   ( lBEAttr.isNullable()    );
					lFedAttr.setStringType ( lBEAttr.getStringType() );

					lFedEntity.add(lFedAttr);
				}
			}
			queryMapServer(false);

			if(nProcType == ID_ADD_ATTR) {
				// IT is create FedAttribute and if the attribute is not
				// there in Fed Datastore then it is created by now
				rc = (lFedAttr != null);
			}
			else if(lFedAttr != null && lBEAttr != null){
				// it is either add or delete Schema Mapping
				// Look in Datastore for schema mapping between specified entities
				dkSchemaMapping ldkSMBE        = null;
				dkCollection    lcollSM        = lFedEntity.listSchemaMappings();
				dkAttrMapping   ldkAttrMapping = null;
		
				dkIterator lIterCollSM = lcollSM.createIterator();
				while(ldkSMBE == null && lIterCollSM.more()) {
					ldkSMBE = (dkSchemaMapping)lIterCollSM.next();
					if(ldkSMBE.getServerName().trim().equals(lBEServerName) &&
							ldkSMBE.getEntityName().trim().equals(lBEEntityName) &&
							ldkSMBE.getMappedEntityName().trim().equals(lFedEntityName)) {
						// schema mapping found
						// look if attribute mapping exist
						dkCollection lcollAttrMapping     = ldkSMBE.listAttrMappings();
						dkIterator   lIterCollAttrMapping = lcollAttrMapping.createIterator();
	
						while(ldkAttrMapping == null && lIterCollAttrMapping.more()) {
							ldkAttrMapping      = (dkAttrMapping)lIterCollAttrMapping.next();
	
							short   nMapToAttr  = ldkAttrMapping.numberOfMapToAttr();
							short   nMappedAttr = ldkAttrMapping.numberOfMappedAttr();
							boolean fFound      = false;
	
							for(short i=1; !fFound && i<=nMappedAttr; i++) {
								for(short j=1; !fFound && j<=nMapToAttr; j++) {
									if(ldkAttrMapping.getMappedAttrName(i).equals(lFedAttrName) &&
											ldkAttrMapping.getMapToAttrName(j).equals(lBEAttrName)) {
										fFound = true;
									}
								}
							}
	
							if(!fFound) {
								ldkAttrMapping = null;
							}
						}
						break;
					}
					ldkSMBE = null;
				}
		
				if(nProcType == ID_ADD_SM) {
					// add schema mapping
					if(ldkSMBE == null) {
						ldkSMBE = new DKSchemaMappingFed(getdkDatastore());
		
						ldkSMBE.setServerName(lBEServerName);
						ldkSMBE.setServerType(getMapServer().datastoreType());
						ldkSMBE.setEntityName(lBEEntityName);
						ldkSMBE.setMappedEntityName(lFedEntityName);
						fNewSM = true;
					}
	
					if(ldkAttrMapping == null) {
						ldkAttrMapping = createAttrMapping(lFedAttr, lBEAttr);
						ldkSMBE.addAttrMapping(ldkAttrMapping);
						rc = true;
					}
					else {
						logmsgError("specified attribute mapping already exist");
						// *** as of v122 system allows to add mapping more than once!
					}
				}
				else {
					// delete schema mapping
					if(ldkAttrMapping != null) {
						ldkSMBE.removeAttrMapping(ldkAttrMapping.getName());
						rc = true;
					}
					else {
						logmsgError("specified attribute mapping not found");
					}
				}
	
				if(rc) {
					if(fNewSM) {
						// true only if it is add schema mapping
						ldkSMBE.add();
					}
					else {
						ldkSMBE.update();
					}
				}
			}
		}
	}
	catch(Exception e3) {
		queryMapServer(false);
		logmsgThrowable(e3);
	}
	
	return rc;
}
protected void initVectVars() {
	if(!_isHelpInfoAdded_TQSetup) {
		super.initVectVars();
		//_vectMethods.addElement(new _MethodInfo("", "", "", "", ""));

		_vectMethods.addElement(new _MethodInfo("addServer",               "Add specified server.", "addServer {_serverName}", "", "servers"));
		_vectMethods.addElement(new _MethodInfo("delServer",               "Remove specified server.", "delServer {_serverName}", "", "remove"));
		_vectMethods.addElement(new _MethodInfo("listAvailableServers",    "Lists all available servers that can be added.", "", "", ""));
		_vectMethods.addElement(new _MethodInfo("listServerNames",         "Lists names of servers that are currently added.", "", "", ""));
		_vectMethods.addElement(new _MethodInfo("listServersInfo",         "Lists information of server(s)", "listServerInfo {_serverName_}", "", "")); 
		_vectMethods.addElement(new _MethodInfo("selectServers",           "Adds specified servers only and will remove other servers", "selectServers {_listOfServers_}", "", "add, del, remove"));
		_vectMethods.addElement(new _MethodInfo("assocSearchServer",       "Associates specified SearchServer with BackEnd server", "assocSearchServer { _BackEndServer_, _SearchServer}", "e.g. assocSearchServer LSMUFASA, TMMUF", "disassociate, backend"));
		_vectMethods.addElement(new _MethodInfo("disassocSearchServer",    "Disassociates specified SearchServer from BackEnd server", "disassocSearchServer { _BackEndServer_, _SearchServer}", "e.g. disassocSearchServer LSMUFASA, TMMUF", "disassociate, backend"));
		_vectMethods.addElement(new _MethodInfo("listAllAssocSearchServerNames", "Lists names of all associated searchServers.", "", "", "disassociate, backend"));
		_vectMethods.addElement(new _MethodInfo("listEntityNames",         "Lists all entity names.", "", "", ""));
		_vectMethods.addElement(new _MethodInfo("addEntity",               "Adds new entity.", "addEntity {_EntityName_, _EntityDescription_, normal|text|image}", "", ""));
		_vectMethods.addElement(new _MethodInfo("delEntity",               "Delete specified entity.", "delEntity {_EntityName_}", "", "remove"));
		_vectMethods.addElement(new _MethodInfo("delSchemaMappingOnEntity","Delete all attribute schema mappings on specified entity.", "delSchemaMappingOnEntity{_EntityName_}", "", "remove"));
		_vectMethods.addElement(new _MethodInfo("listEntityAttrNames",     "Lists names of Entities and attributes.", "listEntityAttrNames [_EntityName_]", "", "attributes"));
		_vectMethods.addElement(new _MethodInfo("listEntityAttrInfo",      "Lists brief information of Entities and attributes.", "listEntityAttrInfo [_EntityName_]", "", "attributes"));
		_vectMethods.addElement(new _MethodInfo("listMappingNames",        "Lists names of all server mappings.", "", "", ""));
		_vectMethods.addElement(new _MethodInfo("listSchemaMappingNames",  "Lists names of schema mappings.", "listSchemaMappingNames [_EntityName_]", "e.g. listSchemaMappingNames()", "entity, attributes"));
		_vectMethods.addElement(new _MethodInfo("listSchemaMappingInfo",   "Lists brief information of schema mappings.", "listSchemaMappingInfo [_EntityName_]", "e.g. listSchemaMappingInfo(FedQSample)", "entity, attributes"));
		_vectMethods.addElement(new _MethodInfo("mapEntity",               "Creates entity schema mapping.", "mapEntity {FedEntity-AssocFedEntity->BESrvr-AssocSrvr.BEEntity-AssocEntity, add|del}", "e.g. addEntity(FedQSample.FedAuthor   -> LSMUFASA.DLSAMPL2.DLSEARCH_Author, add)", "entityschemamapping, adddelremove"));
		_vectMethods.addElement(new _MethodInfo("mapAttr",                 "Add/Del attribute OR creates attribute schema mapping.", "mapAttr {FedEntity.FedAttr -> BEServer.BEEntity.BEAttr, add|del|addAttr|delAttr}",  "e.g. addAttr(FedQTMINDEX-FedQSample -> TMMUF-LSMUFASA.TMINDEX-DLSAMPL2, add)", "attributeschemamapping, adddelremove"));
		_isHelpInfoAdded_TQSetup = true;
	}
}
public static void main(String argv[]) {
	try {
		TQSetup lTQSetup = new TQSetup();
		lTQSetup.processCommandLineArgs(argv);
	}
	catch(Exception exp) {
		exp.printStackTrace();
	}
}

	private static boolean _isHelpInfoAdded_TQSetup;

	private static final int _SRVR_Name           = 0;
	private static final int _SRVR_Type           = 1;
	private static final int _SRVR_RemoteHost     = 2;
	private static final int _SRVR_RemotePort     = 3;
	private static final int _SRVR_ConnectString  = 4;
	private static final int _SRVR_Description    = 5;


	private static final String[][] _serverList = {
			{"LIBARNEY",                        "DL,  \"barney\",                1919,  \"\",                                                                                                                       \"LIBARNEY : DL Library server at barney       (NT)\""},
			{"LIBMONUX",                        "DL,  \"milpitas\",              1919,  \"\",                                                                                                                       \"LIBMONUX : DL Library server at monicac2.stl.ibm.com (AIX)\""},
			{"LIBSRVRX",                        "DL,  \"milpitas\",              1919,  \"\",                                                                                                                       \"LIBSRVRX : DL Library server at hayes.stl.ibm.com (AIX)\""},
			{"LSMUFASA",                        "DL,  \"mufasa\",                1919,  \"\",                                                                                                                       \"LSMUFASA : DL Library server at barney       (AIX)\""},
			{"LSCHIUN",                         "DL,  \"talisman2\",             1919,  \"\",                                                                                                                       \"LSCHIUN  : DL Library server at talisman2    (NT)\""},
			{"LSSNPUX",                         "DL,  \"snoopy\",                1919,  \"\",                                                                                                                       \"LSSNPUX  : DL Library server at snoopy       (AIX)\""},
			{"LSU6MUN",                         "DL,  \"u6mvendor\",             1919,  \"\",                                                                                                                       \"LSU6MUN  : DL Library server at u6mvendor     (NT)\""},
			{"everest",                         "OD,  \"milpitas\",              1919,  \"\",                                                                                                                       \"EVEREST  : OD OnDemand server at everest.boulder.ibm.com (AIX)\""},
			{"hayes",                           "OD,  \"hayes.stl.ibm.com\",     1919,  \"\",                                                                                                                       \"hayes    : OD OnDemand server at hayes.stl.ibm.com (AIX)\""},
			{"w26build.stl.ibm.com/domdoc.nsf", "DD,  \"milpitas\",              1919,  \"\",                                                                                                                       \"w26build : DD Domino.Doc server at w26build.stl.ibm.com (NT)\""},
			{"STEALTH3",                        "V4,  \"STEALTH3\",              1919,  \"\",                                                                                                                       \"STEALTH3 : VI/400 Library server at STEALTH3  (VI/400)\""},
			{"FAFCICS61",                       "IP,  \"mvsd9.raleigh.ibm.com\", 1919,  \"APPL=01 FAFIP=9.67.43.83 IODMIP=9.67.43.83 FAFPORT=3061 IODMPORT=3082 FAFPROT=4000 IODMPROT=4000 FAFSITE=CS61 TRACE=ON\", \"FAFCICS61: IP/390 Library server at mvsd9     (IP/390)\""},
			{"ross.stl.ibm.com",                "DES, \"ross.stl.ibm.com\",      1919,  \"PORT=80;DESAPPID=edms2;DESAPPPW=edms2\",                                                                                  \"ross     : DES server at ross.stl.ibm.com on  (NT)\""},
			{"TMU6M",                           "TS,  \"u6mvendor\",             1919,  \"\",                                                                                                                       \"TMU6M    : TM Text Search server at u6mvendor (NT)\""},
			{"TMMUF",                           "TS,  \"mufasa\",                1919,  \"\",                                                                                                                       \"TMUF     : TM Text Search server at mufasa    (AIX)\""},
			{"TMCHI",                           "TS,  \"talisman2\",             1919,  \"\",                                                                                                                       \"TMUF     : TM Text Search server at talisman2    (NT)\""},
			{"TMSNP",                           "TS,  \"snoopy\",                1919,  \"\",                                                                                                                       \"TMUF     : TM Text Search server at snoopy    (AIX)\""},
			{"as400510",                        "V4,  \"9.82.1.10\",             31015, \"\",                                                                                                                       \"as400510 : DES server\""},
			{"LIBLAHM",                         "DL,  \"servernm\",              23788, \"visearchbuffering=98\",                                                                                                   \"\""},
			{"sahant",                          "OD,  \"sahant.stl.ibm.com\",    1919,  \"\",                                                                                                                       \"\""},
			{"TEXTLAH",                         "TS,  \"REMOTE2\",               67689, \"\",                                                                                                                       \"\""},
			{"LIBSRVRN",                        "DL,  \"kjchou4\",               1919,  \"\",                                                                                                                       \"LIBSRVRN : DL Library server at kjchou4 (NT)\""},
			{"TMCH4",                           "TS,  \"talisman2\",             1919,  \"\",                                                                                                                       \"TMUF     : TM Text Search server at kjchou4    (NT)\""},
			{"LSIKE",                           "DL,  \"IKE\",                   1919,  \"\",                                                                                                                       \"LSIKE    : DL Library server at IKE       (AIX)\""},
			{"TMHSIA",                          "TS,  \"hsia1\",                 1919,  \"\",                                                                                                                       \"TMHSIA   : TM Text Search server at hsia1 (AIX)\""},
	};

	static {
		_isHelpInfoAdded_TQSetup = false;
	}
}
