/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SLinkTypeDefinitionUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Saving Link Type Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Link Type Definitions in Memory
 *                    Saving / Updating Link Type Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SLinkTypeDefinitionUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Link Types must already be defined.  If not, please run Sample
 *                    SLinkTypeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SLinkTypeDefinitionRetrievalICM.java
 ************************************************************************************************/
public class SLinkTypeDefinitionUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SLinkTypeDefinitionUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("=============================================");
        System.out.println("IBM Information Integrator for Content   v"+ver);
        System.out.println("Sample Program:  SLinkTypeDefinitionUpdateICM");
        System.out.println("---------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("=============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve & Print the S_otherPolicies Link Type Definition
            //----------------------------------------------------------------
            // For more information about the defined Link Types, please consult
            // the SLinkTypeDefinitionCreationICM sample.
            System.out.println("Retrieving the Link Type Definition of 'S_otherPolicies'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKLinkTypeDefICM linkTypeDef = (DKLinkTypeDefICM) dsDefICM.retrieveLinkType("S_otherPolicies"); // Retrieve the definition.
                SLinkTypeDefinitionRetrievalICM.printLinkTypeDef(linkTypeDef); // Use wrapper funciton from the Retrieval Sample.
                
            System.out.println("Retrieved the Link Type Definition of 'S_otherPolicies'.");

            //----------------------------------------------------------------
            // Modify the S_otherPolicies Link Type Definition in Memory
            //----------------------------------------------------------------
            System.out.println("Modify the Link Type Definition 'S_otherPolicies' in Memory...");
            
                linkTypeDef.setDescription("Links Insurance Policies Together");

            System.out.println("Modified the Link Type Definition of 'S_otherPolicies' in Memory.");

            //----------------------------------------------------------------
            // Update the S_otherPolicies Link Type Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The Link Type Definition can be updated through its own update() method
            // or by using the Datastore Definition's update() method, passing the attribute
            // definition as the parameter.
            System.out.println("Updating the Link Type Definition 'S_otherPolicies' in Datastore Definition...");

                linkTypeDef.update(); // save the Link Type Definition changes.

            System.out.println("Modified the Link Type Definition of 'S_otherPolicies' in Datastore Definition.");

            //----------------------------------------------------------------
            // Print the Link Type Definition with changes.
            //----------------------------------------------------------------
            System.out.println("Printing Link Type Definition...");
                SLinkTypeDefinitionRetrievalICM.printLinkTypeDef(linkTypeDef); // Sample's Wrapper            
            System.out.println("Printed Link Type Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
    
}//end class SLinkTypeDefinitionUpdateICM
              
