/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

3rd Party Update
    Resource Content may be updated from a 3rd Party source, regardless of whether it was
    created from a 3rd Party source originally or not.  In this sample, Lob Item(s) will 
    created and then updated from content provided by a 3rd Party source.
    
    The same properties and options are available for updating as there were for creating
    from a 3rd Party source.  The only difference is that the item already exists, and the
    "update" is used in place of "add".
    
    Synchronous Updating
        The content is fully-loaded into the Resource Manager before proceeding.
    
    Asynchronous Updating
        The upload is begun for loading the content into the Resource Manager, but the 
        program does not wait for the upload to complete.

    Please refer to the SResourceItem3rdPartyIngestICM Sample for 3rd Party Ingest 
    Property descriptions.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.sql.*;
import java.io.*;
import java.net.*;

/************************************************************************************************
 *          FILENAME: SResourceItem3rdPartyUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Using a 3rd Party source to update Resource Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating from a 3rd Party source
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SResourceItem3rdPartyUpdateICM  <3rd Party Host Name>
 *                                                         <3rd Party Port>
 *                                                         <3rd Party Protocol (http|ftp|file)>
 *                                                         <3rd Party User Name>
 *                                                         <3rd Party Password>
 *                                                         <3rd Party Fully-Qualified File Name>
 *                                                         <3rd Party File Path>
 *                                                         <3rd Party MIME Type>
 *                                                         <3rd Party Content Length>
 *                                                         <database> <userName> <password>
 *
 *                    NOTE:  All 3rd Party Information is Required
 *                        
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: java SResourceItem3rdPartyUpdateICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Document1.doc // application/msword 7168
 *                    Ex: java SResourceItem3rdPartyUpdateICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Picture1.bmp // image/mpg 120054 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                        - SResourceMgrDefSetDefaultICM
 *                        - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    SResourceItemICM_Document2.doc in the root directory.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Document2.doc -> On 3rd Party Host
 ************************************************************************************************/
public class SResourceItem3rdPartyUpdateICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // 3rd Party Information
        String thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        String thirdPartyProtocol       = "NOT_SPECIFIED";
        String thirdPartyUserName       = "NOT_SPECIFIED";
        String thirdPartyPassword       = "NOT_SPECIFIED";
        String thirdPartyFullFileName   = "NOT_SPECIFIED";     
        String thirdPartyFilePath       = "NOT_SPECIFIED";
        String thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 12) { // if not all 10 arguments were specified, report usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceItem3rdPartyUpdateICM  <3rd Party Host Name>");
            System.out.println("                                       <3rd Party Port>");
            System.out.println("                                       <3rd Party Protocol (http|ftp|file)>");
            System.out.println("                                       <3rd Party User Name>");
            System.out.println("                                       <3rd Party Password>");
            System.out.println("                                       <3rd Party Fully-Qualified File Name>");
            System.out.println("                                       <3rd Party File Path>");
            System.out.println("                                       <3rd Party MIME TYpe>");
            System.out.println("                                       <3rd Party Content Length>");
            System.out.println("                                       <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE:  All 3rd Party Information is Required.");
            System.out.println("");
            System.out.println("  NOTE:  Database, UserName, & Password are optional.  If all not specified");
            System.out.println("         defaults will be used.");
            System.out.println("");
            System.out.println("  Ex: java SResourceItem3rdPartyUpdateICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Document2.doc // application/msword 7168");
            System.out.println("  Ex: java SResourceItem3rdPartyUpdateICM myFtpHostName 21 ftp myUserName myPassword SResourceItemICM_Picture2.bmp // image/mpg 120054 icmnlsdb icmadmin password");
            System.out.println("");
        }
        if(argv.length < 9){ // If any 3rd Party info not specified, terminate
            System.out.println("===================================================");
            System.out.println(" TERMINATED:  All 3rd Party info must be specified.");
            System.out.println("===================================================");
            return;        
        } else if(argv.length < 12) {
            System.out.println("*** Some parameters not specified.  Using defaults for rest. ***");
        }
        // Set Command-line options
        if (argv.length > 0)  thirdPartyHostName      = argv[0];
        if (argv.length > 1)  thirdPartyPort          = Integer.valueOf(argv[1]).intValue();
        if (argv.length > 2)  thirdPartyProtocol      = argv[2];
        if (argv.length > 3)  thirdPartyUserName      = argv[3];
        if (argv.length > 4)  thirdPartyPassword      = argv[4];
        if (argv.length > 5)  thirdPartyFullFileName  = argv[5];
        if (argv.length > 6)  thirdPartyFilePath      = argv[6];
        if (argv.length > 7)  thirdPartyMIMEType      = argv[7];
        if (argv.length > 8)  thirdPartyContentLength = Integer.valueOf(argv[8]).intValue();
        if (argv.length > 9)  database                = argv[9];
        if (argv.length > 10) userName                = argv[10];
        if (argv.length > 11) password                = argv[11];
        
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===============================================");
        System.out.println("IBM Information Integrator for Content     v"+ver);
        System.out.println("Sample Program:  SResourceItem3rdPartyUpdateICM");
        System.out.println("-----------------------------------------------");
        System.out.println(" Content Manager:");
        System.out.println("      Database:  "+database);
        System.out.println("      UserName:  "+userName);
        System.out.println(" 3rd Party:");
        System.out.println("      Host Name:  "+thirdPartyHostName);
        System.out.println("           Port:  "+thirdPartyPort);
        System.out.println("       Protocol:  "+thirdPartyProtocol);
        System.out.println("       UserName:  "+thirdPartyUserName);
        System.out.println("       Password:  "+thirdPartyPassword);
        System.out.println("           File:  "+thirdPartyFullFileName);
        System.out.println("      Mime Type:  "+thirdPartyMIMEType);
        System.out.println(" Content Length:  "+thirdPartyContentLength);
        System.out.println("===============================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Resource Items
            //-------------------------------------------------------------
            // Create Resource Items from files in local directory, which
            // will later be updtaed by data stored on a 3rd party source.
            System.out.println("Creating Resource Items Directly From Local Files...");
    
                DKLobICM lob1 = (DKLobICM)dsICM.createDDO("S_lob", DKConstant.DK_CM_ITEM); 
                DKLobICM lob2 = (DKLobICM)dsICM.createDDO("S_lob", DKConstant.DK_CM_ITEM); 

                // Set Mime Type
                lob1.setMimeType("application/msword");
                lob2.setMimeType("application/msword");
                
                // Save to persistent datastore.
                lob1.add("SResourceItemICM_Document1.doc");
                lob2.add("SResourceItemICM_Document1.doc");

            System.out.println("Created Resource Items Directly From Local Files.");

            //-------------------------------------------------------------
            // Check Out / Lock Items for Update
            //-------------------------------------------------------------
            System.out.println("Checking Out / Locking Items...");

                dsICM.checkOut(lob1);
                dsICM.checkOut(lob2);

            System.out.println("Checked Out / Locking Items.");

            //-------------------------------------------------------------
            // Setting the new MIME Type (type of content)
            //-------------------------------------------------------------
            // mimeType: The type of content, such as "image/mpg"
            //
            // For a list of common MIME Types, please refer to the 
            // header documentation and SResourceItemMimeTypesICM.txt.
            System.out.println("Setting New MIME Type / Type of Content...");

                lob1.setMimeType(thirdPartyMIMEType);
                lob2.setMimeType(thirdPartyMIMEType);

            System.out.println("Set New MIME Type / Type of Content.");

            //-------------------------------------------------------------
            // Set 3rd Party Server Info
            //-------------------------------------------------------------
            // When the resource content is to be obtained from a 3rd party,
            // the 3rd party information may be specified as shown below.   
            // Please refer to the Application Programming Reference
            // (javadoc) for more informaiton.
            System.out.println("Setting 3rd Party Server Info...");

                // 3rd Party Info for Lob 1
                DKThirdPartyServerDef thirdParty1 = new DKThirdPartyServerDef();
                thirdParty1.setHostName(thirdPartyHostName);
                thirdParty1.setPort(thirdPartyPort);
                thirdParty1.setProtocol(thirdPartyProtocol);
                thirdParty1.setUserId(thirdPartyUserName);
                thirdParty1.setPassword(thirdPartyPassword);
                thirdParty1.setFileName(thirdPartyFullFileName);
                thirdParty1.setFilePath(thirdPartyFilePath);
                thirdParty1.setInputContentLength(thirdPartyContentLength);

                // 3rd Party Info for Lob 2
                DKThirdPartyServerDef thirdParty2 = new DKThirdPartyServerDef();
                thirdParty2.setHostName(thirdPartyHostName);
                thirdParty2.setPort(thirdPartyPort);
                thirdParty2.setProtocol(thirdPartyProtocol);
                thirdParty2.setUserId(thirdPartyUserName);
                thirdParty2.setPassword(thirdPartyPassword);
                thirdParty2.setFileName(thirdPartyFullFileName);
                thirdParty2.setFilePath(thirdPartyFilePath);
                thirdParty2.setInputContentLength(thirdPartyContentLength);

            System.out.println("Set 3rd Party Server Info.");

            //-------------------------------------------------------------
            // Synchronous Update
            //-------------------------------------------------------------
            // By default, the DKThirdPartyServerDef object is set for
            // syncronous.
            System.out.println("Updating Synchronously...");

                lob1.update(thirdParty1,0);  // Update from the 3rd Party specified above.
                
            System.out.println("Updated Synchronously.");

            //-------------------------------------------------------------
            // Asynchronous Update
            //-------------------------------------------------------------
            // The DKThirdPartyServerDef must be set for Async.
            System.out.println("Updating Asynchronously...");

                // Specify Async. in 3rd Party Info.
                thirdParty2.setAsyncOperation(true); 

                // REQUIREMENT:  In order for Asynchronous operations to complete,
                // the Resource Manager must be able to talk to the Library Server
                // independently, after this operation has completed.  In v8.2
                // this requires that both Library Server be cataloged on the
                // Resource Manager and/or be on the same machine.  If your system
                // meets these conditions, you may uncomment out the line below.
                
                //lob2.update(thirdParty2,0);  // Update from the 3rd Party specified above.

            System.out.println("Updated Asynchronously.");
         
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SResourceItem3rdPartyUpdateICM
		         
	       		    
			
