/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Default Resource Manager & SMS Collection
    The Item Type Definition contains information that specifies the default Resource 
    Manager and default SMS Collection that Items of that Item Type are to be stored
    if not otherwise specified.
    
    Please refer to the corresponding wrapper function at the bottom of this sample, 
    and to the Item Type Creation sample, SItemTypeCreationICM for more information
    and examples of how to set and modify these properties.
    
System SMS Choice
    The System must be told whether to use the default SMS Collection defined for the 
    User or the default SMS Collection defined for the Item Type.  This sample will
    set default SMS Collection codes for Item Types.  If the choice is not set to use
    the Item Type default, the connection to the RM will fail since we are not specifying
    a default SMS Collection for the user.
    
    Please note that SMS "Choice" is not the same as SMS "Code".  
    
    SMS Choice  Description
    ----------  --------------------------------------------------------
        0       Use the default SMS Collection defined for the User.
        1       Use the default SMS Collection defined in the Item Type.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefSetDefaultICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Updating Item Type Definitions for Default SMS Collection specifications.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating the Default SMS Collection for an Item Type
 *                    Updates All Resource Item Types in Samples
 *                    Setting SMS Choice
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SSMSCollectionDefSetDefaultICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SSMSCollectionDefSetDefaultICM RMDB
 *                    ex: java SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    The SMS Collections uses here must already be defined.  Please refer
 *                    to the SSMSCollectionDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SResourceMgrDefRetrievalICM
 *                    SSMSCollectionDefRetrievalICM
 ************************************************************************************************/
public class SSMSCollectionDefSetDefaultICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("   java SSMSCollectionDefSetDefaultICM");
            System.out.println("                         <Existing RM User-Defined Reference Name>");
            System.out.println("                         <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.");
            System.out.println("");
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SSMSCollectionDefSetDefaultICM RMDB");
            System.out.println("  ex: java SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  RM Reference Name Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("===============================================");
        System.out.println("IBM Information Integrator for Content     v"+ver);
        System.out.println("Sample Program:  SSMSCollectionDefSetDefaultICM");
        System.out.println("-----------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("===============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            System.out.println("Obtaining RM Code for RM Definition '"+rmUserDefinedReferenceName+"'...");            
            
                if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw new Exception("Resource Manager Definition Named '"+rmUserDefinedReferenceName+"' Not Defined.");

                short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            System.out.println("Obtained RM Code '"+rmCode+"' for RM Definition '"+rmUserDefinedReferenceName+"'.");            

            //------------------------------------------------------------------
            // Obtaining an SMS Colleciton Code for a SMS Collection Definition
            //------------------------------------------------------------------
            // Obtaining the SMS Collection Code that is used to specify 
            // properties of Items and Item Types.
            System.out.println("Obtaining ID Code for SMS Collection named '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"'...");

                short smsCode = SSMSCollectionDefRetrievalICM.getSMSCollectionCodeForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                System.out.println("Collection '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"' has ID Code '"+rmCode+"'.");
                
            System.out.println("Obtained ID Code for SMS Collection named '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"'.");

            //------------------------------------------------------------------
            // Set Default SMS Collection in an Item Type
            //------------------------------------------------------------------
            // Set the Default SMS Collection for Resource Item Types.
            System.out.println("Setting Default SMS Code for Sample Resource Item Types...");

                // For each Resource Item Type created in SItemTypeCreationICM, each will be
                // updated to use by default the specified SMS Collection.
                setDefaultSMSCollectionForItemType(dsICM, "S_autoClaim", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_adjustrReport", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_policeReport", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_insurePolicy", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_autoPhoto", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_docModel", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_lob", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_text", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_image", smsCode);

            System.out.println("Set Default SMS Code for all Sample Resource Item Types.");

            //-------------------------------------------------------------
            // Set SMS Choice in System
            //-------------------------------------------------------------
            System.out.println("Setting System SMS Choice...");

                setSystemSMSChoice(dsICM,(short) 1);  // '1' means Item Type's default.
            
            System.out.println("Set System SMS Choice.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setDefaultSMSCollectionForItemType
    //******************************************************************
   /**
    * Set the Default SMS Collection in which to store items of this Item Types.  Item Type
    * must already be defined.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that this SMS Collection belongs to.
    * @param rmSMSCollectionUserDefinedReferenceName   The Name of the SMS Collection to use by default.
    **/
    public static void setDefaultSMSCollectionForItemType(DKDatastoreICM dsICM, String itemTypeName, String rmUserDefinedReferenceName, String rmSMSCollectionUserDefinedReferenceName) throws DKException, Exception{
        System.out.println("Setting the default to SMS Collection referred to as '"+rmSMSCollectionUserDefinedReferenceName+"'.");
        
        // Make sure the RM & SMS Collection are setup.
        if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
            throw new Exception("Resource Manager not already setup.  RM must be first setup.");

        short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM,rmUserDefinedReferenceName); // get the RM Code.

        if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM,rmSMSCollectionUserDefinedReferenceName, rmCode)==false) // if it is not defined throw error
            throw new Exception("SMS Collection, '"+rmSMSCollectionUserDefinedReferenceName+"' not setup.");

        short smsCode = SSMSCollectionDefRetrievalICM.getSMSCollectionCodeForRM(dsICM, rmSMSCollectionUserDefinedReferenceName,rmCode);  // Get the SMS Collection ID Code.

        setDefaultSMSCollectionForItemType(dsICM,itemTypeName,smsCode); // set the item type to use this default smsCode.
    }

   /**
    * Set the Default SMS Collection in which to store items of this Item Type.  The Item Type 
    * must already be defined.  For more information on updating Item Type Definitions, please
    * consult SItemTypeUpdateICM Sample.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param smsCode       The ID Code in the SMS collection to use.
    **/
    public static void setDefaultSMSCollectionForItemType(DKDatastoreICM dsICM, String itemTypeName, short smsCode) throws DKException, Exception{
        System.out.println("Setting the default SMS Collection for Item Type Named '"+itemTypeName+"' to SMS Collection with ID Code '"+smsCode+"'.");

            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object

            DKItemTypeDefICM itemTypeDef = (DKItemTypeDefICM) dsDefICM.retrieveEntity(itemTypeName); // retrieve the Item Type Definition
            // If CM Doc Model Item Type, update Relations instead
            if(itemTypeDef.getClassification()==DKConstantICM.DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                System.out.println("Setting Default for Item Type Relations...");
                dkCollection coll = dsDefICM.retrieveItemTypeRelations(itemTypeDef.getItemTypeId());
                dkIterator iter = coll.createIterator();
                while(iter.more()){
                    DKItemTypeRelationDefICM itemTypeRel = (DKItemTypeRelationDefICM) iter.next();
                    itemTypeRel.setDefaultCollCode(smsCode);
                    // Item Type Relations also Require Prefetch Code, which can be set to Default CollCode.
                    itemTypeRel.setDefaultPrefetchCollCode(smsCode);
                    // Both RM & SMS must be set once one is specified.  Detect if RM has not been set, 0,
                    // and set it in that case to '1' as default.
                    if(itemTypeRel.getDefaultRMCode()==0)
                        itemTypeRel.setDefaultRMCode((short)1);       // Set the default Resource Manager for Items of this Item Type to be stored to.

                    itemTypeRel.update();
                }
            } else { // Othwerwise update item type directly
                itemTypeDef.setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Both RM & SMS must be set once one is specified.  Detect if RM has not been set, 0,
                // and set it in that case to '1' as default.
                if(itemTypeDef.getDefaultRMCode()==0)
                    itemTypeDef.setDefaultRMCode((short)1);       // Set the default Resource Manager for Items of this Item Type to be stored to.

                itemTypeDef.update();                       // Update the Item Type Definition.
            }

        System.out.println("ItemType '"+itemTypeName+"' now set to default SMS Code '"+smsCode+"'.");
    }

   /**
    * Set the System SMS Choice, determining which default SMS Collection code it will use, the User's or Item Type's.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param smsChoice     Determines which default SMS Collection code to use, that of the user or that defined in the Item Type.  0 - User, 1 - Item Type.
    **/
    public static void setSystemSMSChoice(DKDatastoreICM dsICM, short smsChoice) throws DKException, Exception{
        if(smsChoice==0)
            System.out.println("Setting System SMS Choice to 'User' (0)...");
        else if(smsChoice==1)
            System.out.println("Setting System SMS Choice to 'Item Type' (1)...");
        else
            System.out.println("Setting System SMS Choice to 'Unknown' ("+smsChoice+")...");

            // Obtain the RM Configuration Management Object
            DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
            DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
            DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
            DKLSConfigurationMgmtICM lsCfgMgmt= (DKLSConfigurationMgmtICM)cfgMgmt.lsConfigurationMgmt();

            // Set the SMS Choice for the System.
            DKLSCfgDefICM configParms = lsCfgMgmt.retrieveConfigParameters();
            configParms.setSMSChoice(smsChoice);
            lsCfgMgmt.defineConfigParameters(configParms);        
            
        System.out.println("Set System SMS Choice to '"+smsChoice+"'.");
    }
   
}//end class SSMSCollectionDefSetDefaultICM
              
