/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

System-Defined Semantic Types
    For information on System-Defined Semantic Types, please refer to the SItemCreationICM
    Sample.
    
Using Semantic Types
    For information on using Semantic Types, please refer to the SItemCreationICM Sample.
    
User-Defined Semantic Types
    User-Defined Semantic Types may be created and used by User Programs.  It is up to the
    User Program to enforce these semantics when used.

---------------------------------------
Semantic Types Defined in Sample
---------------------------------------
    S_myDocument

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSemanticTypeCreationICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining Semantic Types
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Defining Semantic Types
 *                    Creating a New Semantic Type Definition
 *                    Saving / Adding Semantic Type Definition in Datastore Definition
 *                    Semantic Type Name Requirements
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SSemanticTypeCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Semantic Types defined here must not exist.  If any have already been 
 *                    created, please run SSemanticTypeDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SSemanticTypeCreationICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SSemanticTypeCreationICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SSemanticTypeCreationICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Define a Semantic Type
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using Semantic Type names that may already be in use.
            //
            // For more information on options that can be set for Semantic Type
            // definitions, please consult the Programming Reference.
            //
            // Defining Semantic Type Named "S_myDocument".
            System.out.println("Defining Semantic Type 'S_myDocument'...");

                DKSemanticTypeDefICM semTypeDef = new DKSemanticTypeDefICM(dsICM); // Create a new Semantic Type Definition object.
                
                semTypeDef.setName("S_myDocument");                     // Choose a name with a maximum of 15 characters.
                semTypeDef.setDescription("My Document");
                
                semTypeDef.add();                                          // Add it to the persistent datastore.

            System.out.println("Defined Semantic Type 'S_myDocument'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Define a new Semantic Type.
    * For more information on options that can be set for Semantic Type Definitions,
    * please consult the Programming Reference.
    * @param dsICM        Connect DKDatastoreICM object to define the attribute in.
    * @param name         Name of the new attribute.
    * @param description  More detailed description than name.
    * @return Returns a reference to the newly defined DKSemanticTypeDefICM that was added to the datastore.
    **/
    public static DKSemanticTypeDefICM defineSemanticType(DKDatastoreICM dsICM, String name, String description) throws DKException, Exception{
        System.out.println("Defining Semantic Type '"+name+"'...");

            DKSemanticTypeDefICM semTypeDef = new DKSemanticTypeDefICM(dsICM); // Create a new Semantic Type Definition object.
                
            semTypeDef.setName(name);                                          // Choose a name with a maximum of 15 characters.
            semTypeDef.setDescription(description);
                
            semTypeDef.add();                                                  // Add it to the persistent datastore.

        System.out.println("Defined Semantic Type '"+name+"'.");

        return(semTypeDef);
    }


    
}//end class SSemanticTypeCreationICM
              
