/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SResourceItemDeletionICM
    This Sample is an extension of the SResourceItemDeletionICM Sample, providing 
    additional examples using specifically Video Stream Resources.

Deleting Video Resource Items
    Deleting Video Resource Items is exactly the same method as non-resource Item deletion,
    shown in SItemDeletionICM.  Please refer to SItemDeletionICM for complete deletion
    information.  As explained in the Resource Item Creation Sample, XDOs are DDOs.  
    Therefore all functionality of DDOs, including deletion, still applies.
    
    This sample will delete all items created by the Video Resource Item Samples.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SVideoItemDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Video Resource Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Video Resource Items
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SVideoItemDeletionICM  <3rd Party Host Name>
 *                                                <3rd Party Port>
 *                                                <3rd Party Protocol (http|ftp|file)>
 *                                                <3rd Party User Name>
 *                                                <3rd Party Password>
 *                                                <3rd Party Fully-Qualified File Name>
 *                                                <3rd Party File Path>
 *                                                <3rd Party MIME Type>
 *                                                <3rd Party Content Length>
 *                                                <database> <userName> <password>
 *
 *                    NOTE:  No 3rd Pary information is actually required to Delete a video.
 *                           This particular Sample needs the 3rd Party information to first
 *                           create the Video Item before deleting it.
 * 
 *                    NOTE:  Database, UserName, & Password are optional.  If all not specified
 *                           defaults will be used.
 *                           
 *                    Ex: java SVideoItemDeletionICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428
 *                    Ex: java SVideoItemDeletionICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Sample Video Resource Item Type(s) must already be defined.  If not,
 *                    please run the following sample:
 *                        - SVideoItemTypeCreateICM
 *                    The file specified in the commandline parameter must exist on a 3rd Party
 *                    host.  A recommended solution is to setup an FTP server, placing the 
 *                    provided videos in the root directory:
 *                        - SVideoItemICM_Video1.mpg
 *                        - SVideoItemICM_Video2.mpg
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SVideoItemICM_Video1.mpg -> On 3rd Party Host
 *                    AND/OR SVideoItemICM_Video2.mpg -> On 3rd Party Host
 *                    AND/OR Other Video -> On 3rd Party Host
 ************************************************************************************************/
public class SVideoItemDeletionICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // 3rd Party Information
        String thirdPartyHostName       = "NOT_SPECIFIED";    // Please refer to the header documentation for descriptions.
        int    thirdPartyPort           = -1;
        String thirdPartyProtocol       = "NOT_SPECIFIED";
        String thirdPartyUserName       = "NOT_SPECIFIED";
        String thirdPartyPassword       = "NOT_SPECIFIED";
        String thirdPartyFullFileName   = "NOT_SPECIFIED";     
        String thirdPartyFilePath       = "NOT_SPECIFIED";
        String thirdPartyMIMEType       = "NOT_SPECIFIED";
        int    thirdPartyContentLength  = -1;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 12) { // if not all 12 arguments were specified, report usage.
            System.out.println("Usage: ");
            System.out.println("  java SVideoItemDeletionICM  <3rd Party Host Name>");
            System.out.println("                  <3rd Party Port>");
            System.out.println("                  <3rd Party Protocol (http|ftp|file)>");
            System.out.println("                  <3rd Party User Name>");
            System.out.println("                  <3rd Party Password>");
            System.out.println("                  <3rd Party Fully-Qualified File Name>");
            System.out.println("                  <3rd Party File Path>");
            System.out.println("                  <3rd Party MIME Type>");
            System.out.println("                  <3rd Party Content Length>");
            System.out.println("                  <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE:  No 3rd Pary information is actually required to Delete a video.");
            System.out.println("         This particular Sample needs the 3rd Party information to first");
            System.out.println("         create the Video Item before deleting it.");
            System.out.println("");
            System.out.println("  NOTE:  Database, UserName, & Password are optional.  If all not specified");
            System.out.println("         defaults will be used.");
            System.out.println("");
            System.out.println("  Ex: java SVideoItemDeletionICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video1.mpg // video/mpeg 2791428");
            System.out.println("  Ex: java SVideoItemDeletionICM myFtpHostName 21 ftp myUserName myPassword SVideoItemICM_Video2.mpg // video/mpeg 1882116 icmnlsdb icmadmin password");
            System.out.println("");
        }
        if(argv.length < 9){ // If any 3rd Party info not specified, terminate
            System.out.println("===================================================");
            System.out.println(" TERMINATED:  All 3rd Party info must be specified.");
            System.out.println("===================================================");
            return;        
        } else if(argv.length < 12) {
            System.out.println("*** Some parameters not specified.  Using defaults for rest. ***");
        }
        // Set Command-line options
        if (argv.length > 0)  thirdPartyHostName      = argv[0];
        if (argv.length > 1)  thirdPartyPort          = Integer.valueOf(argv[1]).intValue();
        if (argv.length > 2)  thirdPartyProtocol      = argv[2];
        if (argv.length > 3)  thirdPartyUserName      = argv[3];
        if (argv.length > 4)  thirdPartyPassword      = argv[4];
        if (argv.length > 5)  thirdPartyFullFileName  = argv[5];
        if (argv.length > 6)  thirdPartyFilePath      = argv[6];
        if (argv.length > 7)  thirdPartyMIMEType      = argv[7];
        if (argv.length > 8)  thirdPartyContentLength = Integer.valueOf(argv[8]).intValue();
        if (argv.length > 9)  database                = argv[9];
        if (argv.length > 10) userName                = argv[10];
        if (argv.length > 11) password                = argv[11];

        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Content Manager:");
        System.out.println("      Database:  "+database);
        System.out.println("      UserName:  "+userName);
        System.out.println(" 3rd Party:");
        System.out.println("      Host Name:  "+thirdPartyHostName);
        System.out.println("           Port:  "+thirdPartyPort);
        System.out.println("       Protocol:  "+thirdPartyProtocol);
        System.out.println("       UserName:  "+thirdPartyUserName);
        System.out.println("       Password:  "+thirdPartyPassword);
        System.out.println("           File:  "+thirdPartyFullFileName);
        System.out.println("      Mime Type:  "+thirdPartyMIMEType);
        System.out.println(" Content Length:  "+thirdPartyContentLength);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Video Resource Item
            //-------------------------------------------------------------
            // Create a Video Resource Item as shown in SVideoItemCreateICM.
            // For more information, please refer to that Sample.
            System.out.println("Creating Video Resource Item...");
    
                DKVideoStreamICM video = (DKVideoStreamICM)dsICM.createDDO("S_videoType", DKConstant.DK_CM_ITEM); 

                // Set the User-Defined Attributes
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_author"), "IBM"); 
                video.setData(video.dataId(DKConstant.DK_CM_NAMESPACE_ATTR,"S_length"), "48Hours"); 

                // Setting the MIME Type (type of content)
                video.setMimeType(thirdPartyMIMEType);

                // Set 3rd Party Info
                DKThirdPartyServerDef thirdParty = new DKThirdPartyServerDef();
                thirdParty.setHostName(thirdPartyHostName);
                thirdParty.setPort(thirdPartyPort);
                thirdParty.setProtocol(thirdPartyProtocol);
                thirdParty.setUserId(thirdPartyUserName);
                thirdParty.setPassword(thirdPartyPassword);
                thirdParty.setFileName(thirdPartyFullFileName);
                thirdParty.setFilePath(thirdPartyFilePath);
                thirdParty.setInputContentLength(thirdPartyContentLength);

                video.add(thirdParty,0);  // Add from the 3rd Party specified above.

                // Saving Pid String for later retrieval
                String videoPidString = video.getPidObject().pidString();

            System.out.println("Created Video Resource Item.");

            //-------------------------------------------------------------
            // Deleting Resource Items
            //-------------------------------------------------------------
            // Resource Items are treated just as Non-Resource Items are in
            // regards to deletion.
            System.out.println("Deleting Video Resource Item...");

                video.del();
        
            System.out.println("Deleted Video Resource Item.");

            //-------------------------------------------------------------
            // Delete all items created by Video Item Samples
            //-------------------------------------------------------------
            // Deleting all Items created by SVideoItemCreationICM,
            // SVideoItemRetrievalICM, & SVideoItemUpdateICM.
            //
            // Please refer to SItemDeletionICM for more informaiton.
            System.out.println("Deleting All Items Used in Video Item Samples.");

                SItemDeletionICM.deleteAllItems(dsICM,"S_videoType");
            
            System.out.println("Deleted All Items Used in Video Item Samples.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemDeletionICM
              
