/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SItemTypeDeletionICM Sample
    This sample is an extension of the SItemTypeDeletionICM Sample.  Concepts and full
    explanations will not be repeated in this sample.  Please refer to that sample for
    full information on deleting Item Types in general.
    
Deleting Video Item Types
    Deleting Item Types that happen to handle Video Resource Content is no different
    than the rest of the Item Types.  Please refer to the SItemTypeDeletionICM sample
    for complete information and examples.

Delete Rule Affects
    Depending on the product configuration, subEntities / child components / multivalue
    attributes may be required to be deleted first, before the root Item Type can be
    deleted.  If this is the case, please refer to Sample SItemTypeRetrievalICM for 
    learning how to access the Sub-Entities (See Listing Sub-Entitiesin 
    SItemTYpeRetrievalICM).  Once a reference to the entitiy definition is objetained,
    the del() method may be called to deleted it.

This Sample
    This sample will delete the Item Type & Attribute Definitions created in
    SVideoItemTypeCreateICM.

    NOTE:  When retrieving Item Type Definitions, if it does not exist, null is 
           returned.  In this sample, if null is detected, no action will take 
           place for that particular Item Type definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SVideoItemTypeDeleteICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting (Video) Item Type & Attributes created by the Video Item Type
 *                    Creation Sample.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes Item Type(s) created by the SVideoItemTypeCreateICM Sample
 *                    Deletes Attributes created by the SVideoItemTypeCreateICM Sample
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SVideoItemTypeDeleteICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types should be defined.  If not, you may run Sample 
 *                    SVideoItemTypeCreateICM first, otherwise sample will complete, but since
 *                    none of the Item Types will be found, no action will take place.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemTypeDeletionICM.java
 *                    SAttributeDefinitionDeletionICM.java
 ************************************************************************************************/
public class SVideoItemTypeDeleteICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SVideoItemTypeDeleteICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemTypeDeleteICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Delete Item Type(s) Created in SVideoItemTypeCreateICM
            //----------------------------------------------------------------
            SItemTypeDeletionICM.deleteItemTypeDef(dsICM,"S_videoType");

            //----------------------------------------------------------------
            // Delete Attribute(s) Created in SVideoIteTypeCreateICM 
            //----------------------------------------------------------------
            SAttributeDefinitionDeletionICM.deleteAttrDef(dsICM,"S_author");
            SAttributeDefinitionDeletionICM.deleteAttrDef(dsICM,"S_length");
            
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemTypeDeleteICM
              
