
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
// sample connect/execute/disconnect to IP

/*
 * Filename       : TCallbackExecuteIP.java
 * Description    : Connect to IP Datastore.
 *                  Create query string.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes.
 *                  Disconnect from IP Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TCallbackExecuteIP class
public class TCallbackExecuteIP implements DKConstantIP
{


  // Main method
  public static void main(String argv[])
  {

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    DKDatastoreIP dsIP = null;
    try {
      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
         pw = new String(""); // pw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
       System.out.println("Invoke Syntax: " );
       System.out.println("  java TCallbackExecuteIP " +
        "<libSrv> <userid> <pw>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      }

    dsIP = new DKDatastoreIP();
    dkResultSetCursor pCur = null;
    DKNVPair parms[] = null;

    String s = new String( "APPL=01;FAFIP=9.67.43.83;"
                              + "IODMIP=9.67.43.83;FAFPORT=3061;"
                              + "IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;"
                              + "FAFSITE=CS61" );

    System.out.println("connecting to datastore");
    dsIP.connect(libSrv,userid,pw,s);
    System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );
    // prepare the parametric query string
    String cmd = "SEARCH=(ENTITY=DOCUMENT," +
                 "MAX_RESULTS=5," +
                 "COND=(FolderId == \"IPSAMP\") );" +
                 "OPTION=(CONTENT=YES)";

    TCallbackIP cbkIP = new TCallbackIP();
    InputStreamReader isreader = new InputStreamReader(System.in);

    System.out.println("query string " + cmd);
    System.out.println("executing query");
    Date date;
    date  = new Date();
    System.out.println("before query time " + date.toString());
    dsIP.executeWithCallback(cmd,DK_CM_PARAMETRIC_QL_TYPE,parms,(dkCallback)cbkIP);
    System.out.println("datastore executed query");

    boolean isDone = false;
    DKException ckExc = null;
    while (isDone == false)
    {
     System.out.println("press <enter> to continue...");
     isreader.read();
     isDone = cbkIP.isDone();
    }

    ckExc = cbkIP.getException();
    if (ckExc != null)
    {
     System.out.println("Exception name " + ckExc.name());
     System.out.println("Exception message " + ckExc.getMessage());
     System.out.println("Exception error code " + ckExc.errorCode());
     System.out.println("Exception error state " + ckExc.errorState());
     ckExc.printStackTrace();
     cbkIP.resetException(); // reset since it is not needed any more
    }    

    System.out.println("query results processed");
    dsIP.disconnect();
    System.out.println("datastore disconnected");
    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}
