/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TAddDDODemoV4 implements DKConstantV4
{
  // Main method
  public static void main(String argv[])
  {
    System.out.println("begin...");
   
    //*** for input parameters
    String libSrv, userid, pw;
    
    // Checking for input parameters
    if (argv.length < 3) 
    {
       System.out.println("Usage: " );
       System.out.println("  java TAddDDODemoV4 " + "<LibServer> <userid> <password>" );
       return;
    }
    else 
    {
       libSrv = argv[0];
       userid = argv[1];
       pw     = argv[2];
    } 

    String indexclass = "V4DEMO";
    System.out.println("index class = " + indexclass );
    DKDatastoreV4 dsV4 = null;

    try 
    {         
       dsV4 = new DKDatastoreV4(); 
       System.out.println("Connecting to datastore V4 : " + libSrv);
       dsV4.connect(libSrv, userid, pw, "");
       System.out.println(" datastore connected" );
                   
       DKPid pid = new DKPid();
       pid.setObjectType(indexclass);
       
       DKDDO ddo = new DKDDO(dsV4, pid);
       
       // set the item type to be a document
       ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, new Short(DK_CM_DOCUMENT));
       
       short dataId;
       String[] attrName = {
                            "V4USERID",
                            "V4TITLE",
                            "V4DATE"
                           };
       Object[] attrValue = new Object[3];
       attrValue[0] = new String("EDTEST01");
       attrValue[1] = new String("Mgr");
       attrValue[2] = new String("1996");
       
       short[] attr_prop_type = {
                                 DK_CM_VSTRING,
                                 DK_CM_VSTRING,
                                 DK_CM_VSTRING
                                };

       short attrCount = (short) attrName.length;
       
       int i;
       for (i=0; i < attrCount; i++)
       {
          dataId = ddo.addData(attrName[i]);
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, 
                              new Short(attr_prop_type[i])); // mandatory
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, 
                              new Integer(DK_CM_TRUE));     // optional
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT, 
                              new Integer(DK_CM_ATTRIBUTE_READWRITE)); // optional
          ddo.setData(dataId, attrValue[i]);

          // display the created attribute
          //System.out.print(" dataId = " + dataId );
          //System.out.print(", Attr type = " + ddo.getDataPropertyByName(dataId, DK_CM_PROPERTY_TYPE) );
          //System.out.print(", name(" + i + ") = " + ddo.getDataName(dataId) );
          //System.out.println(",    \t value(" + i + ") = " + ddo.getData(dataId) );
       }
       
       System.out.println("\n *** displaying the ddo before adding it ***\n ");
       displayDDO(ddo);
      
       // add the document DDO to the V4 persistent store
       System.out.println("\n *** adding the ddo *** \n" );

       ddo.add();

       pid = ddo.getPidObject();
       System.out.println(" the itemId of the newly added ddo = " + 
                          pid.getPrimaryId() ); 

       // retrieve it back to see it is really the same one
       System.out.println(" retrieve the newly added ddo, with itemId = " + 
                          pid.getPrimaryId() );     
       
       // create a new DDO for retrieving 
       DKDDO nddo = new DKDDO(dsV4, pid);
       try
       {
          nddo.retrieve();
       }
       catch (DKException e)
       {
         System.out.println(" item does not exist " + e.toString() );
         return;
       }
       catch (Exception exc)
       {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
         return;
       }  
    
       System.out.println(" displaying the retrieved ddo" );
       displayDDO(nddo);

       // disconnect from V4 datastore
       dsV4.disconnect();
       System.out.println("datastore disconnected");
    }
    catch(DKException exc)
    {
       System.out.println("Exception name:        " + exc.name());
       System.out.println("Exception message:     " + exc.getMessage());
       System.out.println("Exception error code:  " + exc.errorCode());
       System.out.println("Exception error state: " + exc.errorState());
       exc.printStackTrace();
    }   
    catch (Exception exc)
    {
     System.out.println("Exception message:       " + exc.getMessage());
     exc.printStackTrace();
    }

    System.out.println(" done" );
  }

  public static void displayDDO(DKDDO ddo)
  {
    short i, j;
    String dataName;

    try 
    {
      System.out.println(" data count = " + ddo.dataCount());
      short itemType = 
         ((Short)ddo.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE)).shortValue();
         
      if (itemType == DK_CM_FOLDER)
          System.out.println(" this item is a Folder, itemId = " + 
                             ddo.getPidObject().getPrimaryId());
      else if (itemType == DK_CM_DOCUMENT)
          System.out.println(" this item is a Document, itemId = " + 
                             ddo.getPidObject().getPrimaryId());
      else 
          System.out.println(" this item is of unknown type, type = " + 
                             itemType + ", itemId = " + 
                             ddo.getPidObject().getPrimaryId() );
                             
      // print the properties of this DDO
      System.out.println(" properties of this DDO: " + ddo.propertyCount());
      for (i = 1; i <= ddo.propertyCount(); i++) 
      {
           System.out.println("   property_name(" + i + ") = " + 
                                  ddo.getPropertyName(i) + ", value = " +
                                  ddo.getProperty(i));
      }

      // print out the content
      Object value;
      DKParts  parts;
      DKFolder folder;
      DKBlobV4 blob;
      dkIterator iter;
      
      for ( i=1; i <= ddo.dataCount(); i++) 
      {
           dataName = ddo.getDataName(i);
           System.out.print(" data item name (" + i + ") = " + dataName + " ");
           value = ddo.getData(i);
           
           if (value != null) 
             System.out.println("  \t value (" + i + ") = " + value);
           else 
             System.out.println("  \t value (" + i + ") = null" );
             
           System.out.println(" properties of this data item: " + 
                              ddo.dataPropertyCount(i));
                              
           for ( j=1; j <= ddo.dataPropertyCount(i); j++) 
           {
               System.out.println("   property_name(" + i + "," + j + ") = " + 
                                      ddo.getDataPropertyName(i,j) + 
                                      ", value = " +
                                      ddo.getDataProperty(i,j));
           }
           
           if (dataName.equals(DKFOLDER)) 
           {
             if (value != null) 
             {
               folder = (DKFolder) value;
               iter = folder.createIterator();
               DKDDO mddo;
               
               while(iter.more()) 
               {
                  mddo = (DKDDO) iter.next();
                  mddo.retrieve();               // retrieve a folder member
                  displayDDO(mddo);
               }         
             }
           }
           
           if (dataName.equals(DKPARTS)) 
           {
              if (value != null) 
              {
                parts = (DKParts) value;
                iter  = parts.createIterator();
                
                while(iter.more()) 
                {
                  blob = (DKBlobV4) iter.next();   
                  System.out.println(" \t\t blob part id " +
                       ((DKPidXDOV4) (blob.getPidObject())).getPartId() +
                       " content class = " +
                       blob.getContentClass() +
                       " rep type " +
                       blob.getRepType() +
                       " aff type " +
                       blob.getAffiliatedType() );
                  blob.retrieve();                // retrieve a part
                }
              }
           }
      }
    }
    catch (DKException exc)
    {
       System.out.println(" Exception name:       " + exc.name());
       System.out.println(" Exception message:    " + exc.getMessage());
       System.out.println("Exception error code:  " + exc.errorCode());
       System.out.println("Exception error state: " + exc.errorState());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
       System.out.println("Exception message: " + exc.getMessage());
       exc.printStackTrace();
    }
  }
}
